#include <windows.h>
#include <commctrl.h>
#include <strsafe.h>
#include "app.h"

/////////////////////////////////////////////////////////////////////
//
//	class CRegistry - Register database
//
//

int CRegistry::OpenForRead(HKEY root, char *subkey)
{
	return RegOpenKeyEx(root,subkey,0,KEY_QUERY_VALUE,&m_handle);
}

int CRegistry::OpenForWrite(HKEY root, char *subkey)
{
	DWORD Disposition;

	return RegCreateKeyEx(root, subkey, 0, NULL,
		REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &m_handle, &Disposition);
}

void CRegistry::GetInteger(char *name,int *val)
{
	DWORD type,size;

	type=REG_DWORD; size=4;
	RegQueryValueEx(m_handle,name,NULL,&type,(LPBYTE)val,&size);
}

void CRegistry::GetLimitedInteger(char *name,int *val, int min, int max)
{
	GetInteger(name,val);
	limiter(*val,min,max);
}

void CRegistry::SetInteger(char *name,int val)
{
	RegSetValueEx(m_handle,name,0,REG_DWORD,(CONST BYTE *)&val,4);
}

void CRegistry::GetDouble(char *name,double *val)
{
	DWORD type,size;

	type=REG_BINARY; size=8;
	RegQueryValueEx(m_handle,name,NULL,&type,(LPBYTE)val,&size);
}

void CRegistry::GetLimitedDouble(char *name,double *val, double min, double max)
{
	GetDouble(name,val);
	limiter(*val,min,max);
}

void CRegistry::SetDouble(char *name,double val)
{
	RegSetValueEx(m_handle,name,0,REG_BINARY,(CONST BYTE *)&val,8);
}

void CRegistry::GetText(char *name, char *val, DWORD size)
{
	DWORD type;

	type=REG_SZ;
	RegQueryValueEx(m_handle,name,NULL,&type,(LPBYTE)val,&size);
}

void CRegistry::SetText(char *name,char *val)
{
	RegSetValueEx(m_handle,name,0,REG_SZ,(CONST BYTE *)val,strlen(val)+1);
}

/////////////////////////////////////////////////////////////////////
//
// class CDialog - Dialog window management
//
//

BOOL CDialog::GetInteger(int id, int *val)
{
	BOOL dobrze;
	int w;

	w=GetDlgItemInt(m_handle,id,&dobrze,TRUE);
	if (dobrze)
		*val=w;
	else
		SetFocus(id);
	return dobrze;
}

BOOL CDialog::GetLimitedInteger(int nID, int *nVal, int nMin, int nMax)
{
	if (!GetInteger(nID, nVal))
		return FALSE;
	if (*nVal < nMin || *nVal > nMax)
	{
		SetFocus(nID);
		return FALSE;
	}
	return TRUE;
}

void CDialog::SetDouble(int nID, double fVal)
{
	char napis[80];

	_gcvt(fVal,10,napis);
	SetText(nID,napis);
}

BOOL CDialog::GetDouble(int nID, double *fVal)
{
	char *dobrze;
	double w;
	char napis[80];

	if (GetText(nID,napis,sizeof napis))
	{
		w=strtod(napis,&dobrze);
		if (*dobrze)
			SetFocus(nID);
		else
			*fVal=w;
	}
	else
		return FALSE;
	return (*dobrze == 0);
}

BOOL CDialog::GetLimitedDouble(int nID, double *fVal, double fMin, double fMax)
{
	if (!GetDouble(nID, fVal))
		return FALSE;
	if (*fVal < fMin || *fVal > fMax)
	{
		SetFocus(nID);
		return FALSE;
	}
	return TRUE;
}

BOOL CDialog::GetRadio(int nID, int nRange, int *nVal)
{
	int i;

	for (i=0; i<nRange; i++)
		if (SendMessage(nID+i,BM_GETCHECK,0,0)==BST_CHECKED)
		{
			*nVal=i;
			return TRUE;
		}
	return FALSE;
}

void CDialog::GetSelectedItem(int nID, int *nVal)
{
	int stan;
	int i,items;

	items=SendMessage(nID,LVM_GETITEMCOUNT,0,0);
	for (i=0; i<items; i++)
	{
		stan=SendMessage(nID,LVM_GETITEMSTATE,i,LVIS_SELECTED);
		if (stan==LVIS_SELECTED)
		{
			*nVal=i;
			return;
		}
	}
	*nVal=-1;
	return;
}

////////////////////////////////////////////////
//
// Dialog Window Procedure
//
//		common to all dialog windows
//

BOOL CALLBACK DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	CDialog *lpDialog;

	DebugOut("e1\n");
	switch (message)
	{
		case WM_INITDIALOG :
			DebugOut("e2\n");
			SetWindowLong(hDlg,DWL_USER,lParam);
			return ((CDialog *)lParam)->OnInitDialog(hDlg);

		case WM_NOTIFY:
			DebugOut("e3\n");
			lpDialog=(CDialog *)GetWindowLong(hDlg,DWL_USER);
			return lpDialog->OnNotify(wParam, (NMHDR *)lParam);

		case WM_COMMAND :
			DebugOut("e4\n");
			lpDialog=(CDialog *)GetWindowLong(hDlg,DWL_USER);
			if (lpDialog)
				switch (LOWORD(wParam))
				{
					case IDOK:
						return lpDialog->OnOk();

					case IDCANCEL:
						return lpDialog->OnCancel();

					default:
						return lpDialog->OnCommand(LOWORD(wParam),HIWORD(wParam),(HWND)lParam);
				}
			break;
	}
	DebugOut("e5\n");
	return FALSE ;
}

////////////////////////////////////////////////////////////////////////
//
//	class CClientWnd - client window in MDI aplication
//
//

int CMDIChildWnd::Create(int x, int y, int w, int h, HWND parent, DWORD style)
{
	CreateMDIWindow(
		"MiwilMDIChildClass",
		"MDI Child",
		style,
		x, y, w, h,
		parent,
		(HINSTANCE)GetWindowLong(parent,GWL_HINSTANCE),
		(LPARAM)this
		);
	if (!m_handle)
		return 1;
	return 0;
}

////////////////////////////////////////////////////////////////////////
LRESULT FAR PASCAL MDIChildWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	CMDIChildWnd *wnd;

	wnd = (CMDIChildWnd *)GetWindowLong(hWnd,0);	// tylko w WM_NCCREATE bdzie bdna warto
	switch (uMsg)
	{
	case WM_NCCREATE:
		wnd = (CMDIChildWnd *)((MDICREATESTRUCT *)((CREATESTRUCT *)lParam)->lpCreateParams)->lParam;
		wnd->SetWindowHandle(hWnd);
		return DefMDIChildProc(hWnd, uMsg, wParam, lParam);

	case WM_CREATE:
		return wnd->OnCreate((CREATESTRUCT *)lParam);

	case WM_MDIACTIVATE:
		return wnd->OnMDIActivate((HWND)wParam,(HWND)lParam);

	case WM_SIZE:
		wnd->OnSize(lParam, LOWORD(lParam),HIWORD(lParam));
		return DefMDIChildProc(hWnd, uMsg, wParam, lParam);

	case WM_COMMAND:
		return wnd->OnCommand(wParam,lParam);

	case WM_TIMER:
		wnd->OnTimer(wParam);
		break;

	default:
		if (uMsg>WM_USER)
			return wnd->OnUser(uMsg, wParam, lParam);
		else
			return DefMDIChildProc(hWnd, uMsg, wParam, lParam);
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////
LRESULT FAR PASCAL WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HDC hDC;
	PAINTSTRUCT ps;
	CWnd *wnd;

	wnd = (CWnd *)GetWindowLong(hWnd,0);	// tylko w WM_NCCREATE bdzie bdna warto
	switch (uMsg)
	{
	case WM_NCCREATE:
		wnd = (CWnd *)((CREATESTRUCT *)lParam)->lpCreateParams;
		wnd->SetWindowHandle(hWnd);
		return DefWindowProc(hWnd, uMsg, wParam, lParam);

	case WM_CREATE:
		return wnd->OnCreate((CREATESTRUCT *)lParam);

	case WM_CLOSE:
		if (wnd->OnClose())
			DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		wnd->OnDestroy();
		break;

	case WM_SIZE:
		wnd->OnSize(wParam, LOWORD(lParam),HIWORD(lParam));
		break;
		//return DefWindowProc(hWnd, uMsg, wParam, lParam);

	case WM_PAINT:
		hDC = BeginPaint (hWnd, &ps);
		wnd->OnPaint(hDC);
		EndPaint (hWnd, &ps);
		return 0;

	case WM_COMMAND:
		return wnd->OnCommand(wParam,lParam);

	case WM_VSCROLL:
		wnd->OnVScroll( LOWORD(wParam), ((int)wParam)>>16, (HWND)lParam );
		break;

	case WM_HSCROLL:
		wnd->OnHScroll( LOWORD(wParam), ((int)wParam)>>16, (HWND)lParam );
		break;

	case WM_TIMER:
		wnd->OnTimer(wParam);
		break;

	case WM_CHAR:
		return wnd->OnChar((TCHAR)wParam,lParam);

	case WM_KEYDOWN:
		return wnd->OnKeyDown(wParam,lParam);

	case WM_KEYUP:
		return wnd->OnKeyUp(wParam,lParam);

	case WM_SETFOCUS:
		wnd->OnSetFocus();
		break;

	case WM_KILLFOCUS:
		wnd->OnKillFocus();
		break;

	case WM_MOUSEMOVE:
		wnd->OnMouseMove(wParam, (short int)lParam, ((int)lParam)>>16);
		break;

	case WM_LBUTTONDOWN:
		wnd->OnLButtonDown(wParam, (short int)lParam, ((int)lParam)>>16);
		break;

	case WM_LBUTTONUP:
		wnd->OnLButtonUp(wParam, (short int)lParam, ((int)lParam)>>16);
		break;

	case WM_RBUTTONDOWN:
		wnd->OnRButtonDown(wParam, (short int)lParam, ((int)lParam)>>16);
		break;

	case WM_RBUTTONUP:
		wnd->OnRButtonUp(wParam, (short int)lParam, ((int)lParam)>>16);
		break;

	default:
		if (uMsg>WM_USER)
			return wnd->OnUser(uMsg, wParam, lParam);
		else
			return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}

	return 0;
}

