#include <windows.h>              /* required for all Windows applications */
#include <commctrl.h>
#include <strsafe.h>
#include <math.h>

#include "matrix.h"
#include "app.h"
#include "winfi32.h"
#include "resource.h"
#include "measure.h"
#include "calc.h"
#include "aprox.h"


#define APROXDOC(a) ((CAprox *)a)
#define APROXDATA(a) ((CAproxData *)a)

//double epsilon = 1.0e-7;
//double break_cond = 1.0e-3;

/////////////////////////////////////////////////////////////////////////////
// CAprox

CAprox::CAprox(CMeasure *parent, CSelection &selection, CParametryAproksymacji &parametry_aproksymacji)
    : CCalculation(parent,selection)
{
	m_parametry_aproksymacji=parametry_aproksymacji;
	m_info=new CAproxData();
}

#define VECTOR_SIZE ((MAX_OMEGA<<2)+1)
#define MATRIX_SIZE (VECTOR_SIZE*VECTOR_SIZE)

DWORD WINAPI AproxThreadProc(CAproxData *info)
{
	int t,i,j,ic,is,jc,js,iw,jw,ie,je;
	int M,iter;
	double wynikX,wynikY;
	double olde,x,y,del,dwi,dwj,dai,daj,dbi,dbj,dei,dej;
	double daa,dab,daw,dbb,dbw,dww,dae,dbe,dwe,dee;
	double delww,delwe,delee;
	double e,f,g;

	double s[MAX_OMEGA], c[MAX_OMEGA];
	double sumy[MATRIX_SIZE];
	double wolne[VECTOR_SIZE];
	double abwe[VECTOR_SIZE];
	double ObsErr;

	int ncol,N,skok;
	double dco[MAX_OMEGA];
	double dsi[MAX_OMEGA];
	float *z;
	float *ft;
	double dt;//,old_sigma;

	float *probki = info->m_probki + (info->m_channel + info->m_begin*info->m_channels)*2;
	float *wyniki=info->m_wyniki;

	N = info->m_samples;
	skok = 2*info->m_channels;
	info->m_break=0;
	info->m_calculated=0;
	M=info->m_components;
	dt=1/info->m_rate;
	olde=0.0;

	if (!info->m_use_AD)
	{
		for (i=0; i<M; i++)
		{
			e=info->m_fr[i]*2*M_PI*dt;
			dco[i]=cos(e);
			dsi[i]=sin(e);
			c[i]=1.0;
			s[i]=0.0;
			if ( info->m_use_eta )
			{
				e = exp(-(info->m_eta[i])*dt);
				dco[i] *= e;
				dsi[i] *= e;
			}
		}

		ncol = (M<<1) + 1;

		clear_vector(ncol,wolne);
		clear_matrix(ncol,sumy);

		z = probki;
		for (t=0; t<N; t++)
		{
			y = *z;
			ic=0; is=1;
			for (i=0; i<M; i++)
			{
				wolne[ic] += y*c[i];
				wolne[is] += y*s[i];
				sumy[ic*ncol+ic] += (double)(c[i] * c[i]);
				sumy[ic*ncol+is] += (double)(s[i] * c[i]);
				sumy[is*ncol+is] += (double)(s[i] * s[i]);
				jc=ic+2; js=is+2;
				for (j=i+1; j<M; j++)
				{
					sumy[ic*ncol+jc] += (double)(c[j]*c[i]);
					sumy[ic*ncol+js] += (double)(s[j]*c[i]);
					sumy[is*ncol+jc] += (double)(c[j]*s[i]);
					sumy[is*ncol+js] += (double)(s[j]*s[i]);
					jc += 2;  js += 2;
				}
				sumy[ic*ncol+jc] += (double)(c[i]);
				sumy[is*ncol+jc] += (double)(s[i]);
				ic+=2;  is+=2;
			}
			wolne[ic] += y;
			sumy[ic*ncol+ic] += 1;

			for (i=0; i<M; i++)
			{
				g=c[i]; f=s[i];
				c[i]=g*dco[i]-f*dsi[i];
				s[i]=g*dsi[i]+f*dco[i];
			}
			z += skok;
		}

		make_simetric(ncol,sumy);
		Eliminacja(ncol,sumy,wolne);

		e=0.0;
		for (i=0; i<M; i++)
		{
			c[i]=1.0;
			s[i]=0.0;
		}
		z = probki;
		ft = wyniki;

		for (t=N; t--;)
		{
			y = *z;
			for (i=0,j=0; i<M; i++,j+=2)
			{
				wynikX = wolne[j]*c[i]+wolne[j+1]*s[i];
				wynikY = -wolne[j]*s[i]+wolne[j+1]*c[i];
				y -= wynikX;
				*(ft++)=(float)wynikX;
				*(ft++)=(float)wynikY;
				g=c[i]; f=s[i];
				c[i]=g*dco[i]-f*dsi[i];
				s[i]=g*dsi[i]+f*dco[i];
			}
			y -= wolne[ncol-1];
			*(ft++)=(float)y;
			*(ft++)=(float)0;
			e += y*y;
			z += skok;
			++(info->m_calculated);
		}
		ObsErr = sqrt((double)e/N);

		j=0;
		for (i=0; i<M; i++)
		{
			info->m_A[i] = wolne[j++];
			info->m_D[i] = wolne[j++];
		}
		info->m_A0 = (float)wolne[j];
	}





	if ( !info->m_use_eta )
	{
		ncol=0;
		for (i=0; i<M; i++)
		{
			abwe[ncol++] = info->m_A[i];
			abwe[ncol++] = info->m_D[i];
			abwe[ncol++] = info->m_fr[i]*2*M_PI*dt;
		}
		abwe[ncol++] = info->m_A0;

		j=2;
		for (i=0;i<M;i++)
		{
			dco[i]=cos(abwe[j]);
			dsi[i]=sin(abwe[j]);
			j+=3;
		}

		for (iter=1; iter<50 && !info->m_break; iter++)
		{

			clear_vector(ncol,wolne);
			clear_matrix(ncol,sumy);
			for (i=0; i<M; i++)
			{
				c[i]=1.0;
				s[i]=0.0;
			}

			z = probki;
			x = 0;
			for (t=0; t<N; t++)
			{
				del = - *z;
				for (i=0; i<M; i++)
					del += abwe[3*i+0]*c[i] + abwe[3*i+1]*s[i];
				del += abwe[ncol-1];
				ic=0; is=1; iw=2;
				for (i=0; i<M; i++)
				{
					dai=(double)(c[i]);
					dbi=(double)(s[i]);
					dwi=-x*(abwe[ic]*dbi-abwe[is]*dai);

					wolne[ic] += dai*del;
					wolne[is] += dbi*del;
					wolne[iw] += dwi*del;

					delww = -x*x*(abwe[ic]*dai+abwe[is]*dbi);
					daa = dai*dai;   dab = dai*dbi;    daw = -x*dbi*del+dai*dwi;
					dbb = dbi*dbi;   dbw = x*dai*del+dbi*dwi;
					dww = delww*del+dwi*dwi;

					sumy[ic*ncol+ic] += daa;
					sumy[ic*ncol+is] += dab;
					sumy[ic*ncol+iw] += daw;
					sumy[is*ncol+is] += dbb;
					sumy[is*ncol+iw] += dbw;
					sumy[iw*ncol+iw] += dww;

					jc=ic+3; js=is+3; jw=iw+3;
					for (j=i+1; j<M; j++)
					{
						daj = (double)(c[j]);
						dbj = (double)(s[j]);
						dwj = -x*(abwe[jc]*dbj-abwe[js]*daj);

						sumy[ic*ncol+jc] += dai*daj;
						sumy[ic*ncol+js] += dai*dbj;
						sumy[ic*ncol+jw] += dai*dwj;
						sumy[is*ncol+jc] += dbi*daj;
						sumy[is*ncol+js] += dbi*dbj;
						sumy[is*ncol+jw] += dbi*dwj;
						sumy[iw*ncol+jc] += dwi*daj;
						sumy[iw*ncol+js] += dwi*dbj;
						sumy[iw*ncol+jw] += dwi*dwj;
						jc += 3;  js += 3;  jw += 3;
					}
					sumy[ic*ncol+jc] += dai;
					sumy[is*ncol+jc] += dbi;
					sumy[iw*ncol+jc] += dwi;
					ic+=3;  is+=3;  iw+=3;
				}
				wolne[ic] += del;
				sumy[ic*ncol+ic] += 1;

				for (i=0; i<M; i++)
				{
					g=c[i]; f=s[i];
					c[i]=g*dco[i]-f*dsi[i];
					s[i]=g*dsi[i]+f*dco[i];
				}
				z += skok;
				x++;
			}

			make_simetric(ncol,sumy);
			Eliminacja(ncol,sumy,wolne);
			vector_sub_vector(ncol,abwe,wolne);

			j=2;
			for (i=0;i<M;i++)
			{
				dco[i]=cos(abwe[j]);
				dsi[i]=sin(abwe[j]);
				c[i]=1.0;
				s[i]=0.0;
				j+=3;
			}
			e=0.0;
			z = probki;
			ft = wyniki;
			info->m_calculated=0;

			for (t=N; t--;)
			{
				y = *z;
				for (i=0,j=0; i<M; i++,j+=3)
				{
					wynikX = abwe[j]*c[i]+abwe[j+1]*s[i];
					wynikY = -abwe[j]*s[i]+abwe[j+1]*c[i];
					y -= wynikX;
					*(ft++)=(float)wynikX;
					*(ft++)=(float)wynikY;
					g=c[i]; f=s[i];
					c[i]=g*dco[i]-f*dsi[i];
					s[i]=g*dsi[i]+f*dco[i];
				}
				y -= abwe[ncol-1];
				*(ft++)=(float)y;
				*(ft++)=(float)0;
				e += y*y;
				z += skok;
				++(info->m_calculated);
			}
			ObsErr = sqrt((double)e/N);

			if (iter>1)
			{
				if ( ObsErr-olde>info->m_stop_cond )
					break;         //blad rosnie wiec sie rozbiega
			}
			olde=ObsErr;

			f=0.0;
			for (i=0; i<ncol; i++)
				if ( f<fabs(wolne[i]) ) f=fabs(wolne[i]);
			if (f<info->m_epsilon) break;                //najwiekszy przyrost < 1e-8
		}

		j=0;
		for (i=0; i<M; i++)
		{
			info->m_A[i] = abwe[j++];
			info->m_D[i] = abwe[j++];
			info->m_fr[i] = abwe[j++]/dt/2/M_PI;
		}
		info->m_A0 = abwe[j];
	}





	else





	{
		ncol=0;
		for (i=0; i<M; i++)
		{
			abwe[ncol++] = info->m_A[i];
			abwe[ncol++] = info->m_D[i];
			abwe[ncol++] = info->m_fr[i]*2*M_PI*dt;
			abwe[ncol++] = info->m_eta[i]*dt;
		}
		abwe[ncol++] = info->m_A0;

		j=2;
		for (i=0;i<M;i++)
		{
			e = exp(-abwe[j+1]);
			dco[i]=cos(abwe[j])*e;
			dsi[i]=sin(abwe[j])*e;
			j+=4;
		}

		for (iter=1; iter<50 && !info->m_break; iter++)
		{

			clear_vector(ncol,wolne);
			clear_matrix(ncol,sumy);
			for (i=0; i<M; i++)
			{
				c[i]=1.0;
				s[i]=0.0;
			}

			z = probki;
			x = 0;
			for (t=0; t<N; t++)
			{
				del = - *z;
				for (i=0; i<M; i++)
					del += abwe[(i<<2)+0]*c[i] + abwe[(i<<2)+1]*s[i];
				del += abwe[ncol-1];
				ic=0; is=1; iw=2; ie=3;
				for (i=0; i<M; i++)
				{
					dai=(double)(c[i]);
					dbi=(double)(s[i]);
					dwi=-x*(abwe[ic]*dbi-abwe[is]*dai);
					dei=-x*(abwe[ic]*dai+abwe[is]*dbi);

					wolne[ic] += dai*del;
					wolne[is] += dbi*del;
					wolne[iw] += dwi*del;
					wolne[ie] += dei*del;

					delww = x*dei;   delwe=-x*dwi;   delee=-x*dei;
					daa = dai*dai;   dab = dai*dbi;    daw = -x*dbi*del+dai*dwi; dae=-dai*(x*del-dei);
					dbb = dbi*dbi;   dbw = x*dai*del+dbi*dwi;  dbe=-dbi*(x*del-dei);
					dww = delww*del+dwi*dwi;  dwe=delwe*del+dwi*dei;
					dee = delee*del+dei*dei;

					sumy[ic*ncol+ic] += daa;
					sumy[ic*ncol+is] += dab;
					sumy[ic*ncol+iw] += daw;
					sumy[ic*ncol+ie] += dae;
					sumy[is*ncol+is] += dbb;
					sumy[is*ncol+iw] += dbw;
					sumy[is*ncol+ie] += dbe;
					sumy[iw*ncol+iw] += dww;
					sumy[iw*ncol+ie] += dwe;
					sumy[ie*ncol+ie] += dee;

					jc=ic+4; js=is+4; jw=iw+4; je=ie+4;
					for (j=i+1; j<M; j++)
					{
						daj = (double)(c[j]);
						dbj = (double)(s[j]);
						dwj = -x*(abwe[jc]*dbj-abwe[js]*daj);
						dej = -x*(abwe[jc]*daj+abwe[js]*dbj);

						sumy[ic*ncol+jc] += dai*daj;
						sumy[ic*ncol+js] += dai*dbj;
						sumy[ic*ncol+jw] += dai*dwj;
						sumy[ic*ncol+jw] += dai*dej;
						sumy[is*ncol+jc] += dbi*daj;
						sumy[is*ncol+js] += dbi*dbj;
						sumy[is*ncol+jw] += dbi*dwj;
						sumy[is*ncol+je] += dbi*dej;
						sumy[iw*ncol+jc] += dwi*daj;
						sumy[iw*ncol+js] += dwi*dbj;
						sumy[iw*ncol+jw] += dwi*dwj;
						sumy[iw*ncol+je] += dwi*dej;
						sumy[ie*ncol+jc] += dei*daj;
						sumy[ie*ncol+js] += dei*dbj;
						sumy[ie*ncol+jw] += dei*dwj;
						sumy[ie*ncol+je] += dei*dej;
						jc += 4;  js += 4;  jw += 4;  je += 4;
					}
					sumy[ic*ncol+jc] += dai;
					sumy[is*ncol+jc] += dbi;
					sumy[iw*ncol+jc] += dwi;
					sumy[ie*ncol+jc] += dei;
					ic+=4;  is+=4;  iw+=4;  ie+=4;
				}
				wolne[ic] += del;
				sumy[ic*ncol+ic] += 1;

				for (i=0; i<M; i++)
				{
					g=c[i]; f=s[i];
					c[i]=g*dco[i]-f*dsi[i];
					s[i]=g*dsi[i]+f*dco[i];
				}
				z += skok;
				x++;
			}

			make_simetric(ncol,sumy);
			Eliminacja(ncol,sumy,wolne);
			vector_sub_vector(ncol,abwe,wolne);

			j=2;
			for (i=0;i<M;i++)
			{
				e = exp(-abwe[j+1]);
				dco[i]=cos(abwe[j])*e;
				dsi[i]=sin(abwe[j])*e;
				c[i]=1.0;
				s[i]=0.0;
				j+=4;
			}
			e=0.0;
			z = probki;
			ft = wyniki;
			info->m_calculated=0;

			for (t=N; t--;)
			{
				y = *z;
				for (i=0,j=0; i<M; i++,j+=4)
				{
					wynikX = abwe[j]*c[i]+abwe[j+1]*s[i];
					wynikY = -abwe[j]*s[i]+abwe[j+1]*c[i];
					y -= wynikX;
					*(ft++)=(float)wynikX;
					*(ft++)=(float)wynikY;
					g=c[i]; f=s[i];
					c[i]=g*dco[i]-f*dsi[i];
					s[i]=g*dsi[i]+f*dco[i];
				}
				y -= abwe[ncol-1];
				*(ft++)=(float)y;
				*(ft++)=(float)0;
				e += y*y;
				z += skok;
				++(info->m_calculated);
			}
			ObsErr = sqrt((double)e/N);

			if (iter>1)
			{
				if ( ObsErr-olde>info->m_stop_cond )
					break;         //blad rosnie wiec sie rozbiega
			}
			olde=ObsErr;

			f=0.0;
			for (i=0; i<ncol; i++)
				if ( f<fabs(wolne[i]) ) f=fabs(wolne[i]);
			if (f<info->m_epsilon) break;                //najwiekszy przyrost < 1e-8

		}

		j=0;
		for (i=0; i<M; i++)
		{
			info->m_A[i] = abwe[j++];
			info->m_D[i] = abwe[j++];
			info->m_fr[i] = abwe[j++]/dt/2/M_PI;
			info->m_eta[i] = abwe[j++]/dt;
		}
		info->m_A0 = abwe[j];
	}

	if (!info->m_break)
		PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_STOPED,info->m_calculated);
	return 0;
}

///////////////////////////////////////////
void CAprox::Init()
{
	int pocz,kon,k,liczba_skladowych,i;
	CParametrySygnalu *parametry;
	char *unit;
	char napis[80];

	m_format=6;
	pocz=min(m_selection.m_from, m_selection.m_to);
	kon=max(m_selection.m_from, m_selection.m_to);
	k=m_selection.m_channel;
	parametry=m_parent->GetSignalParams(k);
	liczba_skladowych=parametry->m_liczba_skladowych;
	m_channels=liczba_skladowych+1;
	m_samples=kon-pocz;
	m_rate=m_parent->GetRate();
	unit=m_parent->GetChannelUnit(k);
	for (i=0; i<m_channels-1; i++)
	{
		StringCbPrintf(napis,80,"Component nr %d",i);
		m_opis[i].Init(1.0,0.0,unit,napis);
		m_parametry[i].Init();
	}
	m_opis[m_channels-1].Init(1.0,0.0,unit,"Rest");
	m_parametry[m_channels-1].Init();

	AllocBuffer(m_samples*m_channels<<3);
	m_info->m_channel=m_selection.m_channel;
	m_info->m_begin=m_selection.m_from;
	m_info->m_samples=kon-pocz;
	m_info->m_channels=m_parent->NumberOfChannels();
	m_info->m_probki=m_parent->Buffer();
	m_info->m_wyniki=m_buffer;
	APROXDATA(m_info)->m_rate=m_rate;
	APROXDATA(m_info)->m_components=liczba_skladowych;
	APROXDATA(m_info)->m_estimate_error=m_parametry_aproksymacji.m_estimate_error;
	APROXDATA(m_info)->m_observation_error=m_parametry_aproksymacji.m_observation_error;
	APROXDATA(m_info)->m_use_AD=m_parametry_aproksymacji.m_use_AD;
	APROXDATA(m_info)->m_use_eta=m_parametry_aproksymacji.m_use_eta;
	APROXDATA(m_info)->m_epsilon=1e-8;
	APROXDATA(m_info)->m_stop_cond=0;

	for (i=0;i<liczba_skladowych;i++)
	{
		APROXDATA(m_info)->m_fr[i]=parametry->m_skladowe[i].m_fr;
		APROXDATA(m_info)->m_Vr[i]=parametry->m_skladowe[i].m_Vr;
		APROXDATA(m_info)->m_A[i]=parametry->m_skladowe[i].m_A;
		APROXDATA(m_info)->m_D[i]=parametry->m_skladowe[i].m_D;
		APROXDATA(m_info)->m_eta[i]=parametry->m_skladowe[i].m_eta;
	}
	APROXDATA(m_info)->m_A0=parametry->m_A0;
}

///////////////////////////////////////////
void CAprox::CalcAprox(HWND parent)
{
	Init();
	m_info->m_parent=parent;
	StartThread((LPTHREAD_START_ROUTINE)AproxThreadProc);
}

///////////////////////////////////////////
void CAprox::CopyResults()
{
	int k,i,liczba_skladowych;
	CParametrySygnalu *parametry;

	k=m_selection.m_channel;
	parametry=m_parent->GetSignalParams(k);
	liczba_skladowych=parametry->m_liczba_skladowych;
	for (i=0;i<liczba_skladowych;i++)
	{
		parametry->m_skladowe[i].m_fr=APROXDATA(m_info)->m_fr[i];
		parametry->m_skladowe[i].m_Vr=APROXDATA(m_info)->m_Vr[i];
		parametry->m_skladowe[i].m_A=APROXDATA(m_info)->m_A[i];
		parametry->m_skladowe[i].m_D=APROXDATA(m_info)->m_D[i];
		parametry->m_skladowe[i].m_eta=APROXDATA(m_info)->m_eta[i];
	}
	parametry->m_A0=APROXDATA(m_info)->m_A0;
	m_parametry_aproksymacji.m_observation_error=APROXDATA(m_info)->m_observation_error;
}

/////////////////////////////////////////////////////////////////////////////
// CSkladowaAproksymacjiDlg
class CSkladowaAproksymacjiDlg : public CDialog
{
	CParametrySkladowej *m_params;
	CParametrySkladowej m_new_params;

public:
	CSkladowaAproksymacjiDlg(CParametrySkladowej *params)  : CDialog() { m_params=params; m_new_params=*params; }
	int Execute(HWND parent) { return CDialog::Execute(parent, IDD_APROXCOMPOINFO); }
	virtual BOOL OnInitDialog(HWND hWnd);
	virtual BOOL OnOk();
};


BOOL CSkladowaAproksymacjiDlg::OnInitDialog(HWND hWnd)
{
	char napis[80];

	CDialog::OnInitDialog(hWnd);
	SetText(IDC_NAME,m_new_params.m_name);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_fr);
	SetText(IDC_FR,napis);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_A);
	SetText(IDC_A,napis);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_D);
	SetText(IDC_D,napis);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_eta);
	SetText(IDC_ETA,napis);
	return TRUE;
}

BOOL CSkladowaAproksymacjiDlg::OnOk()
{
	GetText(IDC_NAME,m_new_params.m_name,41);
	GetDouble(IDC_FR,&m_new_params.m_fr);
	GetDouble(IDC_A,&m_new_params.m_A);
	GetDouble(IDC_D,&m_new_params.m_D);
	GetDouble(IDC_ETA,&m_new_params.m_eta);
	*m_params=m_new_params;
	return CDialog::OnOk();
}

/////////////////////////////////////////////////////////////////////////////
// CParametryAproksymacjiDlg

static LV_COLUMN skladowe_aproksymacji_listbox_columns[5]={
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_LEFT,  90,"Name",0,0},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 80,"Frequency",0,1},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 70,"A",0,2},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 70,"D",0,3},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 70,"Eta",0,4}};

void CParametryAproksymacjiDlg::fill_param(int i)
{
	char napis[80];
	LV_ITEM lvi;

	lvi.mask = LVIF_TEXT;
	lvi.iItem=i;
	lvi.iSubItem=1;
	lvi.pszText = napis;   // app. maintains text
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_skladowe[i].m_fr);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=2;
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_skladowe[i].m_A);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=3;
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_skladowe[i].m_D);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=4;
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_skladowe[i].m_eta);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
}

void CParametryAproksymacjiDlg::insert(int i)
{
	LV_ITEM lvi;
	CParametrySkladowej parametry;

	StringCbPrintf(parametry.m_name,41,"Component no %d",i+1);
	parametry.m_fr=0.0;
	parametry.m_Vr=0.0;
	parametry.m_K=0.0;
	parametry.m_A=0.0;
	parametry.m_D=0.0;
	parametry.m_eta=0.0;
	CSkladowaAproksymacjiDlg dlg(&parametry);
	if (dlg.Execute(m_handle)==IDOK)
	{
		m_new_params.InsertComponent(i,&parametry);
		lvi.iItem=i;
		lvi.mask = LVIF_TEXT;
		lvi.pszText = parametry.m_name;
		lvi.iSubItem=0;
		SendMessage(IDC_CHANNELS,LVM_INSERTITEM,0,(LPARAM)&lvi);
		fill_param(i);
		m_byly_zmiany=TRUE;
	}
}

void CParametryAproksymacjiDlg::change_settings()
{
	LV_ITEM lvi;
	int i;

	GetSelectedItem(IDC_CHANNELS,&i);
	if (i<m_new_params.m_liczba_skladowych)
	{
		CSkladowaAproksymacjiDlg dlg(&m_new_params.m_skladowe[i]);
		if (dlg.Execute(m_handle)==IDOK)
		{
			lvi.mask = LVIF_TEXT;
			lvi.pszText = m_new_params.m_skladowe[i].m_name;
			lvi.iSubItem=0;
			lvi.iItem=i;
			SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
			fill_param(i);
			m_byly_zmiany=TRUE;
		}
	}
	else
		if (i==m_new_params.m_liczba_skladowych) insert(i);
}

void CParametryAproksymacjiDlg::delete_component()
{
	int i;

	GetSelectedItem(IDC_CHANNELS,&i);
	if (i>=0 && i<m_new_params.m_liczba_skladowych)
	{
		SendMessage(IDC_CHANNELS,LVM_DELETEITEM,i,0);
		m_new_params.DeleteComponent(i);
	}
}

void CParametryAproksymacjiDlg::insert_component()
{
	int i;

	GetSelectedItem(IDC_CHANNELS,&i);
	if (i>=0 && i<=m_new_params.m_liczba_skladowych) insert(i);
}

void CParametryAproksymacjiDlg::create_columns()
{
	LV_ITEM lvi;
	int i;

	for (i=0; i<5; ++i)
		SendMessage(IDC_CHANNELS,LVM_INSERTCOLUMN,i,(LPARAM)&skladowe_aproksymacji_listbox_columns[i]);
	lvi.mask = LVIF_TEXT;
	lvi.iSubItem=0;
	for (i=0; i<m_new_params.m_liczba_skladowych; ++i)
	{
		lvi.iItem=i;
		lvi.pszText = m_new_params.m_skladowe[i].m_name;
		SendMessage(IDC_CHANNELS,LVM_INSERTITEM,0,(LPARAM)&lvi);
		fill_param(i);
	}
	lvi.iItem=m_new_params.m_liczba_skladowych;
	lvi.pszText = cNewComp;
	SendMessage(IDC_CHANNELS,LVM_INSERTITEM,0,(LPARAM)&lvi);

	lvi.mask = LVIF_STATE;
	lvi.state = LVIS_SELECTED;
	lvi.stateMask = LVIS_SELECTED;
	lvi.iItem=0;
	lvi.iSubItem=0;
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
}


BOOL CParametryAproksymacjiDlg::OnInitDialog(HWND hWnd)
{
	CDialog::OnInitDialog(hWnd);
	SendMessage(IDC_CHANNELS,LVM_SETEXTENDEDLISTVIEWSTYLE,LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES,LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES);
	m_byly_zmiany=FALSE;
	SetInteger(IDC_CHANNEL,m_new_selection.m_channel);
	SetInteger(IDC_FROM,m_new_selection.m_from);
	SetInteger(IDC_TO,m_new_selection.m_to);
	create_columns();
	SetCheck(IDC_USEAD,m_new_aprox_params.m_use_AD);
	SetCheck(IDC_USEETA,m_new_aprox_params.m_use_eta);
	return TRUE;
}

BOOL CParametryAproksymacjiDlg::OnNotify(int id, NMHDR *hdr)
{
	//int i;

	switch (id)
	{
	case IDC_CHANNELS:
		if (hdr->code == NM_DBLCLK) change_settings();
		break;
	}
	return TRUE;
}

BOOL CParametryAproksymacjiDlg::OnCommand(int id, int /*notify*/, HWND /*control*/)
{
	switch (id)
	{
	case IDC_EDIT :
		change_settings();
		break;
	case IDC_DELETE :
		delete_component();
		break;
	case IDC_INSERT :
		insert_component();
		break;
	case IDC_COPY :
		copy_all();
		break;
	}
	return TRUE;
}

BOOL CParametryAproksymacjiDlg::OnOk()
{
	GetInteger(IDC_CHANNEL,&m_new_selection.m_channel);
	GetInteger(IDC_FROM,&m_new_selection.m_from);
	GetInteger(IDC_TO,&m_new_selection.m_to);
	GetCheck(IDC_USEAD,&m_new_aprox_params.m_use_AD);
	GetCheck(IDC_USEETA,&m_new_aprox_params.m_use_eta);
	*m_selection=m_new_selection;
	*m_aprox_params=m_new_aprox_params;
	*m_params=m_new_params;
	return CDialog::OnOk();
}

void CParametryAproksymacjiDlg::copy_all()
{
	HGLOBAL clipboard;
	char *block;
	int i;

	if (OpenClipboard(NULL))
	{
		EmptyClipboard();
		clipboard = GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE|GMEM_DISCARDABLE, 2048);
		block = (char *)GlobalLock(clipboard);

/*
		block += sprintf(block,"%d\t%d\t%d\t%d\r\n",
			m_new_selection.m_channel,
			m_new_selection.m_from,
			m_new_selection.m_to,
			m_new_params.m_liczba_skladowych);
		for (i=0; i<m_new_params.m_liczba_skladowych; i++)
		{
			block += sprintf(block,"%6.4f\t%6.4f\t%6.4f\t%6.4f\r\n",
				m_new_params.m_skladowe[i].m_fr,
				m_new_params.m_skladowe[i].m_A,
				m_new_params.m_skladowe[i].m_D,
				m_new_params.m_skladowe[i].m_eta);
		}
		block += sprintf(block,"%d\t%d\r\n",
			m_new_aprox_params.m_use_AD,
			m_new_aprox_params.m_use_eta);
*/

		StringCbPrintf(block,20,"FRS1=[%6.4f",m_new_params.m_skladowe[0].m_fr);
		block += strlen(block);
		for (i=1; i<m_new_params.m_liczba_skladowych; i++)
		{
			StringCbPrintf(block,20,";%6.4f", m_new_params.m_skladowe[i].m_fr);
			block += strlen(block);
		}
		StringCbPrintf(block,10,"]\r\n");
		block += strlen(block);
		for (i=0; i<m_new_params.m_liczba_skladowych; i++)
		{
			if (m_new_params.m_skladowe[i].m_D<0)
				StringCbPrintf(block,30,"AS1%d=%6.4f-i*%6.4f;\r\n",i+1,m_new_params.m_skladowe[i].m_A,-m_new_params.m_skladowe[i].m_D);
			else
				StringCbPrintf(block,30,"AS1%d=%6.4f+i*%6.4f;\r\n",i+1,m_new_params.m_skladowe[i].m_A,m_new_params.m_skladowe[i].m_D);
			block += strlen(block);
		}

		GlobalUnlock(clipboard);
		SetClipboardData(CF_TEXT, clipboard);
		CloseClipboard();
	}
}

///////////////////////////////////////////////////////////////
// CAproxWnd
///////////////////////////////////////////////////////////////

int CAproxWnd::Create(int x, int y, int w, int h, HWND parent, DWORD style)
{
	CreateMDIWindow(
		"MiwilKalmanClass",
		"Aproximation Analysis",
		style,
		x, y, w, h,
		parent,
		(HINSTANCE)GetWindowLong(parent,GWL_HINSTANCE),
		(LPARAM)this
		);
	if (!m_handle) return 1;
	return 0;
}

int CAproxWnd::OnCreate(LPCREATESTRUCT /*lpCreateStruct*/)
{
	RECT r;

	GetClientRect(m_handle,&r);
	m_display = new CMeasureView(PLOTSTYLE_SIGNAL);
	if (m_display->Create(0,0,r.right,r.bottom,m_handle,WS_CLIPCHILDREN|WS_CHILD|WS_VISIBLE)) return 1;
	return 0;
}

void CAproxWnd::SetDocument(CMeasure *doc)
{
	char nazwa[286];

	CMeasureWnd::SetDocument(doc);
	StringCbCopy(nazwa,286,APROXDOC(doc)->GetParentFileName());
	StringCbCat(nazwa,286," - Approximation");
	SetTitle(nazwa);
	APROXDOC(doc)->CalcAprox(m_handle);
}

LRESULT CAproxWnd::OnCommand(WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam))
	{
	case ID_ANALYSE_RECALC:
		if (APROXDOC(m_doc)->GetParent())
		{
			CParametryAproksymacjiDlg dlg(APROXDOC(m_doc)->GetParentSignalParams(), APROXDOC(m_doc)->GetAproxParams(), APROXDOC(m_doc)->GetSelection());
			if (dlg.Execute(m_handle)==IDOK) APROXDOC(m_doc)->CalcAprox(m_handle);
		}
		break;
	case IDM_RECALC_STOPED:
		APROXDOC(m_doc)->ClearThread();
		m_display->DocumentChanged();
		break;
	}
	return CMeasureWnd::OnCommand(wParam, lParam);
}

