#include <windows.h>
#include <commctrl.h>
#include <strsafe.h>
#include <math.h>

#include "app.h"
#include "winfi32.h"
#include "resource.h"
#include "measure.h"
#include "calc.h"
#include "fft.h"


#define FFTDOC(a) ((CFft *)a)
#define FFTDATA(a) ((CFftData *)a)

/////////////////////////////////////////////////////////////////////////////
// CFft

CFft::CFft(CMeasure *parent, CSelection &selection, CParametryFouriera &fft_params) : CCalculation(parent,selection)
{
	m_parametry_fouriera=fft_params;
	m_info=new CFftData();
}

DWORD WINAPI FftThreadProc(CFftData *info)
{
	int i, j, N, skok;    //, dy;
	double si, co, ssi, sco, a, b;
	float *z;
	float *ft;
	DWORD start_tick=GetTickCount();

	float *probki = info->m_probki + (info->m_channel + info->m_begin*info->m_channels)*2;
	float *wyniki=info->m_wyniki;
	N = info->m_samples;
	skok = 2*info->m_channels;
	ft = wyniki;
	info->m_break=0;
	info->m_calculated=0;

	if (info->m_inverse)
	{
		for (i=0; i<N && !info->m_break; i++)         //obliczenie prazkow
		{
			sco=0.0;
			ssi=0.0;
			z = probki;
			for (j=0; j<N; j++)
			{
				co= cos(2*M_PI*j*i/N);
				si= sin(2*M_PI*j*i/N);
				a = *z;
				b = *(z+1);
				sco+= a*co + b*si;
				ssi+= - a*si + b*co;
				z+=skok;
			}
	
			*(ft++) = (float)(sco/2);
			*(ft++) = (float)(ssi/2);
	
			++(info->m_calculated);
			if (GetTickCount()-start_tick>500)
			{
				start_tick=GetTickCount();
				PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_REFRESH,info->m_calculated);
			}
		}
	}
	else
	{
		for (i=0; i<N && !info->m_break; i++)         //obliczenie prazkow
		{
			sco=0.0;
			ssi=0.0;
			z = probki;
			for (j=0; j<N; j++)
			{
				co= cos(2*M_PI*j*i/N);
				si= sin(2*M_PI*j*i/N);
				a = *z;
				b=0;
				sco+= a*co - b*si;
				ssi+= a*si + b*co;
				z+=skok;
			}
	
			*(ft++) = (float)(2*sco/N);
			*(ft++) = (float)(2*ssi/N);
	
			++(info->m_calculated);
			if (GetTickCount()-start_tick>500)
			{
				start_tick=GetTickCount();
				PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_REFRESH,info->m_calculated);
			}
		}
	}
	if (!info->m_break)
		PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_STOPED,info->m_calculated);
	return 0;
}

/*
// Szybka transformata Fouriera - metoda podzialu czasu
static void FDFT(float *xre, float *xim, int N)
{
	int i;
	double co,si;
	float *xxre = new float[N];
	float *xxim = new float[N];
	if (N > 2)
	{
		for (i=0;i<N/2;i++)   // podzial na czesc nieparzysta i parzysta
		{
			xxre[i] = xre[2*i];
			xxim[i] = xim[2*i];
			xxre[i+N/2] = xre[2*i+1];
			xxim[i+N/2] = xim[2*i+1];
		}
		FDFT(xxre,xxim,N/2);
		FDFT(xxre + N/2,xxim + N/2, N/2);
	}
	else
	{
		for (i=0;i<N;i++)
		{
			xxre[i] = xre[i];
			xxim[i] = xim[i];
		}
	}
	for (i=0;i<N/2;i++)
	{
		co=cos(2*M_PI*i/N);
		si=sin(2*M_PI*i/N);
		xre[i] = (float)(xxre[i] + xxre[i+N/2]*co + xxim[i+N/2]*si);
		xim[i] = (float)(xxim[i] - xxre[i+N/2]*si + xxim[i+N/2]*co);
		xre[i+N/2] = (float)(xxre[i] - xxre[i+N/2]*co - xxim[i+N/2]*si);
		xim[i+N/2] = (float)(xxim[i] + xxre[i+N/2]*si - xxim[i+N/2]*co);
	}
	
	delete []xxre;
	delete []xxim;
}

int najblizsze2N(int n)
{
	int a=1;
	while (a<n) a<<=1;
	return a;
}

DWORD WINAPI FftThreadProc(CFftData *info)
{
	int i, N, skok;
	float *z;
	float *ft;

	N = najblizsze2N(info->m_samples);
	float *rex = new float[N];
	float *imx = new float[N];

	skok = info->m_channels;
	z = info->m_probki;
	ft = info->m_wyniki;
	info->m_break=0;
	info->m_calculated=0;

	i=0;
	if (info->m_inverse)
	{
		while (i<info->m_samples)
		{
			rex[i] = z[i*skok];
			imx[i] = z[i*skok+1];
			++i;
		}
	}
	else
	{
		while (i<info->m_samples)
		{
			rex[i] = z[i*skok];
			imx[i] = 0;
			++i;
		}
	}
	while (i<N)
	{
		rex[i] = 0;
		imx[i] = 0;
		++i;
	}

	FDFT(rex,imx,N);

	if (info->m_inverse)
	{
		for (i=0;i<N;i++)
		{
			*(ft++) = rex[i]/N;
			*(ft++) = -imx[i]/N;
		}
	}
	else
	{
		for (i=0;i<N;i++)
		{
			*(ft++) = rex[i];
			*(ft++) = imx[i];
		}
	}

	info->m_calculated=N;
	PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_STOPED,info->m_calculated);
	delete []rex;
	delete []imx;
	return 0;
}

DWORD WINAPI FftThreadProc(CFftData *info)
{
	int i, j, N, skok;    //, dy;
	float *ptr;
	double si, co, dco, dsi, y0, y1, y2;
	float *z;
	float *ft;
	int limit;
	DWORD start_tick=GetTickCount();

	N = info->m_samples;
	skok = info->m_channels;
	z = info->m_probki;
	ft = info->m_wyniki;
	info->m_break=0;
	info->m_calculated=0;

	dco=cos(2*M_PI/N);
	dsi=sin(2*M_PI/N);

	co=1.0;
	si=0.0;

	ptr = z;                      //obliczenie zerowego prazka
	y2 = *ptr;
	ptr+=skok;
	for (j=N-1; j--;)
		{ y2 += *ptr; ptr+=skok; }
	y2 /= N;
	y1 = 0.0;
	*(ft++) = (float)y2;
	*(ft++) = (float)y1;
	info->m_calculated=1;
	limit = (N >> 1)-1;
	if (info->m_limited)
		limit = (int)floor(N*info->m_limit*info->m_dt)-1;

	for (i=0; i < limit; i++)         //obliczenie pozostalych prazkow
	{
		y0 = co*dco - si*dsi;       //co = cos((k+1)*2pi/N)
		si = co*dsi + si*dco;       //si = sin((k+1)*2pi/N)
		co = y0;

		ptr = z;
		co+=co;                     //co = 2*cos(k*2pi/N)
		y0 = *ptr;
		ptr+=skok;
		y1 = y0*co+ *ptr;
		ptr+=skok;
		for (j=N-2; j--;)
		{
			y2 = co*y1 - y0 + *ptr;
			y0=y1; y1=y2;
			ptr+=skok;
		}
		y2 = co*y1 - y0;            //jeszcze jeden przebieg z z[N+1]=0

		co /= 2;                    //co=cos(k*2pi/N)
		y2 = 2*(y2-co*y1)/N;
		y1 = -2*y1*si/N;
		*(ft++) = (float)y2;
		*(ft++) = (float)y1;

		//*(ft++) = sqrtl(y2*y2 + y1*y1);
		++(info->m_calculated);
		if (GetTickCount()-start_tick>500)
		{
			start_tick=GetTickCount();
			PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_REFRESH,info->m_calculated);
			if ( info->m_break )
				break;
		}
	}
	PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_STOPED,info->m_calculated);
	//_endthread();
	return 0;
}
*/

///////////////////////////////////////////
void CFft::Init()
{
	int pocz,kon,k;
	char *unit;

	m_format=6;
	pocz=min(m_selection.m_from, m_selection.m_to);
	kon=max(m_selection.m_from, m_selection.m_to);
	k=m_selection.m_channel;
	//dlugosc_wynikow=najblizsze2N(kon-pocz)<<3;		//<<2 ma float i <<1 bo dwa kanaly
	m_channels=1;
	m_samples=kon-pocz;
	m_rate=(kon-pocz)/m_parent->GetRate();
	unit=m_parent->GetChannelUnit(k);
	m_opis[0].Init(1.0,0.0,unit,"Spectrum");
	m_parametry[0].Init();

	AllocBuffer((m_samples*m_channels)<<3);
	m_info->m_channel=m_selection.m_channel;
	m_info->m_begin=m_selection.m_from;
	m_info->m_samples=kon-pocz;
	m_info->m_channels=m_parent->NumberOfChannels();
	m_info->m_probki=m_parent->Buffer();
	m_info->m_wyniki=m_buffer;
	FFTDATA(m_info)->m_inverse=m_parametry_fouriera.m_inverse;
	FFTDATA(m_info)->m_limited=m_parametry_fouriera.m_limited;
	FFTDATA(m_info)->m_limit=m_parametry_fouriera.m_limit;
	FFTDATA(m_info)->m_dt=1.0/m_parent->GetRate();
}

///////////////////////////////////////////
void CFft::CalcFFT(HWND parent)
{
	Init();
	m_info->m_parent=parent;
	StartThread((LPTHREAD_START_ROUTINE)FftThreadProc);
}


/////////////////////////////////////////////////////////////////////////////
// CParametryFourieraDlg

BOOL CParametryFourieraDlg::OnInitDialog(HWND hWnd)
{
	char napis[80];

	CDialog::OnInitDialog(hWnd);
	SetInteger(IDC_CHANNEL,m_new_selection.m_channel);
	SetInteger(IDC_FROM,m_new_selection.m_from);
	SetInteger(IDC_TO,m_new_selection.m_to);
	SetCheck(IDC_INVERSE,m_new_fft_params.m_inverse);
	SetCheck(IDC_TUNED,m_new_fft_params.m_tuned);
	StringCbPrintf(napis,80,"%8.6f",m_new_fft_params.m_tune_fr);
	SetText(IDC_TUNEFR,napis);
	return TRUE;
}

BOOL CParametryFourieraDlg::OnOk()
{
	GetInteger(IDC_CHANNEL,&m_new_selection.m_channel);
	GetInteger(IDC_FROM,&m_new_selection.m_from);
	GetInteger(IDC_TO,&m_new_selection.m_to);
	GetCheck(IDC_INVERSE,&m_new_fft_params.m_inverse);
	GetCheck(IDC_TUNED,&m_new_fft_params.m_tuned);
	GetLimitedDouble(IDC_TUNEFR,&m_new_fft_params.m_tune_fr,0.0,999999.0);
	*m_selection=m_new_selection;
	*m_fft_params=m_new_fft_params;
	return CDialog::OnOk();
}

///////////////////////////////////////////////////////////////
// CFftWnd
///////////////////////////////////////////////////////////////

int CFftWnd::Create(int x, int y, int w, int h, HWND parent, DWORD style)
{
	CreateMDIWindow(
		"MiwilFourierClass",
		"Fourier Analysis",
		style,
		x, y, w, h,
		parent,
		(HINSTANCE)GetWindowLong(parent,GWL_HINSTANCE),
		(LPARAM)this
		);
	if (!m_handle) return 1;
	return 0;
}

int CFftWnd::OnCreate(LPCREATESTRUCT /*create_struct*/)
{
	RECT r;

	GetClientRect(m_handle,&r);
	m_display = new CMeasureView(PLOTSTYLE_MODULE);
	if (m_display->Create(0,0,r.right,r.bottom,m_handle,WS_CLIPCHILDREN|WS_CHILD|WS_VISIBLE)) return 1;
	return 0;
}

void CFftWnd::StartCalc()
{
	m_display->SetPlotStyle(FFTDOC(m_doc)->GetFourierParams()->m_inverse ? PLOTSTYLE_SIGNAL : PLOTSTYLE_MODULE);
	FFTDOC(m_doc)->CalcFFT(m_handle);
}

void CFftWnd::SetDocument(CMeasure *doc)
{
	char nazwa[286];

	CMeasureWnd::SetDocument(doc);
	StringCbCopy(nazwa,286,FFTDOC(doc)->GetParentFileName());
	StringCbCat(nazwa,286," - Fourier Analysis");
	SetTitle(nazwa);
	StartCalc();
}

LRESULT CFftWnd::OnCommand(WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam))
	{
	case ID_ANALYSE_RECALC:
		if (FFTDOC(m_doc)->GetParent())
		{
			FFTDOC(m_doc)->ClearThread();
			CParametryFourieraDlg dlg(FFTDOC(m_doc)->GetFourierParams(), FFTDOC(m_doc)->GetSelection());
			if (dlg.Execute(m_handle)==IDOK) StartCalc();
		}
		break;
	case IDM_RECALC_STOPED:
		FFTDOC(m_doc)->ClearThread();
		m_display->DocumentChanged();
		break;
	case IDM_RECALC_REFRESH:
		m_doc->SetNumberOfSamples(FFTDOC(m_doc)->Calculated());
		m_display->DocumentChanged();
		break;
	case ID_ADD_SAMPLE:
		if (FFTDOC(m_doc)->GetParent())
			{
				FFTDOC(m_doc)->ClearThread();
				m_display->DocumentChanged();
				FFTDOC(m_doc)->GetSelection()->m_to = FFTDOC(m_doc)->GetSelection()->m_to + 1;
				StartCalc();
			}
		break;
	case ID_SUB_SAMPLE:
		if (FFTDOC(m_doc)->GetParent())
			{
				FFTDOC(m_doc)->ClearThread();
				m_display->DocumentChanged();
				FFTDOC(m_doc)->GetSelection()->m_to = FFTDOC(m_doc)->GetSelection()->m_to - 1;
				StartCalc();
			}
		break;
	}
	return CMeasureWnd::OnCommand(wParam, lParam);
}



///////////////////////////////////////////////////////////////////////////////
// Inne algorytmy transformaty fouriera

/*

// Szybka transformata Fouriera - metoda podzialu czasu
void DFT(float *xre,float *xim,long N)
{
  float pi = M_PI;
  float *xxre = new float[N];
  float *xxim = new float[N];
  if (N > 2)
  {
    for (long n=0;n<N/2;n++)   // podzial na czesc nieparzysta i parzysta
    {
      xxre[n] = xre[2*n];
      xxim[n] = xim[2*n];
      xxre[n+N/2] = xre[2*n+1];
      xxim[n+N/2] = xim[2*n+1];
    }
    DFT(xxre,xxim,N/2);
    DFT(xxre + N/2,xxim + N/2, N/2);
  }
  else
  {
    for (long n=0;n<N;n++)
    {
      xxre[n] = xre[n];
      xxim[n] = xim[n];
    }
  }
  for (long n=0;n<N/2;n++)
  {
    xre[n] = xxre[n] + xxre[n+N/2]*cos(2*pi*n/N) - xxim[n+N/2]*sin(-2*pi*n/N);
    xim[n] = xxim[n] + xxre[n+N/2]*sin(-2*pi*n/N) + xxim[n+N/2]*cos(2*pi*n/N);
  }
  for (long n=N/2;n<N;n++)
  {
    xre[n] = xxre[n-N/2] + xxre[n]*cos(2*pi*n/N) - xxim[n]*sin(-2*pi*n/N);
    xim[n] = xxim[n-N/2] + xxre[n]*sin(-2*pi*n/N) + xxim[n]*cos(2*pi*n/N);
  }

  delete []xxre;
  delete []xxim;
}

//   Szybka transformata Fouriera 
//   reX,imX - wektory probek czesci rzeczywistej i urojonej w dziedzinie czestotliwosci (na wyjsciu)
//   rex,imx -  -||- w dziedzinie czasu (na wejsciu)
//   N - liczba probek

void FFT(float *reX,float *imX,float *rex,float *imx, long N)
{
  for (long n=0;n<N;n++)
  {
    reX[n] = rex[n];
    imX[n] = imx[n];
  }
  DFT(reX,imX,N);
}

//   Odwrotna transformata Fouriera
//   reX,imX - wektory probek czesci rzeczywistej i urojonej w dziedzinie czestotliwosci (na wejsciu)
//   rex,imx -  -||- w dziedzinie czasu (na wyjsciu)
//   N - liczba probek

void IFFT(float *rex,float *imx,float *reX,float *imX, long N)
{
  for (long n=0;n<N;n++)
  {
    rex[n] = reX[n];
    imx[n] = -imX[n];
  }
  DFT(rex,imx,N);
  for (long n=0;n<N;n++)
  {
    rex[n] /= N;
    imx[n] /= -N;
  }
}

*/