#include <windows.h>
#include <commctrl.h>
#include <strsafe.h>
#include <math.h>

#include "matrix.h"
#include "app.h"
#include "winfi32.h"
#include "resource.h"
#include "measure.h"
#include "calc.h"
#include "kalman.h"


#define KALMANDOC(a) ((CKalman *)a)
#define KALMANDATA(a) ((CKalmanData *)a)

/////////////////////////////////////////////////////////////////////////////
// CKalman

CKalman::CKalman(CMeasure *parent, CSelection &selection, CParametryKalmana &parametry_kalmana)
    : CCalculation(parent, selection)
{
	m_parametry_kalmana=parametry_kalmana;
	m_info=new CKalmanData();
}

static DWORD WINAPI KalmanThreadProc(CKalmanData *info)
{
	double  s,s2;
	int i,j,ncol,N,M,skok;
	double wynikX,wynikY,pochodnaX,pochodnaY,new_sigma; //,fi
	int kierunek;
	double *p, *pnn, *r, *f, *g, *k, *x, *xp, *h;
	double omega[MAX_OMEGA];	//przeliczona czestotliwosc
	double dco[MAX_OMEGA];
	double dsi[MAX_OMEGA];
	double war[MAX_OMEGA];
	double eta[MAX_OMEGA];
	double p11,p12,p22;
	int obliczaj_wszystko;
	float *z;
	float *ft;
	double dt,old_sigma;
	DWORD start_tick=GetTickCount();

	dt=1/info->m_rate;
	//old_sigma = 1; //pKanal->amplituda;
	//old_sigma *= old_sigma*info->m_observation_error/200.0;

	float *probki = info->m_probki + (info->m_channel + info->m_begin*info->m_channels)*2;
	float *wyniki=info->m_wyniki;
	N = info->m_samples;
	skok = 2*info->m_channels;
	M=info->m_components;
	kierunek = info->m_direction;
	z = probki;
	if (kierunek<0) z+=(N-1)*skok;
	ft = wyniki;
	if (kierunek<0) ft+=(N-1)*(M*2+1)*2;
	new_sigma = 0.0;

	for (i=0; i<M; i++)
	{
		omega[i]=info->m_fr[i]*2*M_PI;
		if ( info->m_Vr[i] > 0 )
		{
			s = info->m_Vr[i];
			s *= s/2.0;
		}
		else
		{
			s=1.0;
			//s = pKanal->amplituda;
			//s *= s/2.0/M;			//dzieli na M rownych czesci
		}
		if ( info->m_fr[i]!=0.0 )
		{
			war[i]=(double)s;
			eta[i]=info->m_fr[i]/info->m_K[i];
		}
		else
		{
			war[i]=(double)(2*s);
			eta[i]=1.0/info->m_constant_period/info->m_K[i];
		}
	}

	old_sigma=0.0;
	for (i=0;i<M;i++)
		old_sigma += war[i];
	old_sigma *= info->m_observation_error/100.0;

	for (i=0;i<M;i++)
	{
		dco[i]=cos(kierunek*omega[i]*dt);
		dsi[i]=sin(kierunek*omega[i]*dt);
		if ( info->m_use_damp )
		{
			s=exp(-kierunek*info->m_eta[i]*dt);
			dco[i] *= s;
			dsi[i] *= s;
		}
	}

	switch (info->m_filter_type)
	{
	case 0:	//============================================
		ncol = M << 2;
//.......... poczatek filtracji .........
		p = new double[ncol*ncol];
		r = new double[ncol*ncol];
		f = new double[ncol*ncol];
		g = new double[ncol*ncol];
		k = new double[ncol];
		x = new double[ncol];
		xp = new double[ncol];
		h = new double[ncol];

//......... zerowanie macierzy ......
		clear_vector(ncol,x);
		clear_matrix(ncol,p);
		clear_matrix(ncol,f);
		clear_matrix(ncol,g);

//...... obliczenie warunkow poczatkowych ....
		for (i=0; i<ncol; i+=4)
		{
			j = i >> 2;
			h[i]=1.0;  h[i+1]=0;
			h[i+2]=0.0;  h[i+3]=0;

			f[i*ncol+i]=2*exp(-eta[j]*dt);  f[i*ncol+i+1]=-exp(-2*eta[j]*dt);
			f[(i+1)*ncol+i]=1.0;
			f[(i+2)*ncol+i+2]=f[i*ncol+i];  f[(i+2)*ncol+i+3]=f[i*ncol+i+1];
			f[(i+3)*ncol+i+2]=1.0;

			g[i*ncol+i] = 4*tanh(eta[j]*dt)*war[j]*exp(-2*eta[j]*dt) * sinh(eta[j]*dt)*sinh(eta[j]*dt);
			g[(i+2)*ncol+i+2] = g[i*ncol+i];

			s=war[j];  s2=s/cosh(eta[j]*dt);
			p[i*ncol+i]=(double)s;         p[i*ncol+i+1]=(double)s2;
			p[(i+1)*ncol+i]=(double)s2;    p[(i+1)*ncol+i+1]=(double)s;
			p[(i+2)*ncol+i+2]=(double)s;   p[(i+2)*ncol+i+3]=(double)s2;
			p[(i+3)*ncol+i+2]=(double)s2;  p[(i+3)*ncol+i+3]=(double)s;
		}

		for (i=0; i<M; i++) info->m_Vr[i] = 0.0;  //bo bedzie estymowana wariancja
//....... glowna petla filtracji ....
		while (N-- && !info->m_break)
		{
			 //.... fi*x ...
			matrix_mul_vector(ncol,xp,f,x);
			 //.... fi*p(n|n)*fit + g*gt...
			matrix_mul_matrix(ncol,r,f,p);
			copy_matrix(ncol,p,g);
			matrix_muladd_matrixT(ncol,p,r,f);
			 //... p(n|n-1)*h(n) ...
			matrix_mul_vector(ncol,x,p,h);
			 //... k(n) ...
			s=old_sigma;
			for (i=0; i<ncol; i++)  s += h[i] * x[i];
			for (i=0; i<ncol; i++)  k[i] = x[i]/s;
			 //... p(n|n) - k(n)*x(n)...
			matrix_sub_vector_mul_vectorT(ncol,p,k,x);
			 //... z(n)-h*x ...
			s = *z;
			for (i=0; i<ncol; i++)  s -= h[i] * xp[i];
			for (i=0; i<ncol; i++)  x[i] = xp[i] + k[i] * s;
			 //... wyniki ...
			s = *z;
			for (i=0,j=0; i<ncol; i+=4,j++)
			{
				wynikX= x[i] * h[i] + x[i+2] * h[i+2];
				wynikY= - x[i] * h[i+2] + x[i+2] * h[i];
				s2 = eta[j]*(x[i+1] - x[i]) + omega[j]*x[i+2];
				pochodnaX= (double)(s2 * h[i]);
				pochodnaY= - (double)(s2 * h[i+2]);
				s2 = eta[j]*(x[i+3] - x[i+2]) - omega[j]*x[i];
				pochodnaX += (double)(s2 * h[i+2]);
				pochodnaY += (double)(s2 * h[i]);
				s -= wynikX;
				info->m_Vr[j] += wynikX * wynikX;
				*(ft++) = (float)wynikX; //kierunek!
				*(ft++) = (float)wynikY;
				*(ft++) = (float)pochodnaX;
				*(ft++) = (float)pochodnaY;
			}
			new_sigma += (double)(s*s);
			*(ft++) = (float)s;  //kierunek!
			*(ft++) = (float)0;  //kierunek!
//... h(n) ...
			z += kierunek*skok;
			if (kierunek<0)
				ft-=(M*4+1)*2;
			for (i=0; i<ncol; i+=4)
			{
				j = i>>2;
				s = h[i] * dco[j] - h[i+2] * dsi[j];
				h[i+2] = h[i] * dsi[j] + h[i+2] * dco[j];
				h[i  ] = (double)s;
			}
			 //.............
			++(info->m_calculated);
			if (GetTickCount()-start_tick>500)
			{
				start_tick=GetTickCount();
				PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_REFRESH,info->m_calculated);
			}
		}

		delete p;
		delete r;
		delete f;
		delete g;
		delete k;
		delete x;
		delete xp;
		delete h;
		break;

	case 1://============================================
		ncol = M << 2;
//.......... poczatek filtracji .........
		p = new double[ncol*ncol];
		r = new double[ncol*ncol];
		f = new double[ncol*ncol];
		g = new double[ncol*ncol];
		k = new double[ncol];
		x = new double[ncol];
		xp = new double[ncol];
		h = new double[ncol];

//......... zerowanie macierzy ......
		clear_vector(ncol,x);
		clear_matrix(ncol,p);
		clear_matrix(ncol,f);
		clear_matrix(ncol,g);

//...... obliczenie warunkow poczatkowych ....
		for (i=0; i<ncol; i+=4) {
			j = i >> 2;
			h[i]=0.0;  h[i+1]=1.0;	// [0, cos, 0, sin]
			h[i+2]=0.0;  h[i+3]=0.0;

			s=exp(-eta[j]*dt);
			f[i*ncol+i]=(double)s;                 f[i*ncol+i+1]=0.0;
			f[(i+1)*ncol+i]=(double)(eta[j]*dt*s);   f[(i+1)*ncol+i+1]=(double)s;
			f[(i+2)*ncol+i+2]=(double)s;           f[(i+2)*ncol+i+3]=0.0;
			f[(i+3)*ncol+i+2]=(double)(eta[j]*dt*s); f[(i+3)*ncol+i+3]=(double)s;

			s=war[j];
			p[i*ncol+i]=(double)(2*s);        p[i*ncol+i+1]=(double)s;
			p[(i+1)*ncol+i]=(double)s;      p[(i+1)*ncol+i+1]=(double)s;
			p[(i+2)*ncol+i+2]=(double)(2*s);  p[(i+2)*ncol+i+3]=(double)s;
			p[(i+3)*ncol+i+2]=(double)s;    p[(i+3)*ncol+i+3]=(double)s;
		}

//obliczenie g*gT = P - Fi*P*FiT
		matrix_mul_matrix(ncol,r,f,p);	//r=f*p
		copy_matrix(ncol,g, p);
		matrix_mulsub_matrixT(ncol,g,r,f);	//g=p-r*fT

		for (i=0; i<M; i++) info->m_Vr[i] = 0.0;  //bo bedzie estymowana wariancja
//....... glowna petla filtracji ....
		while (N-- && !info->m_break)
		{
			matrix_mul_vector(ncol,xp,f,x);		// fi*x
			matrix_mul_matrix(ncol,r,f,p);      // fi*p(n|n)*fit + g*gt
			copy_matrix(ncol,p,g);
			matrix_muladd_matrixT(ncol,p,r,f);  // p(n|n-1)*h(n)
			matrix_mul_vector(ncol,x,p,h);      // k(n)
			s=old_sigma;
			for (i=0; i<ncol; i++)  s += h[i] * x[i];
			for (i=0; i<ncol; i++)  k[i] = x[i]/s;
			matrix_sub_vector_mul_vectorT(ncol,p,k,x);// p(n|n) - k(n)*x(n)
			s = *z;								// z(n)-h*x
			for (i=0; i<ncol; i++)  s -= h[i] * xp[i];
			for (i=0; i<ncol; i++)  x[i] = xp[i] + k[i] * s;
			s = *z;								// wyniki
			for (i=0,j=0; i<ncol; i+=4,j++)
			{

				wynikX= x[i+1] * h[i+1] + x[i+3] * h[i+3];
				wynikY= - x[i+1] * h[i+3] + x[i+3] * h[i+1];
				s2 = eta[j]*(x[i] - x[i+1]) + omega[j]*x[i+3];
				pochodnaX= (double)(s2 * h[i+1]);
				pochodnaY= - (double)(s2 * h[i+3]);
				s2 = eta[j]*(x[i+2] - x[i+3]) - omega[j]*x[i+1];
				pochodnaX += (double)(s2 * h[i+3]);
				pochodnaY += (double)(s2 * h[i+1]);
				s -= wynikX;
				info->m_Vr[j] += wynikX * wynikX;
				*(ft++) = (float)wynikX;  //kierunek!
				*(ft++) = (float)wynikY;
				*(ft++) = (float)pochodnaX;
				*(ft++) = (float)pochodnaY;
			}
			new_sigma += (double)(s*s);
			*(ft++) = (float)s;  //kierunek!
			*(ft++) = (float)0;  //kierunek!
			z += kierunek*skok;					// h(n)
			if (kierunek<0)
				ft-=(M*4+1)*2;
			for (i=0; i<ncol; i+=4)
			{
				j = i>>2;
				s = h[i+1]*dco[j] - h[i+3]*dsi[j];
				h[i+3] = h[i+1]*dsi[j] + h[i+3]*dco[j];
				h[i+1] = (double)s;
			}
			 //.............
			++(info->m_calculated);
			if (GetTickCount()-start_tick>500)
			{
				start_tick=GetTickCount();
				PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_REFRESH,info->m_calculated);
			}
		}

		delete p;
		delete r;
		delete f;
		delete g;
		delete k;
		delete x;
		delete xp;
		delete h;
		break;

	case 2://====================================================
		ncol = M * 6;
		//.......... poczatek filtracji .........
		p = new double[ncol*ncol];
		r = new double[ncol*ncol];
		f = new double[ncol*ncol];
		g = new double[ncol*ncol];
		k = new double[ncol];
		x = new double[ncol];
		xp = new double[ncol];
		h = new double[ncol];

		//......... zerowanie macierzy ......
		clear_vector(ncol,x);
		clear_matrix(ncol,p);
		clear_matrix(ncol,f);
		clear_matrix(ncol,g);

		//...... obliczenie warunkow poczatkowych ....
		for (i=0,j=0;i<ncol; i+=6,j++)
		{
			h[i  ]=0.0;  h[i+1]=0.0;  h[i+2]=1.0;	// [0, 0, cos, 0, 0, sin]
			h[i+3]=0.0;  h[i+4]=0.0;  h[i+5]=0.0;

			s = exp(-eta[j]*dt);
			f[i*ncol+i]=(double)s;                                 f[i*ncol+i+1]=0.0;             f[i*ncol+i+2]=0.0;
			f[(i+1)*ncol+i]=(double)(eta[j]*dt*s);                f[(i+1)*ncol+i+1]=(double)s;           f[(i+1)*ncol+i+2]=0.0;
			f[(i+2)*ncol+i]=(double)(eta[j]*dt*eta[j]*dt*s/2);   f[(i+2)*ncol+i+1]=(double)(eta[j]*dt*s); f[(i+2)*ncol+i+2]=(double)s;
			f[(i+3)*ncol+i+3]=(double)s;                          f[(i+3)*ncol+i+4]=0.0;         f[(i+3)*ncol+i+5]=0.0;
			f[(i+4)*ncol+i+3]=(double)(eta[j]*dt*s);              f[(i+4)*ncol+i+4]=(double)s;           f[(i+4)*ncol+i+5]=0.0;
			f[(i+5)*ncol+i+3]=(double)(eta[j]*dt*eta[j]*dt*s/2); f[(i+5)*ncol+i+4]=(double)(eta[j]*dt*s); f[(i+5)*ncol+i+5]=(double)s;

			s=war[j];
			p[i*ncol+i]=(double)(8*s/3);       p[i*ncol+i+1]=(double)(4*s/3);       p[i*ncol+i+2]=(double)(2*s/3);
			p[(i+1)*ncol+i]=(double)(4*s/3);   p[(i+1)*ncol+i+1]=(double)(4*s/3);   p[(i+1)*ncol+i+2]=(double)s;
			p[(i+2)*ncol+i]=(double)(2*s/3);   p[(i+2)*ncol+i+1]=(double)s;       p[(i+2)*ncol+i+2]=(double)s;
			p[(i+3)*ncol+i+3]=(double)(8*s/3); p[(i+3)*ncol+i+4]=(double)(4*s/3);   p[(i+3)*ncol+i+5]=(double)(2*s/3);
			p[(i+4)*ncol+i+3]=(double)(4*s/3); p[(i+4)*ncol+i+4]=(double)(4*s/3);   p[(i+4)*ncol+i+5]=(double)s;
			p[(i+5)*ncol+i+3]=(double)(2*s/3); p[(i+5)*ncol+i+4]=(double)s;       p[(i+5)*ncol+i+5]=(double)s;
		}

		//obliczenie g*gT = P - Fi*P*FiT
		matrix_mul_matrix(ncol,r,f,p);	//r=f*p
		copy_matrix(ncol,g, p);
		matrix_mulsub_matrixT(ncol,g,r,f);	//g=p-r*fT

		for (i=0; i<M; i++) info->m_Vr[i] = 0.0;  //bo bedzie estymowana wariancja
		//....... glowna petla filtracji ....
		while (N-- && !info->m_break)
		{
			 //.... fi*x ...
			matrix_mul_vector(ncol,xp,f,x);
			 //.... fi*p(n|n)*fit + g*gt...
			matrix_mul_matrix(ncol,r,f,p);
			copy_matrix(ncol,p,g);
			matrix_muladd_matrixT(ncol,p,r,f);
			 //... p(n|n-1)*h(n) ...
			matrix_mul_vector(ncol,x,p,h);
			 //... k(n) ...
			s=old_sigma;
			for (i=0; i<ncol; i++)  s += h[i] * x[i];
			for (i=0; i<ncol; i++)  k[i] = x[i]/s;
			 //... p(n|n) - k(n)*x(n)...
			matrix_sub_vector_mul_vectorT(ncol,p,k,x);
			 //... z(n)-h*x ...
			s = *z;
			for (i=0; i<ncol; i++)  s -= h[i] * xp[i];
			for (i=0; i<ncol; i++)  x[i] = xp[i] + k[i] * s;
			 //... wyniki ...
			s = *z;
			for (i=0,j=0;i<ncol; i+=6,j++)
			{
				wynikX= x[i+2] * h[i+2] + x[i+5] * h[i+5];
				wynikY= - x[i+2] * h[i+5] + x[i+5] * h[i+2];
				s2 = eta[j]*(x[i+1] - x[i+2]) + omega[j]*x[i+5];
				pochodnaX= (double)(s2 * h[i+2]);
				pochodnaY= - (double)(s2 * h[i+5]);
				s2 = eta[j]*(x[i+4] - x[i+5]) - omega[j]*x[i+2];
				pochodnaX += (double)(s2 * h[i+5]);
				pochodnaY += (double)(s2 * h[i+2]);
				s -= wynikX;
				info->m_Vr[j] += wynikX * wynikX;
				*(ft++) = (float)wynikX;
				*(ft++) = (float)wynikY;
				*(ft++) = (float)pochodnaX;
				*(ft++) = (float)pochodnaY;
			}
			new_sigma += (double)(s*s);
			*(ft++) = (float)s;
			*(ft++) = (float)0;  //kierunek!
			 //... h(n) ...
			z += kierunek*skok;
			if (kierunek<0)
				ft-=(M*4+1)*2;
			for (i=0,j=0;i<ncol; i+=6,j++)
			{
				s = h[i+2]*dco[j] - h[i+5]*dsi[j];
				h[i+5] = h[i+2]*dsi[j] + h[i+5]*dco[j];
				h[i+2] = (double)s;
			}
			 //.............
			++(info->m_calculated);
			if (GetTickCount()-start_tick>500)
			{
				start_tick=GetTickCount();
				PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_REFRESH,info->m_calculated);
			}
		}

		delete p;
		delete r;
		delete f;
		delete g;
		delete k;
		delete x;
		delete xp;
		delete h;
		break;

	case 3://=====================================================
		ncol = M * 6;
		//.......... poczatek filtracji .........
		p = new double[ncol*ncol];
		pnn = new double[ncol*ncol];
		r = new double[ncol*ncol];
		f = new double[ncol*ncol];
		g = new double[ncol*ncol];
		k = new double[ncol];
		x = new double[ncol];
		xp = new double[ncol];
		h = new double[ncol];

		//......... zerowanie macierzy ......
		clear_vector(ncol,x);
		clear_matrix(ncol,p);
		clear_matrix(ncol,f);
		clear_matrix(ncol,g);

		//...... obliczenie warunkow poczatkowych ....
		for (i=0,j=0;i<ncol; i+=6,j++)
		{
			h[i  ]=0.0;  h[i+1]=0.0;  h[i+2]=0.0;	// [0, 0, 0, 0, 1, 0]
			h[i+3]=0.0;  h[i+4]=1.0;  h[i+5]=0.0;

			s = exp(-eta[j]*dt)*dco[j];
			s2 = exp(-eta[j]*dt)*dsi[j];
			f[i*ncol+i]=(double)s;          f[i*ncol+i+1]=(double)s2;
			f[(i+1)*ncol+i]=-(double)s2;    f[(i+1)*ncol+i+1]=(double)s;
			f[(i+2)*ncol+i+2]=(double)s;    f[(i+2)*ncol+i+3]=(double)s2;
			f[(i+3)*ncol+i+2]=-(double)s2;  f[(i+3)*ncol+i+3]=(double)s;
			f[(i+4)*ncol+i+4]=(double)s;    f[(i+4)*ncol+i+5]=(double)s2;
			f[(i+5)*ncol+i+4]=-(double)s2;  f[(i+5)*ncol+i+5]=(double)s;
			s *= eta[j]*dt;
			s2 *= eta[j]*dt;
			f[(i+2)*ncol+i]=(double)s;      f[(i+2)*ncol+i+1]=(double)s2;
			f[(i+3)*ncol+i]=-(double)s2;    f[(i+3)*ncol+i+1]=(double)s;
			f[(i+4)*ncol+i+2]=(double)s;    f[(i+4)*ncol+i+3]=(double)s2;
			f[(i+5)*ncol+i+2]=-(double)s2;  f[(i+5)*ncol+i+3]=(double)s;
			s *= eta[j]*dt/2;
			s2 *= eta[j]*dt/2;
			f[(i+4)*ncol+i]=(double)s;      f[(i+4)*ncol+i+1]=(double)s2;
			f[(i+5)*ncol+i]=-(double)s2;    f[(i+5)*ncol+i+1]=(double)s;

			s=war[j];
			p11=(double)s; p12=0.0; p22=(double)s;

			p[i*ncol+i]=8*p11/3;      p[i*ncol+i+1]=8*p12/3;      p[i*ncol+i+2]=4*p11/3;      p[i*ncol+i+3]=4*p12/3;      p[i*ncol+i+4]=2*p11/3;      p[i*ncol+i+5]=2*p12/3;
			p[(i+1)*ncol+i]=8*p12/3;  p[(i+1)*ncol+i+1]=8*p22/3;  p[(i+1)*ncol+i+2]=4*p12/3;  p[(i+1)*ncol+i+3]=4*p22/3;  p[(i+1)*ncol+i+4]=2*p12/3;  p[(i+1)*ncol+i+5]=2*p22/3;
			p[(i+2)*ncol+i]=4*p11/3;  p[(i+2)*ncol+i+1]=4*p12/3;  p[(i+2)*ncol+i+2]=4*p11/3;  p[(i+2)*ncol+i+3]=4*p12/3;  p[(i+2)*ncol+i+4]=p11;      p[(i+2)*ncol+i+5]=p12;
			p[(i+3)*ncol+i]=4*p12/3;  p[(i+3)*ncol+i+1]=4*p22/3;  p[(i+3)*ncol+i+2]=4*p12/3;  p[(i+3)*ncol+i+3]=4*p22/3;  p[(i+3)*ncol+i+4]=p12;      p[(i+3)*ncol+i+5]=p22;
			p[(i+4)*ncol+i]=2*p11/3;  p[(i+4)*ncol+i+1]=2*p12/3;  p[(i+4)*ncol+i+2]=p11;      p[(i+4)*ncol+i+3]=p12;      p[(i+4)*ncol+i+4]=p11;      p[(i+4)*ncol+i+5]=p12;
			p[(i+5)*ncol+i]=2*p12/3;  p[(i+5)*ncol+i+1]=2*p22/3;  p[(i+5)*ncol+i+2]=p12;      p[(i+5)*ncol+i+3]=p22;      p[(i+5)*ncol+i+4]=p12;      p[(i+5)*ncol+i+5]=p22;
		}

		//obliczenie g*gT = P - Fi*P*FiT
		matrix_mul_matrix(ncol,r,f,p);	//r=f*p
		copy_matrix(ncol,g, p);
		matrix_mulsub_matrixT(ncol,g,r,f);	//g=p-r*fT

        for (i=0; i<M; i++) info->m_Vr[i] = 0.0;  //bo bedzie estymowana wariancja
		obliczaj_wszystko=1;
		//....... glowna petla filtracji ....
		while (N-- && !info->m_break)
		{
			 //.... fi*x ...
			matrix_mul_vector(ncol,xp,f,x);

			if (obliczaj_wszystko)
			{
				//.... p(n+1|n) = fi*p(n|n)*fit + g*gt...
				matrix_mul_matrix(ncol,r,f,p);
				copy_matrix(ncol,pnn,g);
				matrix_muladd_matrixT(ncol,pnn,r,f);
				//... K(n) = p(n|n-1)*h(n)/( ...
				matrix_mul_vector(ncol,x,pnn,h);
				s=old_sigma;
				for (i=0; i<ncol; i++)  s += h[i]*x[i];
				for (i=0; i<ncol; i++)  k[i] = x[i]/s;
				//... p(n|n) - k(n)*x(n)...
				s=0.0;	//maksymalny przyrost
				for (i=0; i<ncol; i++)
					for (j=0; j<ncol; j++)
					{
						pnn[i*ncol+j] -= k[i]*x[j];
						s2 = fabs( pnn[i*ncol+j] - p[i*ncol+j] );
						p[i*ncol+j] = pnn[i*ncol+j];
						if ( fabs(s2)>s ) s=fabs(s2);
					}
				if (s<info->m_asymptotic) obliczaj_wszystko=0;
			}
			 //... z(n)-h*x ...
			s = *z;
			for (i=0; i<ncol; i++)  s -= h[i] * xp[i];
			for (i=0; i<ncol; i++)  x[i] = xp[i] + k[i] * s;
			 //... wyniki ...
			s = *z;
			for (i=0,j=0;i<ncol; i+=6,j++)
			{
				wynikX= x[i+4];
				wynikY= x[i+5];
				pochodnaX = eta[j]*(x[i+2] - x[i+4]) + omega[j]*x[i+5];
				pochodnaY = eta[j]*(x[i+3] - x[i+5]) - omega[j]*x[i+4];
				s -= wynikX;
				info->m_Vr[j] += wynikX * wynikX;
				*(ft++) = (float)wynikX;
				*(ft++) = (float)wynikY;
				*(ft++) = (float)pochodnaX;
				*(ft++) = (float)pochodnaY;
			}
			new_sigma += (double)(s*s);
			*(ft++) = (float)s;
			*(ft++) = (float)0;  //kierunek!
			 //... h(n) ...
			z += kierunek*skok;
			if (kierunek<0)
				ft-=(M*4+1)*2;
			for (i=0,j=0;i<ncol; i+=6,j++)
			{
				s = h[i+2]*dco[j] - h[i+5]*dsi[j];
				h[i+5] = h[i+2]*dsi[j] + h[i+5]*dco[j];
				h[i+2] = (double)s;
			}
			 //.............
			++(info->m_calculated);
			if (GetTickCount()-start_tick>500)
			{
				start_tick=GetTickCount();
				PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_REFRESH,info->m_calculated);
			}
		}

		delete p;
		delete pnn;
		delete r;
		delete f;
		delete g;
		delete k;
		delete x;
		delete xp;
		delete h;
		break;
	}

	for (i=0; i<M; i++)
		info->m_Vr[i] = sqrt(2*info->m_Vr[i]/info->m_calculated);
	new_sigma /= info->m_calculated;
	if (info->m_estimate_error)
		info->m_observation_error *= new_sigma/old_sigma;

	if (!info->m_break)
		PostMessage(info->m_parent,WM_COMMAND,IDM_RECALC_STOPED,info->m_calculated);
	return 0;
}

///////////////////////////////////////////
void CKalman::Init()
{
	int pocz,kon,k,liczba_skladowych,i;
	CParametrySygnalu *parametry;
	char unit[20];
	char napis[80];

	m_format=6;
	pocz=min(m_selection.m_from, m_selection.m_to);
	kon=max(m_selection.m_from, m_selection.m_to);
	k=m_selection.m_channel;
	parametry=m_parent->GetSignalParams(k);
	liczba_skladowych=parametry->m_liczba_skladowych;
	m_channels=2*liczba_skladowych+1;
	m_samples=kon-pocz;
	m_rate=m_parent->GetRate();
	for (i=0; i<m_channels-1; i+=2)
	{
		StringCbPrintf(napis,80,"%s",GetChannelName(i));
		StringCbPrintf(unit,20,"%s",m_parent->GetChannelUnit(k));
		m_opis[i].Init(1.0,0.0,unit,napis);
		m_parametry[i].Init();
		StringCbPrintf(napis,80,"%s diff",GetChannelName(i));
		StringCbPrintf(unit,20,"%s/s",m_parent->GetChannelUnit(k));
		m_opis[i+1].Init(1.0,0.0,unit,napis);
		m_parametry[i+1].Init();
	}
	StringCbPrintf(unit,20,"%s",m_parent->GetChannelUnit(k));
	m_opis[m_channels-1].Init(1.0,0.0,unit,"Rest");
	m_parametry[m_channels-1].Init();

	AllocBuffer((m_samples*m_channels)<<3);
	m_info->m_channel=m_selection.m_channel;
	m_info->m_begin=m_selection.m_from;
	m_info->m_samples=kon-pocz;
	m_info->m_channels=m_parent->NumberOfChannels();
	m_info->m_probki=m_parent->Buffer();
	m_info->m_wyniki=m_buffer;
	KALMANDATA(m_info)->m_direction=m_selection.m_from<m_selection.m_to ? 1 : -1;
	KALMANDATA(m_info)->m_rate=m_rate;
	KALMANDATA(m_info)->m_components=liczba_skladowych;
	KALMANDATA(m_info)->m_filter_type=m_parametry_kalmana.m_filter_type;
	KALMANDATA(m_info)->m_estimate_error=m_parametry_kalmana.m_estimate_error;
	KALMANDATA(m_info)->m_observation_error=m_parametry_kalmana.m_observation_error;
	KALMANDATA(m_info)->m_constant_period=m_parametry_kalmana.m_constant_period;
	KALMANDATA(m_info)->m_asymptotic=m_parametry_kalmana.m_asymptotic;
	KALMANDATA(m_info)->m_use_damp=m_parametry_kalmana.m_use_damp;

	for (i=0;i<liczba_skladowych;i++)
	{
		KALMANDATA(m_info)->m_fr[i]=parametry->m_skladowe[i].m_fr;
		KALMANDATA(m_info)->m_Vr[i]=parametry->m_skladowe[i].m_Vr;
		KALMANDATA(m_info)->m_K[i]=parametry->m_skladowe[i].m_K;
		KALMANDATA(m_info)->m_eta[i]=parametry->m_skladowe[i].m_eta;
	}
}

///////////////////////////////////////////
void CKalman::CalcKalman(HWND parent)
{
	Init();
	m_info->m_parent=parent;
	StartThread((LPTHREAD_START_ROUTINE)KalmanThreadProc);
}

///////////////////////////////////////////
void CKalman::CopyResults()
{
	int k,i,liczba_skladowych;
	CParametrySygnalu *parametry;

	k=m_selection.m_channel;
	parametry=m_parent->GetSignalParams(k);
	liczba_skladowych=parametry->m_liczba_skladowych;
	for (i=0;i<liczba_skladowych;i++)
	{
		parametry->m_skladowe[i].m_fr=KALMANDATA(m_info)->m_fr[i];
		parametry->m_skladowe[i].m_Vr=KALMANDATA(m_info)->m_Vr[i];
		parametry->m_skladowe[i].m_K=KALMANDATA(m_info)->m_K[i];
		parametry->m_skladowe[i].m_eta=KALMANDATA(m_info)->m_eta[i];
	}
	m_parametry_kalmana.m_observation_error=KALMANDATA(m_info)->m_observation_error;
}

/////////////////////////////////////////////////////////////////////////////
// CSkladowaKalmanaDlg
class CSkladowaKalmanaDlg : public CDialog
{
	CParametrySkladowej *m_params;
	CParametrySkladowej m_new_params;

public:
	CSkladowaKalmanaDlg(CParametrySkladowej *params)  : CDialog() { m_params=params; m_new_params=*params; }
	int Execute(HWND parent) { return CDialog::Execute(parent, IDD_COMPOINFO); }
	virtual BOOL OnInitDialog(HWND hWnd);
	virtual BOOL OnOk();
};


BOOL CSkladowaKalmanaDlg::OnInitDialog(HWND hWnd)
{
	char napis[80];

	CDialog::OnInitDialog(hWnd);
	SetText(IDC_NAME,m_new_params.m_name);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_fr);
	SetText(IDC_FR,napis);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_Vr);
	SetText(IDC_VR,napis);
	StringCbPrintf(napis,80,"%4.2f",m_new_params.m_K);
	SetText(IDC_K,napis);
	return TRUE;
}

BOOL CSkladowaKalmanaDlg::OnOk()
{
	GetText(IDC_NAME,m_new_params.m_name,41);
	GetDouble(IDC_FR,&m_new_params.m_fr);
	GetDouble(IDC_VR,&m_new_params.m_Vr);
	GetDouble(IDC_K,&m_new_params.m_K);
	*m_params=m_new_params;
	return CDialog::OnOk();
}

/////////////////////////////////////////////////////////////////////////////
// CParametryKalmanaDlg

static LV_COLUMN skladowe_kalmana_listbox_columns[4]={
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_LEFT,	 90,"Name",0,0},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 80,"Frequency",0,1},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 80,"Amplitude",0,2},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 80,"K",0,3}};

const char nazwa_typu[LICZBA_FILTROW][30] = {"Discrete",
			  "Once differentiable",
			  "Twice differentiable",
			  "Quick twice differentiable"};

void CParametryKalmanaDlg::fill_param(int i)
{
	char napis[80];
	LV_ITEM lvi;

	lvi.mask = LVIF_TEXT;
	lvi.iItem=i;
	lvi.iSubItem=1;
	lvi.pszText = napis;   // app. maintains text
	StringCbPrintf(napis,80,"%8.6f",m_new_params.m_skladowe[i].m_fr);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=2;
	StringCbPrintf(napis,80,"%8.6f",m_new_params.m_skladowe[i].m_Vr);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=3;
	StringCbPrintf(napis,80,"%4.2f",m_new_params.m_skladowe[i].m_K);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
}

void CParametryKalmanaDlg::insert(int i)
{
	LV_ITEM lvi;
	CParametrySkladowej parametry;

	StringCbPrintf(parametry.m_name,41,"Component no %d",i+1);
	parametry.m_fr=1.0;
	parametry.m_Vr=1.0;
	parametry.m_K=5.0;
	parametry.m_A=1.0;
	parametry.m_D=1.0;
	parametry.m_eta=0.0;
	CSkladowaKalmanaDlg dlg(&parametry);
	if (dlg.Execute(m_handle)==IDOK)
	{
		m_new_params.InsertComponent(i,&parametry);
		lvi.iItem=i;
		lvi.mask = LVIF_TEXT;
		lvi.pszText = parametry.m_name;
		lvi.iSubItem=0;
		SendMessage(IDC_CHANNELS,LVM_INSERTITEM,0,(LPARAM)&lvi);
		fill_param(i);
		m_byly_zmiany=TRUE;
	}
}

void CParametryKalmanaDlg::change_settings()
{
	LV_ITEM lvi;
	int i;

	GetSelectedItem(IDC_CHANNELS,&i);
	if (i<m_new_params.m_liczba_skladowych)
	{
		CSkladowaKalmanaDlg dlg(&m_new_params.m_skladowe[i]);
		if (dlg.Execute(m_handle)==IDOK)
		{
			lvi.mask = LVIF_TEXT;
			lvi.pszText = m_new_params.m_skladowe[i].m_name;
			lvi.iSubItem=0;
			lvi.iItem=i;
			SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
			fill_param(i);
			m_byly_zmiany=TRUE;
		}
	}
	else
		if (i==m_new_params.m_liczba_skladowych)
			insert(i);
}

void CParametryKalmanaDlg::delete_component()
{
	int i;

	GetSelectedItem(IDC_CHANNELS,&i);
	if (i>=0 && i<m_new_params.m_liczba_skladowych)
	{
		SendMessage(IDC_CHANNELS,LVM_DELETEITEM,i,0);
		m_new_params.DeleteComponent(i);
	}
}

void CParametryKalmanaDlg::insert_component()
{
	int i;

	GetSelectedItem(IDC_CHANNELS,&i);
	if (i>=0 && i<=m_new_params.m_liczba_skladowych)
		insert(i);
}

void CParametryKalmanaDlg::create_columns()
{
	LV_ITEM lvi;
	int i;

	for (i=0; i<4; ++i)
		SendMessage(IDC_CHANNELS,LVM_INSERTCOLUMN,i,(LPARAM)&skladowe_kalmana_listbox_columns[i]);
	lvi.mask = LVIF_TEXT;
	lvi.iSubItem=0;
	for (i=0; i<m_new_params.m_liczba_skladowych; ++i)
	{
		lvi.iItem=i;
		lvi.pszText = m_new_params.m_skladowe[i].m_name;
		SendMessage(IDC_CHANNELS,LVM_INSERTITEM,0,(LPARAM)&lvi);
		fill_param(i);
	}
	lvi.iItem=m_new_params.m_liczba_skladowych;
	lvi.pszText = cNewComp;
	SendMessage(IDC_CHANNELS,LVM_INSERTITEM,0,(LPARAM)&lvi);

	lvi.mask = LVIF_STATE;
	lvi.state = LVIS_SELECTED;
	lvi.stateMask = LVIS_SELECTED;
	lvi.iItem=0;
	lvi.iSubItem=0;
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
}


BOOL CParametryKalmanaDlg::OnInitDialog(HWND hWnd)
{
	int i;
	char napis[80];

	CDialog::OnInitDialog(hWnd);
	SendMessage(IDC_CHANNELS,LVM_SETEXTENDEDLISTVIEWSTYLE,LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES,LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES);
	m_byly_zmiany=FALSE;
	SetInteger(IDC_CHANNEL,m_new_selection.m_channel);
	SetInteger(IDC_FROM,m_new_selection.m_from);
	SetInteger(IDC_TO,m_new_selection.m_to);
	create_columns();
	StringCbPrintf(napis,80,"%3.1f",m_new_kalman_params.m_observation_error);
	SetText(IDC_OBERR,napis);
	SetCheck(IDC_USE_DAMP,m_new_kalman_params.m_use_damp);
	for (i=0; i<LICZBA_FILTROW; ++i)
		SendMessage(IDC_FTYPE,CB_ADDSTRING,0,(LPARAM) (LPCTSTR) nazwa_typu[i]);
	SendMessage(IDC_FTYPE,CB_SETCURSEL,m_new_kalman_params.m_filter_type,0);
	return TRUE;
}

BOOL CParametryKalmanaDlg::OnNotify(int id, NMHDR *hdr)
{
	switch (id)
	{
	case IDC_CHANNELS:
		if (hdr->code == NM_DBLCLK)
			change_settings();
		break;
	}
	return TRUE;
}

BOOL CParametryKalmanaDlg::OnCommand(int id, int /*notify*/, HWND /*control*/)
{
	switch (id)
	{
	case IDC_EDIT :
		change_settings();
		break;
	case IDC_DELETE :
		delete_component();
		break;
	case IDC_INSERT :
		insert_component();
		break;
	case IDC_COPY :
		copy_all();
		break;
	}
	return TRUE;
}

BOOL CParametryKalmanaDlg::OnOk()
{
	GetInteger(IDC_CHANNEL,&m_new_selection.m_channel);
	GetInteger(IDC_FROM,&m_new_selection.m_from);
	GetInteger(IDC_TO,&m_new_selection.m_to);
	GetLimitedDouble(IDC_OBERR,&m_new_kalman_params.m_observation_error,0.0,100.0);
	GetCheck(IDC_USE_DAMP,&m_new_kalman_params.m_use_damp);
	m_new_kalman_params.m_filter_type=SendMessage(IDC_FTYPE,CB_GETCURSEL,0,0);
	*m_selection=m_new_selection;
	*m_kalman_params=m_new_kalman_params;
	*m_params=m_new_params;
	return CDialog::OnOk();
}

void CParametryKalmanaDlg::copy_all()
{
	HGLOBAL clipboard;
	char *block;
	int i;

	if (OpenClipboard(NULL))
	{
		EmptyClipboard();
		clipboard = GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE|GMEM_DISCARDABLE, 2048);
		block = (char *)GlobalLock(clipboard);

		StringCbPrintf(block,40,"%d\t%d\t%d\t%d\r\n",
			m_new_selection.m_channel,
			m_new_selection.m_from,
			m_new_selection.m_to,
			m_new_params.m_liczba_skladowych);
		block += strlen(block);
		for (i=0; i<m_new_params.m_liczba_skladowych; i++)
		{
			block += StringCbPrintf(block,60,"%8.6f\t%8.6f\t%8.6f\r\n",
				m_new_params.m_skladowe[i].m_fr,
				m_new_params.m_skladowe[i].m_Vr,
				m_new_params.m_skladowe[i].m_K);
			block += strlen(block);
		}
		StringCbPrintf(block,40,"%4.2f\t%d\t%d\r\n",
			m_new_kalman_params.m_observation_error,
			m_new_kalman_params.m_use_damp,
			m_new_kalman_params.m_filter_type);
		GlobalUnlock(clipboard);
		SetClipboardData(CF_TEXT, clipboard);
		CloseClipboard();
	}
}

///////////////////////////////////////////////////////////////
// CKalmanWnd
///////////////////////////////////////////////////////////////

int CKalmanWnd::Create(int x, int y, int w, int h, HWND parent, DWORD style)
{
	CreateMDIWindow(
		"MiwilKalmanClass",
		"Kalman Analysis",
		style,
		x, y, w, h,
		parent,
		(HINSTANCE)GetWindowLong(parent,GWL_HINSTANCE),
		(LPARAM)this
		);
	if (!m_handle)
		return 1;
	return 0;
}

int CKalmanWnd::OnCreate(LPCREATESTRUCT /*create_struct*/)
{
	RECT r;

	GetClientRect(m_handle,&r);
	m_display = new CMeasureView(PLOTSTYLE_SIGNAL);
	if (m_display->Create(0,0,r.right,r.bottom,m_handle,WS_CLIPCHILDREN|WS_CHILD|WS_VISIBLE))
		return 1;
	return 0;
}

void CKalmanWnd::StartCalc()
{
	KALMANDOC(m_doc)->CalcKalman(m_handle);
}

void CKalmanWnd::SetDocument(CMeasure *doc)
{
	char nazwa[286];

	CMeasureWnd::SetDocument(doc);
	StringCbCopy(nazwa,286,KALMANDOC(doc)->GetParentFileName());
	StringCbCat(nazwa,286," - Kalman Analysis");
	SetTitle(nazwa);
	StartCalc();
}

LRESULT CKalmanWnd::OnCommand(WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam))
	{
	case ID_ANALYSE_RECALC:
		if (KALMANDOC(m_doc)->GetParent())
		{
			KALMANDOC(m_doc)->ClearThread();
			CParametryKalmanaDlg dlg(KALMANDOC(m_doc)->GetParentSignalParams(), KALMANDOC(m_doc)->GetKalmanParams(), KALMANDOC(m_doc)->GetSelection());
			if (dlg.Execute(m_handle)==IDOK)
				StartCalc();
		}
		break;
	case IDM_RECALC_STOPED:
		KALMANDOC(m_doc)->ClearThread();
		m_display->DocumentChanged();
		{
			int visible=m_doc->NumberOfChannels();
			for (int i=1; i<visible; i+=2)
				m_display->colapse(i);
			m_display->OnSize(0,0,0);
		}
		break;
	case IDM_RECALC_REFRESH:
		m_doc->SetNumberOfSamples(KALMANDOC(m_doc)->Calculated());
		m_display->DocumentChanged();
		break;
	}
	return CMeasureWnd::OnCommand(wParam, lParam);
}

