#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "matrix.h"

///////////////////////////////////////////////
//procedury mnozenia macierzy
//////////////////////////////////////////////

//odwracanie macieryz
//c musi miec wielkosc N+M-1 komrek
double *alloc_matrix(int N)
{
	return (double *)malloc(N*N*sizeof(double));
}

double *alloc_vector(int N)
{
	return (double *)malloc(N*sizeof(double));
}

double *dup_vector(int N, double *a)
{
	double *b=alloc_vector(N);
	copy_vector(N,b,a);
	return b;
}

//print matrix A
void print_matrix(int N, double *a, char *name)
{
	int i,j;
	FILE *f;

	f=fopen(name,"wt");
	for (i=N;i--;)
	{
		for (j=N;j--;)
			fprintf(f,"%g\t", *(a++));
		fprintf(f,"\n");
	}
	fclose(f);
}

//print vector A
void print_vector(int N, double *a, char *name)
{
	int i;
	FILE *f;

	f=fopen(name,"wt");
	for (i=N;i--;)
		fprintf(f,"%g\n", *(a++));
	fclose(f);
}

// c = a * b
void scalar_mul_vector(int N, double *c, double a, double *b)
{
	while (N--)
		*(c++)=a * *(b++);
}

// c = a * b
void scalar_mul_matrix(int N, double *c, double a, double *b)
{
	N*=N;
	while (N--)
		*(c++)=a * *(b++);
}

// c = a * b
void matrix_mul_vector(int N, double *c, double *a, double *b)
{
	int i,j;
	double *p;

	i=N;
	while (i--)
	{
		*c=0.0;
		p=b;
		j=N;
		while (j--)
			*c += *(a++) * *(p++);
		c++;
	}
}

// c = a * b
void matrix_mul_matrix(int N, double *c, double *a, double *b)
{
	int i,j,k;
	double *p;

	k=N;
	while (k--)
	{
		i=N;
		while (i--)
		{
			*c=0.0;
			p=b;
			j=N;
			while (j--)
			{
				*c += *(a++) * *p;
				p+=N;
			}
			b++;
			a -= N;
			c++;
		}
		b -= N;
		a += N;
	}
}

// c = a * bT
void matrix_mul_matrixT(int N, double *c, double *a, double *b)
{
	int i,j,k;
	double *p;

	k=N;
	while (k--)
	{
		p=b;
		i=N;
		while (i--)
		{
			*c=0.0;
			j=N;
			while (j--)
				*c += *(a++) * *(p++);
			a -= N;
			c++;
		}
		a += N;
	}
}

// operacja C = C - A x BT

void matrix_mulsub_matrixT(int N, double *c, double *a, double *b)
{
	int i,j,k;
	double *p;

	k=N;
	while (k--)
	{
		p=b;
		i=N;
		while (i--)
		{
			j=N;
			while (j--)
				*c -= *(a++) * *(p++);
			a -= N;
			c++;
		}
		a += N;
	}
}

// operacja C = C + A x BT

void matrix_muladd_matrixT(int N, double *c, double *a, double *b)
{
	int i,j,k;
	double *p;

	k=N;
	while (k--)
	{
		p=b;
		i=N;
		while (i--)
		{
			j=N;
			while (j--)
				*c += *(a++) * *(p++);
			a -= N;
			c++;
		}
		a += N;
	}
}

// operacja C = C - A x BT

void matrix_sub_vector_mul_vectorT(int N, double *c, double *a, double *b)
{
    int i,j;
    double *p;

    i=N;
    while (i--)
    {
        p = b;
        j=N;
        while (j--)
            *(c++) -= *a * *(p++);
        a++;
    }
}

//A = A - B * A * BT
void calc_sqrform(int N, double *a, double *b)
{
	double *tmp;

	tmp=alloc_matrix(N);
	matrix_mul_matrix(N,tmp,b,a);
	matrix_mulsub_matrixT(N,a,tmp,b);
	free(tmp);
}

//A = (A + AT)/2
void matrix_add_matrixT_div_2(int N, double *a)
{
    int d,i,j;
    double *p;

    d=0;
    i= N++;
    while (--i)
    {
        a++;
        p=a;
        d += N-2;
        j=i;
        while (j--)
        {
			*p = 0.5*( *(p+d) + *p );
            *(p+d) = *p;
             p += N;
        }
    }
}

//operacja A = A - B
void vector_sub_vector(int N, double *a, double *b)
{
	while (N--)
		*(a++) -= *(b++);
}

//operacja A = B - A
void vector_rsub_vector(int N, double *a, double *b)
{
	while (N--)
	{
		*a = *(b++) - *a;
		++a;
	}
}

//operacja A = A + B
void vector_add_vector(int N, double *a, double *b)
{
	while (N--)
		*(a++) += *(b++);
}

//operacja return AT * B
double vectorT_mul_vector(int N, double *a, double *b)
{
	double c;

	c=0;
	while (N--)
		c += *(a++) * *(b++);
	return c;
}

//copy vector B = A
void copy_vector(int N, double *b, double *a)
{
	while (N--)
		*(b++) = *(a++);
}

//copy matrix B = A
void copy_matrix(int N, double *b, double *a)
{
	copy_vector(N*N,b,a);
}

//clear matrix A = 0
void clear_matrix(int N, double *a)
{
	N *= N;
	while (N--)
		*(a++) = 0.0;
}
//clear matrix A = 0
void clear_vector(int N, double *a)
{
	while (N--)
		*(a++) = 0.0;
}

//copy hi to lo
void make_simetric(int N, double *a)
{
    int d,i,j;
    double *p;

    d=0;
    i= N++;
    while (--i)
    {
        a++;
        p=a;
        d += N-2;
        j=i;
        while (j--)
        {
            *(p+d) = *p;
             p += N;
        }
    }
}

// A = AT
void transpose(int N, double *a)
{
    int d,i,j;
    double *p;
	double s;

    d=0;
    i= N++;
    while (--i)
    {
        a++;
        p=a;
        d += N-2;
        j=i;
        while (j--)
        {
			s = *(p+d);
            *(p+d) = *p;
			*p = s;
            p += N;
        }
    }
}

//mnoy wielomian a przez wielomian b i wynik umieszcza w c
//c musi miec wielkosc N+M-1 komrek
void conv(double *c, int N, double *a, int M, double *b)
{
	int i,j;

	clear_vector(N+M-1,c);
	for (i=0; i<M; i++)
	{
		for (j=0; j<N; j++)
			*(c++) += *(a++) * *b;
		c -= N-1;
		a -= N;
		++b;
	}
}
////////////////////////////////////////////////////////
double ZeroTolerance = 1.0e-17;


double Eliminacja(int N, double *A, double *b)
{
	int maxi, maxj, i, j, k;
	double *b_ptr, *corner, *ptr, *sptr, *dptr;
	double max, nb;
	int delta;
	int w[100];
	double wyz;

	wyz=1.0;
	delta = 0;
	corner = A;
	b_ptr = b;
	for (k=N; k--; )
		w[k]=-1;

	k=0;
	do
	{
		//....szukanie maximalnej wartosci w podmacierzy A(k,k)..........
		maxi=k;
		maxj=k;
		ptr=corner;
		max=0.0;
		for (i=k; i<N; i++)
		{
			for (j=k; j<N; j++)
			{
				if ( fabs(*ptr) > max )
				{
					max=fabs(*ptr);
					maxi=i;
					maxj=j;
				}
				ptr++;
			}
			ptr+=delta;
		}
		if ( max<ZeroTolerance ) return 0.0;
		//.......wartosc max znaleziona..........
		if (maxi != k)
		{
			sptr=corner+(maxi-k)*N;
			dptr=corner;
			for (i=N-k;i--;sptr++,dptr++)
			{
				nb=*dptr;
				*dptr=*sptr;
				*sptr=nb;
			}
			sptr=b_ptr+(maxi-k);
			nb=*b_ptr;
			*b_ptr=*sptr;
			*sptr=nb;
		}
		//..........przestawionio wiersze i b[k]..............
		if (maxj != k)
		{
			sptr=A+maxj;
			dptr=A+k;
			for (i=N;i--;sptr+=N,dptr+=N)
			{
				nb=*dptr;
				*dptr=*sptr;
				*sptr=nb;
			}
			w[k]=maxj;
		}
		//..........przestawianie kolumn i kolejnosci zmiennych zrobione..........
		max = *corner;
		wyz *= max;
		k++;
		ptr=corner+1;
		for (j=N-k;j--;ptr++)
			*ptr /= max;
		*b_ptr /= max;

		nb = *b_ptr;
		b_ptr++;

		dptr = corner+N;
		corner++;
		ptr = b_ptr;
		for (i=N-k; i--; dptr+=delta,ptr++)
		{
			sptr = corner;
			max = *dptr;
			dptr++;
			for (j=N-k; j--; sptr++,dptr++)
				*dptr -= *sptr * max;
			*ptr -= nb * max;
		}

		delta++;
		corner+=N;
	} while (k<N-1);

	if ( fabs(*corner) < ZeroTolerance ) return 0.0;
	wyz *= *corner;
	*b_ptr /= *corner;

	delta=N;
	dptr = b_ptr;
	dptr--;
	for (i=N; --i; dptr--,delta--)
	{
		ptr=b_ptr;
		corner-=delta;
		for (j=N-i+1; --j; corner--,ptr--)
			*dptr -= *corner * *ptr;
	}

	for (i=N;i--;b_ptr--)
	{
		if ( w[i] >=0 )
		{
			ptr = b+w[i];
			nb=*ptr;
			*ptr=*b_ptr;
			*b_ptr=nb;
		}
	}
	return wyz;
}



//odwracanie macieryz
//c musi miec wielkosc N+M-1 komrek
void inverse_matrix(int /*N*/, double * /*b*/, double * /*a*/)
{
}

// Rozkalada macierz symetryczna dodatnio okreslona
// na dwie macierze trojkatne taz, ze ich iloczyn (z transpozycja)
// jest rowny macierzy wejsciowej
// Wynikiem jest macierz trojkatna dolna;
void Choleski(int N, double *a)
{
	int i,j,k;
	double s;

	for (j=0; j<N; j++)
	{
		s=0.0;
		for (k=0; k<j; k++)
			s+=a[j*N+k]*a[j*N+k];
		a[j*N+j] = sqrt( a[j*N+j] - s );
		for (i=j+1; i<N; i++)
		{
			s=0.0;
			for (k=0; k<j; k++)
				s+=a[i*N+k]*a[j*N+k];
			a[i*N+j]=(a[i*N+j]-s)/a[j*N+j];
			a[j*N+i]=0.0;
		}
	}
}