#define OBL_MIN         0x0001
#define OBL_MAX         0x0002
#define OBL_SR          0x0004
#define OBL_STD         0x0008
#define OBL_MIN_FALA    0x0010
#define OBL_MAX_FALA    0x0020
#define OBL_SR_FALA     0x0040
#define OBL_FALA        (OBL_MIN_FALA | OBL_MAX_FALA | OBL_SR_FALA)
#define OBL_MIN_OKRES   0x0080
#define OBL_MAX_OKRES   0x0100
#define OBL_SR_OKRES    0x0200
#define OBL_OKRES       (OBL_MIN_OKRES | OBL_MAX_OKRES | OBL_SR_OKRES)
#define OBL_SR_KAT      0x0400
#define OBL_ALL			0xFFFF

#define PLOTSTYLE_SIGNAL	0x01
#define PLOTSTYLE_COSIGNAL	0x02
#define PLOTSTYLE_ENVELOPE	0x04
#define PLOTSTYLE_MODULE	0x08
#define PLOTSTYLE_PHASE		0x10
#define PLOTSTYLE_TEXT		0x20
#define PLOTSTYLE_SIGENV	(PLOTSTYLE_SIGNAL|PLOTSTYLE_ENVELOPE)
#define PLOTSTYLE_BOTH		(PLOTSTYLE_SIGNAL|PLOTSTYLE_COSIGNAL)
#define PLOTSTYLE_BOTHENV	(PLOTSTYLE_SIGNAL|PLOTSTYLE_COSIGNAL|PLOTSTYLE_ENVELOPE)

#define WYSOKOSC_TYT      12

///////////////////////////////////////
class CFile
{
	HANDLE m_handle;
public:
	CFile() {}
	~CFile() {}
	int Open(char *name);
	int Create(char *name);
	int Close() { CloseHandle(m_handle); return 0; }
	int Read(void *buf, int size);
	int Write(void *buf, int size);
};


//////////////////////////////////////
#define M11_RECORD_SIZE 122
#define sqr(a) ((a)*(a))
typedef unsigned char TM11Record[M11_RECORD_SIZE];

struct  TChannelStatistic
{
	double min;
	double max;
	double mean;
	double war;
	double acor;
	int poz_min;
	int poz_max;
};

//////////////////////////////////////
#define LICZBA_PROBEK 7

class CFiltr
{
	double m_probki[LICZBA_PROBEK];
	int m_wsk;
	int m_liczba_wpisow;

public:
	CFiltr();
	double od_konca(int i);
	int jest_zero();
	double miejsce_zerowe();
	double ekstremum(double x);
	double pochodna();
	double wartosc();
	void AddSample(double a);
	void wpisz_na_szczyt(double a) { m_probki[m_wsk]=a; }
	double na_szczycie() { return m_probki[m_wsk]; }
	int liczba_wpisow() { return m_liczba_wpisow; }
};
//////////////////////////////////////
struct CStatyData
{
	float *m_probki;
	int m_samples;
	int m_channels;
	double m_rate;

	unsigned m_obliczono;
	double m_min_wartosc;
	double m_max_wartosc;
	double m_srednia;
	double m_odchylenie_std;
	double m_min_fala;
	double m_max_fala;
	double m_srednia_fala;
	double m_min_okres;
	double m_max_okres;
	double m_sredni_okres;
	int m_liczba_fal;
	int m_liczba_okresow;
};

//////////////////////////////////////
struct CParametrySkladowej
{
public:
	char m_name[41];
	double m_fr;   //czestotliwosc
	double m_A;    //amplituda cosinusa skladowej do filtracji
	double m_D;    //amplituda sinusa skladowej do filtracji
	double m_eta;   //tlumienie
	double m_Vr;	//wariancja skladowej do filtracji
	double m_K;	//szerokosc filtru dla skladowej

public:
	void Init(char *nazwa);
};
//////////////////////////////////////
class CParametrySygnalu
{
public:
	double m_A0;
	double m_fr0;		//czestotliwosc glowna * 2*PI
	double m_amplituda;
	int m_liczba_skladowych;
	CParametrySkladowej m_skladowe[MAX_OMEGA];

public:
	void Init();
	void DeleteComponent(int skladnik);
	void InsertComponent(int skladnik, CParametrySkladowej *parametry);
	void AppendComponent(CParametrySkladowej *parametry);
};

//////////////////////////////////////
class COpisKanalu
{
public:
	double m_wspol;
	double m_stala;
	char m_jednostka[8];
	char m_nazwa[41];
	double m_user1;
	double m_user2;
	double m_user3;
	double m_user4;

public:
	COpisKanalu() { }
	~COpisKanalu() {}
	int Load(CFile &file);
	int Save(CFile &file);
	BOOL LoadFactor(CFile &file);
	void Init(double a, double b, const char *unit, const char *name);
};

//////////////////////////////////////
class CHeader
{
public:
	CParametrySygnalu m_parametry[MAX_CHANNEL];

protected:
	char m_format;            //3-szeregowe inty, 4-rownolegle inty, 5-rownolegle floaty
	char m_file_name[256];
	double m_rate;
	int m_channels;
	int m_samples;
	char m_nazwa[61];
	unsigned m_rec_date;
	COpisKanalu m_opis[MAX_CHANNEL];

public:
	CHeader() { m_samples=0; m_channels=0; m_file_name[0]='\0'; }
	virtual ~CHeader() { }
	void SetNumberOfSamples(int samples) { m_samples=samples; }
	int NumberOfChannels() { return m_channels; }
	int NumberOfSamples() { return m_samples; }
	char *GetChannelName(int channel) { return m_opis[channel].m_nazwa; }
	char *GetChannelUnit(int channel) { return m_opis[channel].m_jednostka; }
	char *GetName() { return m_nazwa; }
	double GetRate() { return m_rate; }
	void SetRate(double rate) { m_rate=rate; }
	char *GetDateTime(char *buffer, int len);
	CParametrySygnalu *GetSignalParams(int channel) { return m_parametry+channel; }
	COpisKanalu *GetChannel(int channel) { return m_opis+channel; }
	char *GetFileName() { return m_file_name; }
	void SetDateTime(int rok, int miesiac, int dzien, int godzina, int minuta, int sekunda);
	void SetDateTime();
	void GetCoeff(int channel, double *a, double *b) { *a=m_opis[channel].m_wspol; *b=m_opis[channel].m_stala; }
	void GetCoeff(double *a, double *b) { for (int i=0; i<m_channels; i++) { *(a+i)=m_opis[i].m_wspol; *(b+i)=m_opis[i].m_stala; } }
};

//////////////////////////////////////
class CMeasure : public CHeader
{
protected:
	float *m_buffer;
	int m_buffer_size;

public:
	CMeasure() : CHeader() { m_buffer=NULL; m_buffer_size=0; }
	virtual ~CMeasure() {  AllocBuffer(0); }
	virtual void CopyToClipboard(int channel, int from, int to);
	void LoadTextFile(char *name);
	void LoadM11File(char *name);
	void LoadWaveFile(char *name);
	BOOL LoadFile(char *name);
	void SaveTextFile(char *name);
	void SaveM11File(char *name);
	void SaveWaveFile(char *name);
	void SaveFile(char *name);
	void SaveBlock(int channel, int from, int to, char *file_name);
	float *Buffer() { return m_buffer; }
	void AllocBuffer(int dlugosc) { if (m_buffer_size!=dlugosc) { if (m_buffer) { GlobalFree(m_buffer); m_buffer=NULL; m_buffer_size=0; } if (dlugosc) {m_buffer=(float *)GlobalAlloc(GMEM_FIXED,dlugosc); m_buffer_size=dlugosc; } } }
	void policz_pochodna();
	virtual void Erease(int channel, int from, int to);
	void CalcRange(float *min, float *max, float *amp);
	virtual void AddComponent(int /*channel*/, int /*sample*/) {}
	virtual int SaveText(FILE *f);

private:
	void clear_record(BYTE *r, BYTE record_type);
	void calc_stat(TChannelStatistic &st);
	void M11_write_line_1(CFile &f);
	void M11_write_line_2(CFile &f);
	void M11_write_line_3(CFile &f);
	void M11_write_line_4(CFile &f);
	void M11_write_line_kan_1(CFile &f);
	void M11_write_line_kan_2(CFile &f);
	void M11_write_line_kan_3(CFile &f);
	void M11_write_dane(CFile &f);
	int M11_read_line_1(CFile &f, int &n, double &dt);
	int M11_read_line_2(CFile &f);
	int M11_read_line_3(CFile &f);
	int M11_read_line_4(CFile &f);
	int M11_read_line_kan_1(CFile &f);
	int M11_read_line_kan_2(CFile &f);
	int M11_read_line_kan_3(CFile &f);
	int M11_read_dane(CFile &f, int n);
	void SaveM11(CFile &f);
	void LoadM11(CFile &f);
	int LoadText(FILE *f);
	int Load(CFile &file);
	int Save(CFile &file);
};

////////////////////////////////////////
class CMeasureView : public CWnd
{
	// Attributes

protected:
	CMeasure *m_doc;

	int m_range;

	int m_begin_block;
	int m_end_block;
	int m_sel_channel;
	int m_drag_object;	//0=nic, 1=poczatek, 2=koniec
	int m_drag_offset;

	HWND m_old_capture;
	int m_rb_sample;
	int m_rb_channel;
	int m_expanded_channels[MAX_CHANNEL];
	float m_min[MAX_CHANNEL];
	float m_max[MAX_CHANNEL];
	float m_amp[MAX_CHANNEL];

	int m_visible_samples;
	int m_visible_channels;

	int m_plot_style;
	int m_width;
	int m_height;
	int m_channel_height;
	double m_x_scale;

public:
	CMeasureView(int plot_style);
	virtual ~CMeasureView() { }

	virtual int Create(int nX, int nY, int nW, int nH, HWND hParent, DWORD style);
	virtual int OnCreate(LPCREATESTRUCT lpCreateStruct);
	virtual LRESULT OnCommand(WPARAM wParam, LPARAM lParam);
	virtual void OnSize(UINT uType, UINT uWidth, UINT uHeight);
	virtual void OnPaint(HDC hDC);
	virtual void OnVScroll(int /*nScrollCode*/, int /*nPos*/, HWND /*hwndScrollBar*/);
	virtual void OnHScroll(int /*nScrollCode*/, int /*nPos*/, HWND /*hwndScrollBar*/);
	virtual void OnMouseMove(int nKeys, int nXPos, int nYPos);
	virtual void OnLButtonDown(int nKeys, int nXPos, int nYPos);
	virtual void OnLButtonUp(int nKeys, int nXPos, int nYPos);
	virtual void OnRButtonDown(int nKeys, int nXPos, int nYPos);

	void SetPlotStyle(int style) { m_plot_style=style; }
	void SetDocument(CMeasure *doc);
	void DocumentChanged();
	void GetSelection(CSelection *selection);
	void GetState(int *state);
	void GetRightSelection(CSelection *selection);
	void statystyka(int do_obliczenia, int pocz, int kon);
	void expand(int k) { m_expanded_channels[k]=1; }
	void colapse(int k) { m_expanded_channels[k]=0; }
	void SetBlock(int channel, int begin, int end);

private:
	virtual void RefreshScrollBars();
	virtual void UpdateBlockInfo();
	int num_visible();
	void size(POINT &s);
	int klikniety_kanal(int &pos);
	int gora_kanalu(int kanal);
	void CalcRange();
	void grid_step(double &ymin, double &ymax, double &step);
	void xy2sample(int x, int y, int &n);
	void sample2xy(int n, int &x, int &y);
	int pointed_object(int x, int y, int channel);
	int begin_block_pos() { int x,y; sample2xy(m_begin_block,x,y); return x; }
	int end_block_pos() { int x,y; sample2xy(m_end_block,x,y); return x; }
	void set_begin_block(int x) { xy2sample(x,0,m_begin_block); }
	void set_end_block(int x) { xy2sample(x,0,m_end_block); }
	void clear_block_at(int x) { xy2sample(x,0,m_end_block); m_begin_block=m_end_block; }
	void CurrentScrollPos(int &x, int &y);
	void SetScrollPos(int x, int y);
	virtual void PosToPoint(int nXPos, int nYPos, int ch, double &x, double &y);
	virtual int ChangeRange(int direction);
	virtual int ChangeZoomX(int direction);
	virtual int ChangeZoomY(int direction);
	void SetBeginOfBlock();
	void SetEndOfBlock();
	void SelectAll();
	void AddComponent() { if (m_doc) m_doc->AddComponent(m_rb_channel, m_rb_sample); }
};

////////////////////////////////////////
class CMeasureWnd : public CMDIChildWnd
{
protected:
	CMeasureView *m_display;
	CMeasure *m_doc;

public:
	CMeasureWnd();
	~CMeasureWnd();
	virtual int Create(int nX, int nY, int nW, int nH, HWND hParent, DWORD style);
	virtual LRESULT OnCommand(WPARAM wParam, LPARAM lParam);
	virtual int OnCreate(LPCREATESTRUCT lpCreateStruct);
	virtual void OnDestroy();
	virtual void OnSize(UINT uType, UINT uWidth, UINT uHeight);
	virtual LRESULT OnMDIActivate(HWND hLast, HWND hNew);
	virtual void SetDocument(CMeasure *doc) { m_doc=doc; m_display->SetDocument(m_doc); SetTitle(m_doc->GetFileName());}
	CMeasure *GetDocument() { return m_doc; }
	void GetSelection(CSelection *selection) { m_display->GetSelection(selection); }
};


//////////////////////////////////////////////////////
class CParametrySkladowejDlg : public CDialog
{
	CParametrySkladowej *m_params;
	CParametrySkladowej m_new_params;

public:
	CParametrySkladowejDlg(CParametrySkladowej *parametry)  : CDialog() { m_params=parametry; m_new_params=*parametry; }
	int Execute(HWND parent) { return CDialog::Execute(parent, IDD_SIGCOMPOINFO); }
	virtual BOOL OnInitDialog(HWND hWnd);
	virtual BOOL OnOk();
};


//////////////////////////////////////////////////////
class CParametrySygnaluDlg : public CDialog
{
	BOOL m_byly_zmiany;
	CParametrySygnalu *m_params;
	CParametrySygnalu m_new_params;
	CSelection *m_selection;
	CSelection m_new_selection;

public:
	CParametrySygnaluDlg(CParametrySygnalu *params, CSelection *selection)  : CDialog() { m_params=params; m_new_params=*params; m_selection=selection; m_new_selection=*selection; }
	int Execute(HWND parent) { return CDialog::Execute(parent,IDD_SIGNAL); }
	virtual BOOL OnInitDialog(HWND hWnd);
	virtual BOOL OnOk();
	virtual BOOL OnNotify(int nID, NMHDR *lpHdr);
	virtual BOOL OnCommand(int nID, int nNotify, HWND hControl);

private:
	void fill_param(int i);
	void change_settings();
	void create_columns();
	void delete_component();
	void insert_component();
	void insert(int i);
	void copy_all();
};

//////////////////////////////////////////////////////
class CBlockDlg : public CDialog
{
	CSelection *m_selection;
	CSelection m_new_selection;

public:
	CBlockDlg(CSelection *selection)  : CDialog() { m_selection=selection; m_new_selection=*selection; }
	int Execute(HWND parent) { return CDialog::Execute(parent,IDD_BLOCK); }
	virtual BOOL OnInitDialog(HWND hWnd);
	virtual BOOL OnOk();
};

//////////////////////////////////////////////
void statystyka(CStatyData *info, unsigned do_obliczenia);
