#include <windows.h>
#include <commctrl.h>
#include <strsafe.h>
#include <math.h>

#include "app.h"
#include "winfi32.h"
#include "resource.h"
#include "measure.h"

#define SAVEFREE(a) if (a) { GlobalFree((HANDLE)a); a=NULL; }
typedef char *WAVEBUFFER;

////////////////////////////////////////////////////////
//  CFile

int CFile::Open(char *name)
{
	m_handle=CreateFile(name,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,
					  FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if (m_handle==INVALID_HANDLE_VALUE)
		return 1;	// nie udao sie otworzy pliku
	return 0;
}

int CFile::Create(char *name)
{
	m_handle=CreateFile(name,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,
				  FILE_FLAG_SEQUENTIAL_SCAN ,NULL);
	if (m_handle==INVALID_HANDLE_VALUE)
		return 1;	// nie udao sie otworzy pliku
	return 0;
}

int CFile::Read(void *buf, int size)
{
	DWORD odczytal;

	ReadFile(m_handle,buf,size,&odczytal,NULL);
	return odczytal;
}

int CFile::Write(void *buf, int size)
{
	DWORD zapisal;

	WriteFile(m_handle,buf,size,&zapisal,NULL);
	return zapisal;
}

////////////////////////////////////////////////////////
//  CParametrySkladowej

void CParametrySkladowej::Init(char *nazwa)
{
	StringCbCopy(m_name,41,nazwa);
	m_fr=10.0;		//czestotliwosc
	m_A=0.0;		//amplituda cosinusa skladowej do filtracji
	m_D=0.0;		//amplituda sinusa skladowej do filtracji
	m_eta=0.0;		//tlumienie
	m_Vr=100.0;		//wariancja skladowej do filtracji
	m_K=5.0;		//szerokosc filtru dla skladowej
}

////////////////////////////////////////////////////////
//  CParametrySygnalu

void CParametrySygnalu::Init()
{
	int i;
	char nazwa[41];

	m_A0=0.0;
	m_fr0=100.0;		//czestotliwosc glowna * 2*PI
	m_amplituda=100.0;
	m_liczba_skladowych=0;
	for (i=0; i<MAX_OMEGA; i++)
	{
		StringCbPrintf(nazwa,41,"Component no %d",i+1);
		m_skladowe[i].Init(nazwa);
	}
}

void CParametrySygnalu::DeleteComponent(int skladnik)
{
	if (skladnik>=0 && skladnik<m_liczba_skladowych)
	{
		m_liczba_skladowych--;
		while (skladnik<m_liczba_skladowych)
		{
			m_skladowe[skladnik]=m_skladowe[skladnik+1];
			skladnik++;
		}
	}
}

void CParametrySygnalu::InsertComponent(int skladnik,CParametrySkladowej *parametry)
{
	int i;

	if (m_liczba_skladowych<MAX_OMEGA)
	{
		if (skladnik>=0 && skladnik<m_liczba_skladowych)
		{
			for (i=m_liczba_skladowych;i>=skladnik;i--)
				m_skladowe[i+1]=m_skladowe[i];
		}
		m_liczba_skladowych++;
		m_skladowe[skladnik]=*parametry;
	}
}

void CParametrySygnalu::AppendComponent(CParametrySkladowej *parametry)
{
	if (m_liczba_skladowych<MAX_OMEGA)
	{
		m_skladowe[m_liczba_skladowych]=*parametry;
		m_liczba_skladowych++;
	}
}

////////////////////////////////////////////////////////
//  COpisKanalu

void COpisKanalu::Init(double a, double b, const char *unit, const char *name)
{
	m_wspol=a;
	StringCbCopy(m_jednostka,8,unit);
	StringCbCopy(m_nazwa,41,name);
	m_stala=b;
}


BOOL COpisKanalu::Load(CFile &file)
{
	unsigned char funit_len;

	if ( file.Read(&m_wspol, 8) != 8 ) return FALSE;
	if ( file.Read(&funit_len, 1) != 1 ) return FALSE;
	if ( funit_len<0 || funit_len>7 ) return FALSE;
	if ( file.Read(m_jednostka, 7) != 7 ) return FALSE;
	m_jednostka[funit_len]='\0';
	if ( file.Read(&funit_len, 1) != 1 ) return FALSE;
	if ( funit_len<0 || funit_len>40 ) return FALSE;
	if ( file.Read(m_nazwa, 40) != 40 ) return FALSE;
	m_nazwa[funit_len]='\0';
	if ( file.Read(&m_stala, 8) != 8 ) return FALSE;
	if ( file.Read(&m_user1, 8) != 8 ) return FALSE;
	if ( file.Read(&m_user2, 8) != 8 ) return FALSE;
	if ( file.Read(&m_user3, 8) != 8 ) return FALSE;
	if ( file.Read(&m_user4, 8) != 8 ) return FALSE;
	return TRUE;
}

BOOL COpisKanalu::LoadFactor(CFile &file)
{
	if ( file.Read(&m_wspol, 8) != 8 )
		return FALSE;
	m_jednostka[0]='\0';
	m_nazwa[0]='\0';
	m_stala=0.0;
	m_user1=0.0;
	m_user2=0.0;
	m_user3=0.0;
	m_user4=0.0;
	return TRUE;
}



BOOL COpisKanalu::Save(CFile &file)
{
	unsigned funit_len;

	if ( file.Write(&m_wspol, 8) != 8 ) return FALSE;
	funit_len=strlen(m_jednostka);
	if ( file.Write(&funit_len, 1) != 1 ) return FALSE;
	if ( file.Write(m_jednostka, 7) != 7 ) return FALSE;
	funit_len=strlen(m_nazwa);
	if ( file.Write(&funit_len, 1) != 1 ) return FALSE;
	if ( file.Write(m_nazwa, 40) != 40 ) return FALSE;
	if ( file.Write(&m_stala, 8) != 8 ) return FALSE;
	if ( file.Write(&m_user1, 8) != 8 ) return FALSE;
	if ( file.Write(&m_user2, 8) != 8 ) return FALSE;
	if ( file.Write(&m_user3, 8) != 8 ) return FALSE;
	if ( file.Write(&m_user4, 8) != 8 ) return FALSE;
	return TRUE;
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFiltr::CFiltr()
{
	m_wsk=0;
	m_liczba_wpisow=0;
}

void CFiltr::AddSample(double a)
{
	if (++m_wsk>=LICZBA_PROBEK)
		m_wsk-=LICZBA_PROBEK;
	m_probki[m_wsk]=a;
	m_liczba_wpisow++;
}

double CFiltr::wartosc()
{
	double a;
	int i,j;

	i=m_wsk;
	j=i-4;
	if (j<0)
		j+=LICZBA_PROBEK;
	a=-(m_probki[i--]+m_probki[j++])*3;
	if (i<0) i+=LICZBA_PROBEK;
	if (j>=LICZBA_PROBEK) j-=LICZBA_PROBEK;
	a+=(m_probki[i--]+m_probki[j])*12;
	if (i<0) i+=LICZBA_PROBEK;
	a+=m_probki[i]*17;
	return a/35.0;
}

double CFiltr::pochodna()
{
	double a;
	int i,j;

	i=m_wsk;
	j=i-4;
	if (j<0) j+=LICZBA_PROBEK;
	a=2*(m_probki[i--]-m_probki[j++]);
	if (i<0) i+=LICZBA_PROBEK;
	if (j>=LICZBA_PROBEK) j-=LICZBA_PROBEK;
	a+=m_probki[i]-m_probki[j];
	return a/10.0;
}

double CFiltr::ekstremum(double x)
{
	double a0,a1,a2;
	int i;

	i=m_wsk-2;
	if (i<0) i+=LICZBA_PROBEK;
	a0=-m_probki[i]*3;
	a1=m_probki[i];
	a2=m_probki[i]*5;
	if (--i<0) i+=LICZBA_PROBEK;
	a0+=m_probki[i]*9;
	a1+=m_probki[i]*7;
	a2-=m_probki[i]*5;
	if (--i<0) i+=LICZBA_PROBEK;
	a0+=m_probki[i]*11;
	a1+=m_probki[i]*3;
	a2-=m_probki[i]*5;
	if (--i<0) i+=LICZBA_PROBEK;
	a0+=m_probki[i]*3;
	a1-=m_probki[i]*11;
	a2+=m_probki[i]*5;
	return (a0 + x*(a1 + x*a2))/20;
}

double CFiltr::miejsce_zerowe()
{
	double a0,a1;
	int i;

	i=m_wsk;
	a0=m_probki[i];
	a1=m_probki[i]*3;
	if (--i<0) i+=LICZBA_PROBEK;
	a0+=m_probki[i]*2;
	a1+=m_probki[i];
	if (--i<0) i+=LICZBA_PROBEK;
	a0+=m_probki[i]*3;
	a1-=m_probki[i];
	if (--i<0) i+=LICZBA_PROBEK;
	a0+=m_probki[i]*4;
	a1-=m_probki[i]*3;
	return -a0/a1;
}

int CFiltr::jest_zero()
{
	double a0,a1;
	int i;

	i=m_wsk;
	a0=m_probki[i];
	a1=m_probki[i]*4;
	if (--i<0) i+=LICZBA_PROBEK;
	a0+=m_probki[i]*2;
	a1+=m_probki[i]*3;
	if (--i<0) i+=LICZBA_PROBEK;
	a0+=m_probki[i]*3;
	a1+=m_probki[i]*2;
	if (--i<0) i+=LICZBA_PROBEK;
	a0+=m_probki[i]*4;
	a1+=m_probki[i];
	return a0*a1<0.0 || (a0==0.0 && a1!=0.0);
}

double CFiltr::od_konca(int i)
{
	i=m_wsk-i;
	if (i<0) i+=LICZBA_PROBEK;
	return m_probki[i];
}


////////////////////////////////////////////////////////
// ustalenie czstotliwoci prbkowania odbywa si na
// podstawie rnicy pierwszej kolumny dwch pierwszych wierszy
// ustalenie liczby kanaw odbywa si na podstawie
// iloci wierszy. strumie jest otwarty i jest ustawiony
// na pocztku

char *CHeader::GetDateTime(char *buffer, int len)
{
	int rok=((m_rec_date>>25) & 0x7F) + 1980;
	int miesiac=(m_rec_date>>21) & 0x0F;
	int dzien=(m_rec_date>>16) & 0x1F;
	int godzina=(m_rec_date>>11) & 0x1F;
	int minuta=(m_rec_date>>5) & 0x3F;
	int sekunda=(m_rec_date<<1) & 0x3F;
	StringCbPrintf(buffer,len,"%04d-%02d-%02d at %02d:%02d:%02d",rok,miesiac,dzien,godzina,minuta,sekunda);
	return buffer;
}

////////////////////////////////////////////////////////
// ustalenie czstotliwoci prbkowania odbywa si na
// podstawie rnicy pierwszej kolumny dwch pierwszych wierszy
// ustalenie liczby kanaw odbywa si na podstawie
// iloci wierszy. strumie jest otwarty i jest ustawiony
// na pocztku

void CHeader::SetDateTime(int rok, int miesiac, int dzien, int godzina, int minuta, int sekunda)
{
	m_rec_date=((sekunda & 0x3F)>>1) | ((minuta & 0x3F)<<5) | ((godzina & 0x1F)<<11) |
			((dzien & 0x1F)<<16) | ((miesiac & 0x0F)<<21) | (((rok-1980) & 0x1F)<<25);
}

////////////////////////////////////////////////////////
// ustalenie czstotliwoci prbkowania odbywa si na
// podstawie rnicy pierwszej kolumny dwch pierwszych wierszy
// ustalenie liczby kanaw odbywa si na podstawie
// iloci wierszy. strumie jest otwarty i jest ustawiony
// na pocztku

void CHeader::SetDateTime()
{
	SYSTEMTIME t;
	GetLocalTime(&t);
	SetDateTime(t.wYear,t.wMonth,t.wDay,t.wHour,t.wMinute,t.wSecond);
}

////////////////////////////////////////////////////////
// ustalenie czstotliwoci prbkowania odbywa si na
// podstawie rnicy pierwszej kolumny dwch pierwszych wierszy
// ustalenie liczby kanaw odbywa si na podstawie
// iloci wierszy. strumie jest otwarty i jest ustawiony
// na pocztku

#define MAX_LINE_LEN 512
static const char *format_linii="%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f";

BOOL CMeasure::LoadText(FILE *f)
{
	char linia[MAX_LINE_LEN];
	float *dest;
	int n,i;
	float t1,t2;
	float a[16];

	//czytanie pierwszego czasu
	if (!fgets(linia,MAX_LINE_LEN,f))
		return FALSE;			//plik pusty albo bd czytania
	m_samples=1;
	m_channels=sscanf(linia,format_linii,&t1,a,a+1,a+2,a+3,a+4,a+5,a+6,a+7,a+8,a+9,a+10,a+11,a+12,a+13,a+14,a+15);
	if (m_channels<2) return FALSE;		//musz by minimum dwie kolumny
	--m_channels;

	//czytanie drugiego czasu i poczliczenie czstotliwoci prbkowania
	if (!fgets(linia,MAX_LINE_LEN,f)) return FALSE;			//jest tylko jedna linia
	m_samples=2;
	n=sscanf(linia,format_linii,&t2,a,a+1,a+2,a+3,a+4,a+5,a+6,a+7,a+8,a+9,a+10,a+11,a+12,a+13,a+14,a+15);
	if (n-1!=m_channels) return FALSE;
	m_rate=1.0/(t2-t1);

	//czytanie pozostaych linii w celu ustalenia liczby prbek
	while (fgets(linia,MAX_LINE_LEN,f))
		++m_samples;

	//inicjowanie pozostaych pl
	m_format=5;
	m_rec_date=0;
	StringCbPrintf(m_nazwa,61,"Pomiar");
	for (i=0; i<(int)m_channels; i++)
	{
		StringCbPrintf(linia,MAX_LINE_LEN,"Channel %d",i+1);
		m_opis[i].Init(1,0,"?",linia);
		m_parametry[i].Init();
	}

	//alokowanie miejsca w pamici
	AllocBuffer(m_channels*8*m_samples);

	//czytanie
	dest=m_buffer;
	fseek(f,0,SEEK_SET);
	while (fgets(linia,MAX_LINE_LEN,f))
	{
		n=sscanf(linia,format_linii,&t1,a,a+1,a+2,a+3,a+4,a+5,a+6,a+7,a+8,a+9,a+10,a+11,a+12,a+13,a+14,a+15);
		if (n-1!=m_channels) return FALSE;
		for (i=0;i<m_channels;i++)
		{
			*(dest++)=a[i];
			*(dest++)=0;
		}
	}
	return TRUE;
}

////////////////////////////////////////////////////////
BOOL CMeasure::SaveText(FILE *f)
{
	float *dest;
	int n,i;

	if (m_buffer)
	{
		dest=m_buffer;
		for (i=0;i<m_samples;i++)
		{
			fprintf(f,"%6e",i/m_rate);
			for (n=0;n<m_channels;n++)
			{
				fprintf(f,"\t%6e",*dest);
				dest+=2;
			}
			fprintf(f,"\n");
		}
	}
	return TRUE;
}

////////////////////////////////////////////////////////
BOOL CMeasure::Load(CFile &file)
{
	unsigned char title_len;
	int dlugosc_pomiaru;
	int i,j;
	double a[MAX_CHANNEL], b[MAX_CHANNEL];
	short int *buforek;
	float *dest;
	BOOL retval=TRUE;

	//----- cz wsplna dla wszystkich formatw -----
	if ( file.Read(&m_rate, 8) != 8 ) return FALSE;
	if ( m_rate<0.0 || m_rate>330000.0 ) return FALSE;
	m_channels=0;
	if ( file.Read(&m_channels, 1) != 1 ) return FALSE;
	if ( file.Read(&m_format, 1) != 1 ) return FALSE;
	if ( m_format<0 || m_format>7 ) return FALSE;
	if ( file.Read(&m_samples, 4) != 4 ) return FALSE;
	if ( m_samples<0 || m_samples>2*1024*1024 ) return FALSE;

	//----- nagwki poszczeglnych kanaw -----
	if (m_format==0)
	{
		//stary naglowek mia 16 liczba double z mnonikami bez
		//wzgldu na liczb mierzonych kanaw
		m_rec_date=0;
		m_nazwa[0]='\0';
		for (i=0; i<16; i++)
			m_opis[i].LoadFactor(file);
	}
	else
	{
		//nowy naglowek mia dodatkowe pola z dat pomiaru i nazw
		//potem byo tyle nagwkw kanaw ile mierzono kanaw
		if ( file.Read(&m_rec_date, 4) != 4 ) return FALSE;
		if ( file.Read(&title_len, 1) != 1 ) return FALSE;
		if ( title_len<0 || title_len>60 ) return FALSE;
		if ( file.Read(m_nazwa, 60) != 60 ) return FALSE;
		m_nazwa[title_len]='\0';
		for (i=0; i<(int)m_channels; i++)
			if (!m_opis[i].Load(file)) return FALSE;
	}

	//inicjowanie opisw kanaw
	for (i=0; i<(int)m_channels; i++)
		m_parametry[i].Init();

	//----- rezerwowanie miejsca na dane pomiarowe -----
	AllocBuffer(m_channels*2*sizeof(float)*m_samples);
	dest=m_buffer;
	dlugosc_pomiaru=m_channels*m_samples;

	//----- czytanie danych pomiarowych -----
	switch (m_format)
	{
	case 0:
	case 2:
		//dane s liczbami typu int, trzeba je przemnoy przez odpowiedni wspczynnik
		//najpierw jest cay pierwszy kana, potem cay drugi kana i tak dalej.
		buforek=new short[m_samples];
		GetCoeff(a,b);
		for (i=0; i<m_channels && retval; i++)
		{
			if ( file.Read(buforek, m_samples<<1) == (m_samples<<1) )
			{
				for (j=0;j<m_samples;j++)
				{
					dest[(j*m_channels+i)*2]=(float)(buforek[j]*a[i]);		//realis
					dest[(j*m_channels+i)*2+1]=(float)0;					//imaginaris
				}
			}
			else
				retval=FALSE;
		}
		delete []buforek;
		break;

	case 3:
		//dane s liczbami typu int, trzeba je przemnoy przez odpowiedni wspczynnik i doda sta
		//najpierw jest cay pierwszy kana, potem cay drugi kana i tak dalej.
		buforek=new short[m_samples];
		GetCoeff(a,b);
		for (i=0; i<m_channels && retval; i++)
		{
			if ( file.Read(buforek, m_samples<<1) == (m_samples<<1) )
			{
				for (j=0;j<m_samples;j++)
				{
					dest[(j*m_channels+i)*2]=(float)(buforek[j]*a[i]+b[i]);		//realis
					dest[(j*m_channels+i)*2+1]=(float)0;							//imaginaris
				}
			}
			else
				retval=FALSE;
		}
		delete []buforek;
		break;

	case 4:
		GetCoeff(a,b);
		buforek=(short *)dest;
		buforek+=m_channels*4*m_samples;		//jest na kocu
		buforek-=m_channels*m_samples;			//cofam si o wiartk
		if ( file.Read(buforek, m_buffer_size>>2) == (m_buffer_size>>2) )
		{
			for (j=0;j<m_samples;j++)
			{
				for (i=0; i<m_channels; i++)
				{
					dest[(j*m_channels+i)*2]=(float)(buforek[j*m_channels+i]*a[i]+b[i]);		//realis
					dest[(j*m_channels+i)*2+1]=(float)0;							//imaginaris
				}
			}
		}
		else
			retval=FALSE;
		break;

	case 5:
		if ( file.Read(dest+dlugosc_pomiaru, dlugosc_pomiaru<<2) == dlugosc_pomiaru<<2 )	//czytam floaty
		{
			for (i=0; i<dlugosc_pomiaru; i++)
			{
				dest[i*2]=dest[i+dlugosc_pomiaru];			//realis
				dest[i*2+1]=(float)0;		//imaginaris
			}
		}
		else
			retval=FALSE;
		break;

	case 6:
		if ( file.Read(dest, m_buffer_size) != m_buffer_size )
			retval=FALSE;
		break;
	}

	if (m_format<5) m_format=5;
	return retval;
}


BOOL CMeasure::Save(CFile &file)
{
	unsigned title_len;
	int dlugosc_pomiaru;
	int i;
	BOOL retval=TRUE;

	if (!m_buffer) return TRUE;
	if ( file.Write(&m_rate, 8) != 8 ) return FALSE;
	if ( file.Write(&m_channels, 1) != 1 ) return FALSE;
	if ( file.Write(&m_format, 1) != 1 ) return FALSE;
	if ( file.Write(&m_samples, 4) != 4 ) return FALSE;
	if ( file.Write(&m_rec_date, 4) != 4 ) return FALSE;
	title_len=strlen(m_nazwa);
	if ( file.Write(&title_len, 1) != 1 ) return FALSE;
	if ( file.Write(m_nazwa, 60) != 60 ) return FALSE;
	for (i=0; i<(int)m_channels; i++)
		m_opis[i].Save(file);

	switch (m_format)
	{
	case 5:
		dlugosc_pomiaru=m_channels*m_samples*2;
		for (i=0;i<dlugosc_pomiaru && retval;i+=2)
			if ( file.Write(m_buffer+i, 4) != 4 ) retval=FALSE;
		break;
	
	case 6:
		dlugosc_pomiaru=m_channels*8*m_samples;
		if ( file.Write(m_buffer, dlugosc_pomiaru) != dlugosc_pomiaru ) retval=FALSE;
		break;
	}

	return retval;
}


void CMeasure::LoadTextFile(char *name)
{
	//otwiera plik do odczytu i
	FILE *f;

	f=fopen(name,"rt");
	if (f)
	{
		LoadText(f);
		fclose(f);
		StringCbCopy(m_file_name,256,name);
	}
}


BOOL CMeasure::LoadFile(char *name)
{
	//otwiera plik do odczytu i
	CFile f;

	f.Open(name);
	BOOL r=Load(f);
	f.Close();
	StringCbCopy(m_file_name,256,name);
	return r;
}


void CMeasure::LoadM11File(char *name)
{
	//otwiera plik do odczytu i
	CFile f;

	f.Open(name);
	LoadM11(f);
	f.Close();
	StringCbCopy(m_file_name,256,name);
}


void CMeasure::SaveTextFile(char *name)
{
	//otwiera plik do zapisu
	FILE *f;

	f=fopen(name,"wt");
	if (f)
	{
		SaveText(f);
		fclose(f);
	}
}


void CMeasure::SaveFile(char *name)
{
	//otwiera plik do odczytu i
	CFile f;

	f.Create(name);
	Save(f);
	f.Close();
	StringCbCopy(m_file_name,256,name);
}


void CMeasure::SaveM11File(char *name)
{
	//otwiera plik do odczytu i
	CFile f;

	f.Create(name);
	SaveM11(f);
	f.Close();
	StringCbCopy(m_file_name,256,name);
}

//////////////////////////////////////////////////////////
void CMeasure::clear_record(BYTE *r, BYTE record_type)
{
	int i;

	*(r++)=record_type;
	for (i=1;i<M11_RECORD_SIZE-1;i++)
		*(r++)=0;
	*r=0x7F;
}

void CMeasure::calc_stat(TChannelStatistic &st)
{
	int i;
	double sigma;
	float *points;

	if (!m_buffer) return;
	points=m_buffer;
	st.poz_max=0;
	for (i=1;i<m_samples;i++)
		if (points[i]>points[st.poz_max])
			st.poz_max=i;
	st.max=points[st.poz_max];

	st.poz_min=0;
	for (i=1;i<m_samples;i++)
		if (points[i]<points[st.poz_min])
			st.poz_min=i;
	st.min=points[st.poz_min];

	st.mean=points[0];
	for (i=1;i<m_samples;i++)
		st.mean += points[i];
	st.mean/=m_samples;

	st.war=sqr(points[0]);
	for (i=1;i<m_samples;i++)
		st.war += sqr(points[i]);
	st.war/=m_samples;

	sigma=sqr(points[0]-st.mean);
	for (i=1;i<m_samples;i++)
		sigma += sqr(points[i]-st.mean);
	sigma/=m_samples;

	st.acor=points[0]*points[1];
	for (i=1;i<m_samples-1;i++)
		st.acor += points[i]*points[i+1];
	st.acor /= m_samples-1;
	st.acor /= sigma;
}

void CMeasure::M11_write_line_1(CFile &f)
{
	TM11Record r;

	clear_record(r, 2);
	r[1] = 1;
	r[69] = 2;
	*(double *)(r+81) = m_samples;           //liczba probek
	*(double *)(r+89) = 0.0444375;   //nieznane
	*(double *)(r+97) = 0;           //start time
	*(double *)(r+105) = 1.0/m_rate;         //time step
	f.Write(r,M11_RECORD_SIZE);
}

void CMeasure::M11_write_line_2(CFile &f)
{
	TM11Record r;
	int i,n;

	clear_record(r, 5);
	n=strlen(m_nazwa);
	for (i=0;i<n;i++)
		r[1+i]=m_nazwa[i];
	r[69]=2;
	f.Write(r,M11_RECORD_SIZE);
}

static const char *g_jednostka_czasu="Sec";

void CMeasure::M11_write_line_3(CFile &f)
{
	TM11Record r;
	int i,n;

	clear_record(r, 5);
	n=strlen(g_jednostka_czasu);
	for (i=0;i<n;i++)
		r[1+i]=g_jednostka_czasu[i];
	r[69]=2;
	f.Write(r,M11_RECORD_SIZE);
}

void CMeasure::M11_write_line_4(CFile &f)
{
	TM11Record r;
	int i,n;

	clear_record(r, 0);
	*(WORD *)(r+1) = 30000;    //object type
	n=strlen(m_nazwa);
	for (i=0;i<n;i++)
		r[3+i]=m_nazwa[i];
	r[69]=2;
	f.Write(r,M11_RECORD_SIZE);
}

void CMeasure::M11_write_line_kan_1(CFile &f)
{
	TM11Record r;
	int i,n;
	TChannelStatistic st;

	clear_record(r, 1);
	*(WORD *)(r+1) = 30000;    //object type
	*(WORD *)(r+3) = 1;        //nieznane
	n=strlen(m_opis[0].m_nazwa);
	for (i=0;i<n;i++)
		r[5+i]=m_opis[0].m_nazwa[i];
	calc_stat(st);
	*(double *)(r+46) = st.min;
	*(double *)(r+54)  = st.poz_min;
	*(double *)(r+62) = st.max;
	*(double *)(r+70) = st.poz_max;
	r[78]=2;
	*(double *)(r+88) = st.mean;
	*(double *)(r+96) = st.war;
	*(double *)(r+104) = st.acor;
	f.Write(r,M11_RECORD_SIZE);
}

void CMeasure::M11_write_line_kan_2(CFile &f)
{
	TM11Record r;

	clear_record(r, 3);
	r[78]=2;
	f.Write(r,M11_RECORD_SIZE);
}

void CMeasure::M11_write_line_kan_3(CFile &f)
{
	TM11Record r;

	clear_record(r, 4);
	*(double *)(r+1) = 1.0;    //time scale
	f.Write(r,M11_RECORD_SIZE);
}

void CMeasure::M11_write_dane(CFile &f)
{
	TM11Record r;
	int i;
	long poz;
	float *points;
	int n=2*m_channels;
	char *jed;

	if (m_buffer)
	{
		points=m_buffer;
		i=0;
		clear_record(r, 7);
		for (poz=0; poz<m_samples; poz++)
		{
			jed=m_opis[poz%m_channels].m_jednostka;
			if (jed[0]=='c' && jed[1]=='m') *(double *)(r+1+8*i) = *points /100.0; else *(double *)(r+1+8*i) = *points;
			points+=n;
			++i;
			if (i==15)
			{
				f.Write(r,M11_RECORD_SIZE);
				clear_record(r, 7);
				i=0;
			}
		}
		if (i>0)
			f.Write(r,M11_RECORD_SIZE);
	}
}
////////////////////////////////////////////////
int CMeasure::M11_read_line_1(CFile &f, int &n, double &dt)
{
	TM11Record r;

	f.Read(r,M11_RECORD_SIZE);
	if (r[0]!=2) return 1;
	if (r[M11_RECORD_SIZE-1]!=0x7F) return 2;
	if (r[1] != 1) return 3;
	if (r[69] != 2) return 4;
	n = (int)floor( *(double *)(r+81) + 0.5);
	dt = *(double *)(r+105);
	return 0;
}

int CMeasure::M11_read_line_2(CFile &f)
{
	TM11Record r;

	f.Read(r,M11_RECORD_SIZE);
	if (r[0]!=5) return 5;
	if (r[69]!=2) return 6;
	return 0;
}

int CMeasure::M11_read_line_3(CFile &f)
{
	TM11Record r;

	f.Read(r,M11_RECORD_SIZE);
	if (r[0]!=5) return 7;
	if (r[69]!=2) return 8;
	return 0;
}

int CMeasure::M11_read_line_4(CFile &f)
{
	TM11Record r;

	f.Read(r,M11_RECORD_SIZE);
	if (r[0]!=0) return 9;
	if ( *(WORD *)(r+1) != 30000 ) return 10;
	if (r[69]!=2) return 11;
	return 0;
}

int CMeasure::M11_read_line_kan_1(CFile &f)
{
	TM11Record r;

	f.Read(r,M11_RECORD_SIZE);
	if (r[0]!=1) return 12;
	if ( *(WORD *)(r+1) != 30000 ) return 13;
	if ( *(WORD *)(r+3) != 1 ) return 14;
	if ( r[78]!=2 ) return 15;
	return 0;
}

int CMeasure::M11_read_line_kan_2(CFile &f)
{
	TM11Record r;

	f.Read(r,M11_RECORD_SIZE);
	if (r[0]!=3) return 16;
	if ( r[78]!=2 ) return 17;
	return 0;
}

int CMeasure::M11_read_line_kan_3(CFile &f)
{
	TM11Record r;

	f.Read(r,M11_RECORD_SIZE);
	if (r[0]!=4) return 18;
	if ( *(double *)(r+1) != 1.0 ) return 19;
	return 0;
}

int CMeasure::M11_read_dane(CFile &f, int n)
{
	TM11Record r;
	int i;
	double a;

	AllocBuffer(m_channels*8*n);
	float *points=m_buffer;

	m_samples=0;
	i=0;
	f.Read(r,M11_RECORD_SIZE);
	if (r[0]!=7) return 20;
	while (n--)
	{
		a = *(double *)(r+1+8*i);
		*(points++)=(float)a;
		*(points++)=(float)0;		//brak kosygnau
		*(points++)=(float)0;		//brak pochodnej
		*(points++)=(float)0;		//brak kosygnau pochodnej
		++m_samples;
		++i;
		if (i==15)
		{
			f.Read(r,M11_RECORD_SIZE);
			if (r[0]!=7) return 20;
			i=0;
		}
	}
	return 0;
}

void CMeasure::SaveM11(CFile &f)
{
	M11_write_line_1(f);
	M11_write_line_2(f);
	M11_write_line_3(f);
	M11_write_line_4(f);
	M11_write_line_kan_1(f);
	M11_write_line_kan_2(f);
	M11_write_line_kan_3(f);
	M11_write_dane(f);
}

void CMeasure::policz_pochodna()
{
	int i,n=2*m_channels;
	double a1,a2,det,sx0y,sx1y,sx2y,sx0,sx2,sx4;
	double dt=1.0/m_rate;
	float *points;

	if (!m_buffer) return;
	points=m_buffer;

	//--------------------------------------------------
	sx0=5.0;
	sx2=10*dt*dt;
	sx4=34*dt*dt*dt*dt;
	det=(sx0*sx4-sx2*sx2);

	//--------------------------------------------------
	i=2;
	sx0y=points[(i-2)*n]+points[(i-1)*n]+points[i*n]+points[(i+1)*n]+points[(i+2)*n];
	sx1y=dt*(-2*points[(i-2)*n]-points[(i-1)*n]+points[(i+1)*n]+2*points[(i+2)*n]);
	sx2y=dt*dt*(4*points[(i-2)*n]+points[(i-1)*n]+points[(i+1)*n]+4*points[(i+2)]);
	a1=sx1y/sx2;
	a2=(sx0*sx2y-sx2*sx0y)/det;
	points[(i-2)*n+2]=(float)(a1+2*a2*(-2*dt));
	points[(i-1)*n+2]=(float)(a1+2*a2*(-1*dt));
	points[i*n+2]=(float)(a1+2*a2*(-0*dt));
	for (i=3; i<m_samples-3; i++)
	{
		sx1y=dt*(-2*points[(i-2)*n]-points[(i-1)*n]+points[(i+1)*n]+2*points[(i+2)*n]);
		a1=sx1y/sx2;
		points[i*n+2]=(float)a1;
	}

	i=m_samples-3;
	sx0y=points[(i-2)*n]+points[(i-1)*n]+points[i*n]+points[(i+1)*n]+points[(i+2)*n];
	sx1y=dt*(-2*points[(i-2)*n]-points[(i-1)*n]+points[(i+1)*n]+2*points[(i+2)*n]);
	sx2y=dt*dt*(4*points[(i-2)*n]+points[(i-1)*n]+points[(i+1)*n]+4*points[(i+2)*n]);
	a1=sx1y/sx2;
	a2=(sx0*sx2y-sx2*sx0y)/det;
	points[i*n+2]=(float)(a1+2*a2*0*dt);
	points[(i+1)*n+2]=(float)(a1+2*a2*1*dt);
	points[(i+2)*n+2]=(float)(a1+2*a2*2*dt);

	//--------------------------------------------------
	++points;
	i=2;
	sx0y=points[(i-2)*n]+points[(i-1)*n]+points[i*n]+points[(i+1)*n]+points[(i+2)*n];
	sx1y=dt*(-2*points[(i-2)*n]-points[(i-1)*n]+points[(i+1)*n]+2*points[(i+2)*n]);
	sx2y=dt*dt*(4*points[(i-2)*n]+points[(i-1)*n]+points[(i+1)*n]+4*points[(i+2)*n]);
	a1=sx1y/sx2;
	a2=(sx0*sx2y-sx2*sx0y)/det;
	points[(i-2)*n+2]=(float)(a1+2*a2*(-2*dt));
	points[(i-1)*n+2]=(float)(a1+2*a2*(-1*dt));
	points[i*n+2]=(float)(a1+2*a2*(-0*dt));
	for (i=3; i<m_samples-3; i++)
	{
		sx1y=dt*(-2*points[(i-2)*n]-points[(i-1)*n]+points[(i+1)*n]+2*points[(i+2)*n]);
		a1=sx1y/sx2;
		points[i*n+2]=(float)a1;
	}

	i=m_samples-3;
	sx0y=points[(i-2)*n]+points[(i-1)*n]+points[i*n]+points[(i+1)*n]+points[(i+2)*n];
	sx1y=dt*(-2*points[(i-2)*n]-points[(i-1)*n]+points[(i+1)*n]+2*points[(i+2)*n]);
	sx2y=dt*dt*(4*points[(i-2)*n]+points[(i-1)*n]+points[(i+1)*n]+4*points[(i+2)*n]);
	a1=sx1y/sx2;
	a2=(sx0*sx2y-sx2*sx0y)/det;
	points[i*n+2]=(float)(a1+2*a2*0*dt);
	points[(i+1)*n+2]=(float)(a1+2*a2*1*dt);
	points[(i+2)*n+2]=(float)(a1+2*a2*2*dt);
}

void CMeasure::LoadM11(CFile &f)
{
	int n;
	double dt;

	if (M11_read_line_1(f,n,dt)) return;
	if (M11_read_line_2(f)) return;
	if (M11_read_line_3(f)) return;
	if (M11_read_line_4(f)) return;
	if (M11_read_line_kan_1(f)) return;
	if (M11_read_line_kan_2(f)) return;
	if (M11_read_line_kan_3(f)) return;
	m_channels=2;
	m_rate=1.0/dt;
	if (M11_read_dane(f,n)) return;

	//inicjowanie opisw kanaw
	m_format=5;  //rownolege floaty
	m_rec_date=0;
	StringCbCopy(m_nazwa,61,"Przemieszczenie klapy z pliku M11");
	
	m_opis[0].Init(1.0, 0.0, "m", "Przemieszczenie klapy");
	m_opis[1].Init(1.0, 0.0, "m/s", "Prdko przemieszczenia klapy");
	m_parametry[0].Init();
	m_parametry[1].Init();
	policz_pochodna();

	//normalizacja
	//m_max_abs=fabs(m_points[0]);
	//for (i=1;i<m_number_of_points;i++)
	//	if (fabs(m_points[i])>m_max_abs)
	//		m_max_abs=fabs(m_points[i]);
}

void CMeasure::LoadWaveFile(char *name)
{
	HMMIO          f;
	MMCKINFO       parentchunk;
	MMCKINFO       subchunk;
	DWORD          format_size;
	LPWAVEFORMATEX format;
	WAVEBUFFER data;
	DWORD size;
	// Open the given file for reading using buffered I/O.
	
	f = mmioOpen(name, NULL, MMIO_READ | MMIO_ALLOCBUF);
	if(!f)
	{
		DebugOut("Blad otwarcia mmioOpen()\n");
		return;
	}
	
	// Locate a 'RIFF' chunk with a 'WAVE' form type
	// to make sure it's a WAVE file.
	
	parentchunk.fccType = mmioFOURCC('W', 'A', 'V', 'E');
	if (mmioDescend(f, &parentchunk, NULL, MMIO_FINDRIFF))
	{
		DebugOut("Blad poszukiwania 'WAVE' mmioDescend()\n");
		mmioClose(f, 0);
		return;
	}
	
	// Now, find the format chunk (form type 'fmt '). It should be
	// a subchunk of the 'RIFF' parent chunk.
	
	subchunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
	if (mmioDescend(f, &subchunk, &parentchunk,MMIO_FINDCHUNK))
	{
		DebugOut("Blad poszukiwania 'fmt ' mmioDescend()\n");
		mmioClose(f, 0);
		return;
	}
	
	// Get the size of the format chunk, allocate and lock memory for it.
	
	format_size = subchunk.cksize;
	format = (WAVEFORMATEX *)GlobalAlloc(GMEM_FIXED, LOWORD(format_size));
	if (!format)
	{
		DebugOut("Brak pamieci dla WAVEFORMATEX\n");
		mmioClose(f, 0);
		return;
	}
	
	// Read the format chunk.
	
	if (mmioRead(f, (HPSTR) format, format_size) != (LONG) format_size)
	{
		DebugOut("Blad odczytu naglowka mmioRead()\n");
		SAVEFREE( format )
		mmioClose(f, 0);
		return;
	}
	
	// Make sure it's a PCM file.
	
	if (format->wFormatTag != WAVE_FORMAT_PCM)
	{
		DebugOut("To nie jest WAVE_FORMAT_PCM\n");
		SAVEFREE( format )
		mmioClose(f, 0);
		return;
	}
	
	m_channels=format->nChannels;
	m_rate=format->nSamplesPerSec;
	m_samples=0;		//naraze 0, okae si w czasie czytania
	m_format=5;
	m_rec_date=0;
	StringCbPrintf(m_nazwa,61,"Pomiar");
	for (int i=0; i<m_channels; i++)
	{
		char linia[MAX_LINE_LEN];
		StringCbPrintf(linia,MAX_LINE_LEN,"Channel %d",i+1);
		m_opis[i].Init(1,0,"?",linia);
		m_parametry[i].Init();
	}
	// Ascend out of the format subchunk. ???
	
	mmioAscend(f, &subchunk, 0);
	
	// Find the data subchunk.
	
	subchunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
	if (mmioDescend(f, &subchunk, &parentchunk, MMIO_FINDCHUNK))
	{
		DebugOut("Blad poszukiwania 'data' mmioDescend()\n");
		SAVEFREE( format )
		mmioClose(f, 0);
		return;
	}
	
	// Get the size of the data subchunk.
	
	size = subchunk.cksize;
	if (size == 0L)
	{
		DebugOut("Dlugosc danych jest rowna 0\n");
		SAVEFREE( format )
		format=NULL;
		mmioClose(f, 0);
		return;
	}
	
	// Allocate and lock memory for the waveform data.
	
	data = (char *)GlobalAlloc(GMEM_FIXED, size );
	if (!data)
	{
		DebugOut("Brak pamici dla danych\n");
		SAVEFREE( format );
		mmioClose(f, 0);
		return;
	}
	
	// Read the waveform data subchunk.
	
	if(mmioRead(f, data, size) != (LONG) size)
	{
		DebugOut("Blad odczytu danych mmioRead()\n");
		SAVEFREE( data )
		SAVEFREE( format );
		mmioClose(f, 0);
		return;
	}
	
	// We're done with the file, close it.
	
	mmioClose(f, 0);
	
	m_samples=(size>>1)/m_channels;		//prbek jest 2 razy mniej ni bajtw
	AllocBuffer(m_channels*2*sizeof(float)*m_samples);
	short *src=(short *)data;
	float *dest=m_buffer;
	for (int i=0;i<m_samples*m_channels;i++)
		{
			*(dest++)=*(src++);
			*(dest++)=0;
		}

	SAVEFREE(data);
	SAVEFREE( format );
	
	return;
}

void CMeasure::SaveWaveFile(char *name)
{
	HMMIO           f;
	MMCKINFO        ckOut;
	MMCKINFO        ckOutRIFF;
	MMIOINFO        mmioinfoOut;
	DWORD           do_zapisu;            // Contains the actual fact chunk. Garbage until WaveCloseWriteFile.
	DWORD           zapisal;            // Contains the actual fact chunk. Garbage until WaveCloseWriteFile.
	UINT            cT;
	int             nError=0;
	WAVEFORMATEX *format;
	WAVEBUFFER pbData;
	UINT cbSize;

	f = mmioOpen(name, NULL, MMIO_ALLOCBUF|MMIO_READWRITE|MMIO_CREATE);
	
	if (f == NULL)
		return;
	
	// Create the output file RIFF chunk of form type 'WAVE'.

	ckOutRIFF.fccType = mmioFOURCC('W', 'A', 'V', 'E');       
	ckOutRIFF.cksize = 0; 
	nError = mmioCreateChunk(f, &ckOutRIFF, MMIO_CREATERIFF);
	if (nError)
		goto CLOSE_AND_RETURN;
	
	// We are now descended into the 'RIFF' chunk we just created.
	// Now create the 'fmt ' chunk. Since we know the size of this chunk,
	// specify it in the MMCKINFO structure so MMIO doesn't have to seek
	// back and set the chunk size after ascending from the chunk.

	ckOut.ckid = mmioFOURCC('f', 'm', 't', ' ');
	ckOut.cksize = sizeof(PCMWAVEFORMAT);   // we know the size of this ck.
	nError = mmioCreateChunk(f, &ckOut, 0);
	if (nError)
		goto CLOSE_AND_RETURN;
	
	// Write the PCMWAVEFORMAT structure to the 'fmt ' chunk if its that type.
	
	if (format->wFormatTag == WAVE_FORMAT_PCM)
		do_zapisu=sizeof(PCMWAVEFORMAT);
	else 
		do_zapisu=sizeof(WAVEFORMATEX) + format->cbSize;
	zapisal=(UINT)mmioWrite(f, (HPSTR) format, do_zapisu);
	if ( zapisal != do_zapisu)
	{
		nError=5;
		goto CLOSE_AND_RETURN;
	} 
	
	// Ascend out of the 'fmt ' chunk, back into the 'RIFF' chunk.

	nError = mmioAscend(f, &ckOut, 0);
	if (nError)
		goto CLOSE_AND_RETURN;
	
	// Create the 'data' chunk that holds the waveform samples.
	
	ckOut.ckid = mmioFOURCC('d', 'a', 't', 'a');
	ckOut.cksize = 0;
	nError = mmioCreateChunk(f, &ckOut, 0);
	if (nError)
		goto CLOSE_AND_RETURN;
	
	nError = mmioGetInfo(f, &mmioinfoOut, 0);
	if (nError)
		goto CLOSE_AND_RETURN;

	for (cT=0; cT < cbSize; cT++)
	{       
		if (mmioinfoOut.pchNext == mmioinfoOut.pchEndWrite)
		{
			mmioinfoOut.dwFlags |= MMIO_DIRTY;
			nError = mmioAdvance(f, &mmioinfoOut, MMIO_WRITE);
			if (nError)
				goto CLOSE_AND_RETURN;
		}
		*mmioinfoOut.pchNext = pbData[cT];
		mmioinfoOut.pchNext++;
	}

	mmioinfoOut.dwFlags |= MMIO_DIRTY;
	nError = mmioSetInfo(f, &mmioinfoOut, 0);
	if (nError)
		goto CLOSE_AND_RETURN;
	
	// Ascend the output file out of the 'data' chunk -- this will cause
	// the chunk size of the 'data' chunk to be written.

	nError = mmioAscend(f, &ckOut, 0);
	if (nError)
		goto CLOSE_AND_RETURN;
	
	// Do this here instead...

	nError = mmioAscend(f, &ckOutRIFF, 0);
	if (nError)
		goto CLOSE_AND_RETURN;
	

CLOSE_AND_RETURN:
	mmioClose(f, 0);
	return;
}

void CMeasure::CopyToClipboard(int channel, int from, int to)
{
	HGLOBAL clipboard;
	char *block;
	int pocz,kon;
	float *probki;
	int wielkosc_linii;


	if (m_buffer && OpenClipboard(NULL))
	{
		EmptyClipboard();

		pocz=from; if (pocz<0) pocz=0; else if (pocz>m_samples) pocz=m_samples;
		kon=to; if (kon<0) kon=0; else if (kon>m_samples) kon=m_samples;
		probki=m_buffer + (pocz*m_channels + channel)*2;
		wielkosc_linii=15*(2*m_channels+1)+2;		//czas + n par kanaw + znak koca linii
		clipboard = GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE|GMEM_DISCARDABLE, abs(kon-pocz)*wielkosc_linii);
		block = (char *)GlobalLock(clipboard);
		if (pocz<kon)
		{
			while (pocz<kon)
			{
				StringCbPrintf(block,60,"%5e\t%5e\t%5e\r\n",pocz/m_rate,*probki,*(probki+1)); block += strlen(block);
				++pocz;
				probki+=2*m_channels;
			}
		}
		else
		{
			while (pocz>kon)
			{
				--pocz;
				probki-=2*m_channels;
				StringCbPrintf(block,60,"%5e\t%5e\t%5e\r\n",pocz/m_rate,*probki,*(probki+1)); block += strlen(block);
			}
		}
		GlobalUnlock(clipboard);
		SetClipboardData(CF_TEXT, clipboard);
		CloseClipboard();
	}
}

void CMeasure::Erease(int channel, int from, int to)
{
	int pocz,kon;
	float *probki;

	if (!m_buffer) return;
	pocz=from; if (pocz<0) pocz=0; else if (pocz>m_samples) pocz=m_samples;
	kon=to; if (kon<0) kon=0; else if (kon>m_samples) kon=m_samples;
	probki=m_buffer + (pocz*m_channels + channel)*2;
	if (pocz<kon)
	{
		while (pocz<kon)
		{
			*probki=0;
			*(probki+1)=0;
			++pocz;
			probki+=2*m_channels;
		}
	}
	else
	{
		while (pocz>kon)
		{
			--pocz;
			probki-=2*m_channels;
			*probki=0;
			*(probki+1)=0;
		}
	}
}

void CMeasure::SaveBlock(int channel, int from, int to, char *file_name)
{
	int pocz,kon;
	float *probki;
	FILE *f;

	if (!m_buffer) return;
	pocz=from; if (pocz<0) pocz=0; else if (pocz>m_samples) pocz=m_samples;
	kon=to; if (kon<0) kon=0; else if (kon>m_samples) kon=m_samples;
	probki=m_buffer + (pocz*m_channels + channel)*2;
	f=fopen(file_name,"wt");
	if (pocz<kon)
	{
		while (pocz<kon)
		{
			fprintf(f,"%5e",pocz/m_rate);
			fprintf(f,"\t%5e\t%5e",*probki,*(probki+1));
			fprintf(f,"\n");
			++pocz;
			probki+=2*m_channels;
		}
	}
	else
	{
		while (pocz>kon)
		{
			--pocz;
			probki-=2*m_channels;
			fprintf(f,"%5e",pocz/m_rate);
			fprintf(f,"\t%5e\t%5e",*probki,*(probki+1));
			fprintf(f,"\r\n");
		}
	}
	fclose(f);
}

///////////////////////////////////////////
void CMeasure::CalcRange(float *min, float *max, float *amp)
{
	float a,b,c;
	float *probki;
	int i,j;
	int m_scalenum[MAX_CHANNEL];

	if (m_buffer)
	{
		//wypenienie numerw skal
		m_scalenum[0]=1;
		for (i=1;i<m_channels;i++)
			m_scalenum[i]=0;
		i=0;
		while (i<m_channels)
		{
			for (j=i+1;j<m_channels;j++)
				if (strcmp(m_opis[i].m_jednostka, m_opis[j].m_jednostka)==0)
					m_scalenum[j]=m_scalenum[i];
			j=i;
			++i;
			while (i<m_channels && m_scalenum[i])
				++i;
			if (i<m_channels) m_scalenum[i]=m_scalenum[j]+1;
		}

		//wypenienie tablicy z ekstremami
		probki=m_buffer;
		for (i=0;i<m_channels;i++)
		{
			a=*(probki++);
			b=*(probki++);
			min[i]=a;
			max[i]=a;
			if (b>max[i]) max[i]=b; else if (b<min[i]) min[i]=b;
			a=sqrt(a*a+b*b);
			amp[i]=a;
		}
		for (j=1;j<m_samples;j++)
		{
			for (i=0;i<m_channels;i++)
			{
				a=*(probki++);
				b=*(probki++);
				if (a>max[i]) max[i]=a; else if (a<min[i]) min[i]=a;
				if (b>max[i]) max[i]=b; else if (b<min[i]) min[i]=b;
				a=sqrt(a*a+b*b);
				if (a>amp[i]) amp[i]=a;
			}
		}

		//ujednolicenie skal dla jednakowych jednostek
		i=0;
		while (i<m_channels)
		{
			a=min[i];
			b=max[i];
			c=amp[i];
			for (j=i+1;j<m_channels;j++)
				if (m_scalenum[j]==m_scalenum[i])
				{
					if (min[j]<a) a=min[j];
					if (max[j]>b) b=max[j];
					if (amp[j]>c) c=amp[j];
				}
			for (j=i;j<m_channels;j++)
				if (m_scalenum[j]==m_scalenum[i])
				{
					min[j]=a;
					max[j]=b;
					amp[j]=c;
				}
			j=i;
			while (i<m_channels && m_scalenum[j]==m_scalenum[i])
				++i;
		}
	}
}

////////////////////////////////////////////////////////
void statystyka(CStatyData *info, unsigned do_obliczenia)
{
	int N;
	double dt;
	float *probki;
	long i,skok;
	double a,suma,min,max;
	double srednia;
	double min_fala;
	double max_fala;
	double srednia_fala;
	double min_okres;
	double max_okres;
	double sredni_okres;
	int liczba_fal;
	int liczba_okresow;
	CFiltr sygnal;
	CFiltr pochodna;
	CFiltr zera;
	CFiltr ekstrema;

	info->m_obliczono=0;
	if (!info->m_probki) return;

	skok = 2*info->m_channels;
	dt=1/info->m_rate;
	probki = info->m_probki;
	N=info->m_samples;

	min=*probki;
	probki+=skok;
	max=min;
	srednia=min;
	info->m_obliczono=OBL_MIN | OBL_MAX | OBL_SR;
	info->m_min_wartosc=min;
	info->m_max_wartosc=max;
	info->m_srednia=srednia;
	if (N == 1)
		return;

	for (i=1; i<N; i++)
	{
		a=*probki;
		if (a<min) min=a; else if (a>max) max=a;
		srednia += a;
		probki+=skok;
	}
	srednia /= N;
	info->m_min_wartosc=min;
	info->m_max_wartosc=max;
	info->m_srednia=srednia;

	if (do_obliczenia & OBL_STD)
	{
		probki = info->m_probki;
		suma=*probki;
		probki+=skok;
		for (i=1; i<N; i++)
		{
			a=*probki - srednia;
			suma += a*a;
			probki+=skok;
		}
		info->m_odchylenie_std=sqrt(suma/N);
		info->m_obliczono |= OBL_STD;
	}

	if (N < 10) return;

	if (do_obliczenia & (OBL_FALA|OBL_OKRES))
	{
		min_fala=0.0;
		max_fala=0.0;
		srednia_fala=0.0;
		min_okres=0.0;
		max_okres=0.0;
		sredni_okres=0.0;
		liczba_fal=0;
		liczba_okresow=0;
		probki = info->m_probki;
		for (i=0;i<4;i++)
		{
			sygnal.AddSample(*probki - srednia);
			probki+=skok;
		}
		for (i=4;i<9;i++)
		{
			sygnal.AddSample(*probki - srednia);
			pochodna.AddSample(sygnal.pochodna());
			probki+=skok;
		}

		for (i=9; i<N; i++)
		{
			sygnal.AddSample(*probki - srednia);
			if (sygnal.jest_zero())
			{
				a=sygnal.miejsce_zerowe()+i;
				if (zera.liczba_wpisow() > 0)
				{
					if (a-zera.na_szczycie() > 1)
					{
						zera.AddSample(a);
						if (zera.liczba_wpisow()>2)
						{
							a=(zera.na_szczycie() - zera.od_konca(2))*dt;
							if (liczba_okresow)
							{
								if (a>max_okres) max_okres=a; else if (a<min_okres) min_okres=a;
								sredni_okres+=a;
							}
							else
							{
								max_okres=a;
								min_okres=a;
								sredni_okres=a;
							}
							liczba_okresow++;
						}
					}
				}
				else
					zera.AddSample(a);
			}

			pochodna.AddSample(sygnal.pochodna());
			if (pochodna.jest_zero())
			{
				a=pochodna.miejsce_zerowe();
				a=sygnal.ekstremum(a);
				if (ekstrema.liczba_wpisow())
				{
					if (ekstrema.na_szczycie() > 0)
					{
						if (a>0)
						{
							if (a>ekstrema.na_szczycie()) ekstrema.wpisz_na_szczyt(a);
						}
						else //zmiana znaku
						{
							ekstrema.AddSample(a);
							if (ekstrema.liczba_wpisow()>2)
							{
								a=fabs(ekstrema.od_konca(1)-ekstrema.od_konca(2));
								if (liczba_fal)
								{
									if (a>max_fala) max_fala=a; else if (a<min_fala) min_fala=a;
									srednia_fala+=a;
								}
								else
								{
									max_fala=a;
									min_fala=a;
									srednia_fala=a;
								}
								liczba_fal++;
							}
						}
					}
					else			// (ekstrema.na_szczycie() < 0)
					{
						if (a<0)
						{
							if (a<ekstrema.na_szczycie()) ekstrema.wpisz_na_szczyt(a);
						}
						else //zmiana znaku
						{
							ekstrema.AddSample(a);
							if (ekstrema.liczba_wpisow()>2)
							{
								a=fabs(ekstrema.od_konca(1)-ekstrema.od_konca(2));
								if (liczba_fal)
								{
									if (a>max_fala) max_fala=a; else if (a<min_fala) min_fala=a;
									srednia_fala+=a;
								}
								else
								{
									max_fala=a;
									min_fala=a;
									srednia_fala=a;
								}
								liczba_fal++;
							}
						}
					}
				}
				else
					ekstrema.AddSample(a);
			}
			probki+=skok;
		}
		if (liczba_okresow>0) sredni_okres=sredni_okres/liczba_okresow;
		if (liczba_fal>0) srednia_fala=srednia_fala/liczba_fal;
		info->m_min_fala=min_fala;
		info->m_max_fala=max_fala;
		info->m_srednia_fala=srednia_fala;
		info->m_min_okres=min_okres;
		info->m_max_okres=max_okres;
		info->m_sredni_okres=sredni_okres;
		info->m_liczba_fal=liczba_fal;
		info->m_liczba_okresow=liczba_okresow;
		info->m_obliczono |= (OBL_FALA|OBL_OKRES);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CParametrySkladowejDlg
BOOL CParametrySkladowejDlg::OnInitDialog(HWND hWnd)
{
	char napis[80];

	CDialog::OnInitDialog(hWnd);
	SetText(IDC_NAME,m_new_params.m_name);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_fr);
	SetText(IDC_FR,napis);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_Vr);
	SetText(IDC_VR,napis);
	StringCbPrintf(napis,80,"%4.2f",m_new_params.m_K);
	SetText(IDC_K,napis);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_A);
	SetText(IDC_A,napis);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_D);
	SetText(IDC_D,napis);
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_eta);
	SetText(IDC_ETA,napis);
	return TRUE;
}

BOOL CParametrySkladowejDlg::OnOk()
{
	GetText(IDC_NAME,m_new_params.m_name,41);
	GetDouble(IDC_FR,&m_new_params.m_fr);
	GetDouble(IDC_VR,&m_new_params.m_Vr);
	GetDouble(IDC_K,&m_new_params.m_K);
	GetDouble(IDC_A,&m_new_params.m_A);
	GetDouble(IDC_D,&m_new_params.m_D);
	GetDouble(IDC_ETA,&m_new_params.m_eta);
	*m_params=m_new_params;
	return CDialog::OnOk();
}

/////////////////////////////////////////////////////////////////////////////
// CParametrySygnaluDlg

static LV_COLUMN skladowe_listbox_columns[7]={
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_LEFT, 100,"Name",0,0},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 70,"Frequency",0,1},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 70,"Amplitude",0,2},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 60,"K",0,3},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 60,"A",0,4},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 60,"D",0,5},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT, 60,"Eta",0,6}};

void CParametrySygnaluDlg::fill_param(int i)
{
	char napis[80];
	LV_ITEM lvi;

	lvi.mask = LVIF_TEXT;
	lvi.iItem=i;
	lvi.iSubItem=1;
	lvi.pszText = napis;   // app. maintains text
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_skladowe[i].m_fr);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=2;
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_skladowe[i].m_Vr);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=3;
	StringCbPrintf(napis,80,"%4.2f",m_new_params.m_skladowe[i].m_K);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=4;
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_skladowe[i].m_A);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=5;
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_skladowe[i].m_D);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=6;
	StringCbPrintf(napis,80,"%6.4f",m_new_params.m_skladowe[i].m_eta);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
}

void CParametrySygnaluDlg::insert(int i)
{
	LV_ITEM lvi;
	CParametrySkladowej parametry;

	StringCbPrintf(parametry.m_name,41,"Component no %d",i+1);
	parametry.m_fr=1.0;
	parametry.m_Vr=1.0;
	parametry.m_K=5.0;
	parametry.m_A=1.0;
	parametry.m_D=1.0;
	parametry.m_eta=0.0;
	CParametrySkladowejDlg dlg(&parametry);
	if (dlg.Execute(m_handle)==IDOK)
	{
		m_new_params.InsertComponent(i,&parametry);
		lvi.mask = LVIF_TEXT;
		lvi.pszText = parametry.m_name;
		lvi.iSubItem=0;
		lvi.iItem=i;
		SendMessage(IDC_CHANNELS,LVM_INSERTITEM,0,(LPARAM)&lvi);
		fill_param(i);
		m_byly_zmiany=TRUE;
	}
}

void CParametrySygnaluDlg::change_settings()
{
	LV_ITEM lvi;
	int i;

	GetSelectedItem(IDC_CHANNELS,&i);
	if (i<m_new_params.m_liczba_skladowych)
	{
		CParametrySkladowejDlg dlg(&m_new_params.m_skladowe[i]);
		if (dlg.Execute(m_handle)==IDOK)
		{
			lvi.mask = LVIF_TEXT;
			lvi.pszText = m_new_params.m_skladowe[i].m_name;
			lvi.iSubItem=0;
			lvi.iItem=i;
			SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
			fill_param(i);
			m_byly_zmiany=TRUE;
		}
	}
	else
		if (i==m_new_params.m_liczba_skladowych) insert(i);
}

void CParametrySygnaluDlg::delete_component()
{
	int i;

	GetSelectedItem(IDC_CHANNELS,&i);
	if (i>=0 && i<m_new_params.m_liczba_skladowych)
	{
		SendMessage(IDC_CHANNELS,LVM_DELETEITEM,i,0);
		m_new_params.DeleteComponent(i);
	}
}

void CParametrySygnaluDlg::insert_component()
{
	int i;

	GetSelectedItem(IDC_CHANNELS,&i);
	if (i>=0 && i<m_new_params.m_liczba_skladowych) insert(i);
}

char cNewComp[]="New component";

void CParametrySygnaluDlg::create_columns()
{
	LV_ITEM lvi;
	int i;

	for (i=0; i<7; ++i) SendMessage(IDC_CHANNELS,LVM_INSERTCOLUMN,i,(LPARAM)&skladowe_listbox_columns[i]);
	lvi.mask = LVIF_TEXT;
	lvi.iSubItem=0;
	for (i=0; i<m_new_params.m_liczba_skladowych; ++i)
	{
		lvi.iItem=i;
		lvi.pszText = m_new_params.m_skladowe[i].m_name;
		SendMessage(IDC_CHANNELS,LVM_INSERTITEM,0,(LPARAM)&lvi);
		fill_param(i);
	}
	lvi.iItem=m_new_params.m_liczba_skladowych;
	lvi.pszText = cNewComp;
	SendMessage(IDC_CHANNELS,LVM_INSERTITEM,0,(LPARAM)&lvi);

	lvi.mask = LVIF_STATE;
	lvi.state = LVIS_SELECTED;
	lvi.stateMask = LVIS_SELECTED;
	lvi.iItem=0;
	lvi.iSubItem=0;
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
}


BOOL CParametrySygnaluDlg::OnInitDialog(HWND hWnd)
{
	CDialog::OnInitDialog(hWnd);
	SendMessage(IDC_CHANNELS,LVM_SETEXTENDEDLISTVIEWSTYLE,LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES,LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES);
	m_byly_zmiany=FALSE;
	SetInteger(IDC_CHANNEL,m_new_selection.m_channel);
	SetInteger(IDC_FROM,m_new_selection.m_from);
	SetInteger(IDC_TO,m_new_selection.m_to);
	create_columns();
	return TRUE;
}

BOOL CParametrySygnaluDlg::OnNotify(int id, NMHDR *hdr)
{
	//int i;

	switch (id)
	{
	case IDC_CHANNELS:
		if (hdr->code == NM_DBLCLK) change_settings();
		break;
	}
	return TRUE;
}

BOOL CParametrySygnaluDlg::OnCommand(int id, int /*notify*/, HWND /*control*/)
{
	switch (id)
	{
	case IDC_EDIT :
		change_settings();
		break;
	case IDC_DELETE :
		delete_component();
		break;
	case IDC_INSERT :
		insert_component();
		break;
	case IDC_COPY :
		copy_all();
		break;
	}
	return TRUE;
}

BOOL CParametrySygnaluDlg::OnOk()
{
	GetInteger(IDC_CHANNEL,&m_new_selection.m_channel);
	GetInteger(IDC_FROM,&m_new_selection.m_from);
	GetInteger(IDC_TO,&m_new_selection.m_to);
	*m_params=m_new_params;
	*m_selection=m_new_selection;
	return CDialog::OnOk();
}

void CParametrySygnaluDlg::copy_all()
{
	HGLOBAL clipboard;
	char *block;
	int i;

	if (OpenClipboard(NULL))
	{
		EmptyClipboard();
		clipboard = GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE|GMEM_DISCARDABLE, 2048);
		block = (char *)GlobalLock(clipboard);

		StringCbPrintf(block,40,"%d\t%d\t%d\t%d\r\n",
			m_new_selection.m_channel,
			m_new_selection.m_from,
			m_new_selection.m_to,
			m_new_params.m_liczba_skladowych);
		block += strlen(block);
		for (i=0; i<m_new_params.m_liczba_skladowych; i++)
		{
			StringCbPrintf(block,60,"%6.4f\t%6.4f\t%6.4f\t%6.4f\t%6.4f\t%6.4f\r\n",
				m_new_params.m_skladowe[i].m_fr,
				m_new_params.m_skladowe[i].m_Vr,
				m_new_params.m_skladowe[i].m_K,
				m_new_params.m_skladowe[i].m_A,
				m_new_params.m_skladowe[i].m_D,
				m_new_params.m_skladowe[i].m_eta);
			block += strlen(block);
		}
		GlobalUnlock(clipboard);
		SetClipboardData(CF_TEXT, clipboard);
		CloseClipboard();
	}
}


/////////////////////////////////////////////////////////////////////////////
// CWMeasureView

BOOL CBlockDlg::OnInitDialog(HWND hWnd)
{
	CDialog::OnInitDialog(hWnd);
	SetInteger(IDC_CHANNEL,m_new_selection.m_channel);
	SetInteger(IDC_FROM,m_new_selection.m_from);
	SetInteger(IDC_TO,m_new_selection.m_to);
	return TRUE;
}

BOOL CBlockDlg::OnOk()
{
	GetInteger(IDC_CHANNEL,&m_new_selection.m_channel);
	GetInteger(IDC_FROM,&m_new_selection.m_from);
	GetInteger(IDC_TO,&m_new_selection.m_to);
	*m_selection=m_new_selection;
	return CDialog::OnOk();
}

/////////////////////////////////////////////////////////////////////////////
// CWMeasureView

CMeasureView::CMeasureView(int plot_style) : CWnd()
{
	m_doc=NULL;
	m_range=1;
	m_begin_block=0;
	m_end_block=0;
	m_sel_channel=0;
	m_drag_object=0;
	m_drag_offset=0;
	m_plot_style=plot_style;
	m_visible_samples=2000;		//liczba prbek widocznych na szerokoci okna
	m_visible_channels=1;		//liczba prbek widocznych na szerokoci okna
}

///////////////////////////////////////////
int CMeasureView::Create(int nX, int nY, int nW, int nH, HWND hParent, DWORD style)
{
	CreateWindowEx(WS_EX_CLIENTEDGE,
		"MiwilDisplayClass",		   /* See RegisterClass() call.		   */
		"" ,						   /* Window title bar	*/
		style,	  /* Window style.			*/
		nX, nY, nW, nH,				/* Default height.					 */
		hParent,					   /* Parent */
		0,
		(HINSTANCE)GetWindowLong(hParent,GWL_HINSTANCE),
		this						   /* Pointer not needed.				 */
		);
	if (!m_handle) return 1;
	return 0;
}

///////////////////////////////////////
int CMeasureView::OnCreate(LPCREATESTRUCT /*lpCreateStruct*/)
{
	ShowScrollBar(m_handle,SB_BOTH,FALSE);
	EnableScrollBar(m_handle,SB_BOTH,ESB_DISABLE_BOTH);
	return 0;
}

///////////////////////////////////////////
void CMeasureView::CalcRange()
{
	if (m_doc)
	{
		m_doc->CalcRange(m_min,m_max,m_amp);
		int visible=m_doc->NumberOfChannels();
		for (int i=0; i<visible; i++) m_expanded_channels[i]=1;
		if (m_visible_samples<20) m_visible_samples=1000;
		if (m_doc->NumberOfSamples()>100)
			if (m_visible_samples>m_doc->NumberOfSamples()) m_visible_samples=m_doc->NumberOfSamples();
	}
}

///////////////////////////////////////////
void CMeasureView::SetDocument(CMeasure *doc)
{
	m_doc=doc;
	CalcRange();
	EnableScrollBar(m_handle,SB_BOTH,ESB_ENABLE_BOTH);
	ShowScrollBar(m_handle,SB_BOTH,TRUE);
	RefreshScrollBars();
	Invalidate(FALSE);
}

////////////////////////////////////////
void CMeasureView::DocumentChanged()
{
	CalcRange();
	RefreshScrollBars();
	Invalidate(FALSE);
}
///////////////////////////////////////////
void CMeasureView::GetSelection(CSelection *selection)
{
	selection->m_channel = m_sel_channel;
	selection->m_from = m_begin_block;
	selection->m_to = m_end_block;
}

///////////////////////////////////////////
void CMeasureView::GetState(int *state)
{
	*state = m_plot_style;
}

///////////////////////////////////////////
void CMeasureView::GetRightSelection(CSelection *selection)
{
	selection->m_channel = m_rb_channel;
	selection->m_from = 0;
	selection->m_to = m_doc->NumberOfSamples();
}

///////////////////////////////////////////
void CMeasureView::OnVScroll(int scroll_code, int /*nPos*/, HWND /*hwndScrollBar*/)
{
	SCROLLINFO si;

	si.cbSize=sizeof(SCROLLINFO);
	si.fMask = SIF_ALL;
	GetScrollInfo(m_handle,SB_VERT,&si);
	switch (scroll_code)
	{
		case SB_TOP: si.nPos = si.nMin; break;
		case SB_BOTTOM:	si.nPos = si.nMax; break;
		case SB_LINEUP: si.nPos -= 1; break;
		case SB_LINEDOWN: si.nPos += 1; break;
		case SB_PAGEUP:	si.nPos -= (int)si.nPage; break;
		case SB_PAGEDOWN: si.nPos += si.nPage; break;
		case SB_THUMBTRACK:	si.nPos = si.nTrackPos; break;
	}
	si.fMask = SIF_POS;
	SetScrollInfo(m_handle,SB_VERT,&si,TRUE);
	Invalidate(FALSE);
}

///////////////////////////////////////////
void CMeasureView::OnHScroll(int scroll_code, int /*nPos*/, HWND /*hwndScrollBar*/)
{
	SCROLLINFO si;

	si.cbSize=sizeof(SCROLLINFO);
	si.fMask = SIF_ALL;
	GetScrollInfo(m_handle,SB_HORZ,&si);
	switch (scroll_code)
	{
		case SB_TOP: si.nPos = si.nMin; break;
		case SB_BOTTOM:	si.nPos = si.nMax; break;
		case SB_LINEUP: si.nPos -= 1; break;
		case SB_LINEDOWN: si.nPos += 1; break;
		case SB_PAGEUP:	si.nPos -= (int)si.nPage; break;
		case SB_PAGEDOWN: si.nPos += si.nPage; break;
		case SB_THUMBTRACK:	si.nPos = si.nTrackPos; break;
	}
	si.fMask = SIF_POS;
	SetScrollInfo(m_handle,SB_HORZ,&si,TRUE);
	Invalidate(FALSE);
}

///////////////////////////////////////////
void CMeasureView::RefreshScrollBars()
{
	SCROLLINFO si;
	POINT s;
	RECT r;

	GetClientRect(m_handle,&r);
	m_width=r.right-r.left;
	m_height=r.bottom-r.top;
	m_channel_height=(r.bottom-r.top-WYSOKOSC_TYT*m_visible_channels)/m_visible_channels;
	m_x_scale = (r.right-r.left)/(double)m_visible_samples;

	size(s);
	si.cbSize=sizeof(SCROLLINFO);
	si.fMask=SIF_PAGE|SIF_RANGE;
	si.nMin=0;
	if (m_doc) si.nMax=s.x; else si.nMax=0;
	si.nPage=(r.right-r.left)+1;
	SetScrollInfo(m_handle,SB_HORZ,&si,TRUE);
	si.nMin=0;
	if (m_doc) si.nMax=s.y; else si.nMax=0;
	si.nPage=(r.bottom-r.top)+1;
	SetScrollInfo(m_handle,SB_VERT,&si,TRUE);
}

///////////////////////////////////////////
//podaje aktualn liczb widocznych kanaw uwgldniajc
//ilosc kanaw zadeklarowan w konfiguracji oraz stan
//wcznikw widocznoci kanau.
//korzysta ze zmiennej globalnej: g_channel_visible
int CMeasureView::num_visible()
{
	int n=0;
	if (m_doc)
	{
		int visible=m_doc->NumberOfChannels();
		while (visible)
			n+=m_expanded_channels[--visible];
	}
	return n;
}

////////////////////////////////////////////////////////////////////
void CMeasureView::xy2sample(int x, int /*y*/, int &n)
{
	if(m_doc)
	{
		n=(int)floor(x/m_x_scale+0.5);
		if (n<0) n=0; else if (n>m_doc->NumberOfSamples()) n=m_doc->NumberOfSamples();
	}
	else
		n=0;
}

////////////////////////////////////////////////////////////////////
void CMeasureView::sample2xy(int n, int &x, int &y)
{
	x=(int)floor(n*m_x_scale+0.5);
	y=0;
}

////////////////////////////////////////////////////////////////////
// x,y jest juz po przerobkach.
int CMeasureView::pointed_object(int x, int /*y*/, int channel)
{
	int xpos,ypos;

	if (m_end_block!=m_begin_block && channel==m_sel_channel)
	{
		sample2xy(m_end_block,xpos,ypos);
		if (x>xpos-3 && x<xpos+3) return 2;
		sample2xy(m_begin_block,xpos,ypos);
		if (x>xpos-3 && x<xpos+3) return 1;
	}
	return 0;
}

///////////////////////////////////////
//podaje prawdziwy numer kanau na podstawie pozycji kliknicia
//z uwgldnieniem niewidocznych kanaw. Kanay s numerowane
//od zera.
//korzysta ze zmiennej globalnej: g_channel_visible, g_yscale
int CMeasureView::klikniety_kanal(int &pos)
{
	if (m_doc)
	{
		int i,n=m_doc->NumberOfChannels();
		for (i=0; i<n; i++)
		{
			pos -= WYSOKOSC_TYT;
			if (pos<0) return i;
			if (m_expanded_channels[i])
			{
				if (pos<m_channel_height) return i;
				pos -= m_channel_height;
			}
		}
	}
	return -1;
}

///////////////////////////////////////
//korzysta ze zmiennej globalnej: g_channel_visible, g_yscale
int CMeasureView::gora_kanalu(int kanal)
{
	int top,i;

	top=WYSOKOSC_TYT;
	for (i=0;i<kanal;i++)
	{
		if (m_expanded_channels[i]) top += m_channel_height;
		top += WYSOKOSC_TYT;
	}
	return top;
}

///////////////////////////////////////
void CMeasureView::PosToPoint(int xpos, int ypos, int ch, double &x, double &y)
{
	double gridy,sy;
	double ymin;
	double ymax;
	if (m_plot_style & PLOTSTYLE_MODULE)
	{
		ymin=0;
		ymax=m_amp[ch]/m_range;
	}
	else
	{
		ymin=m_min[ch]/m_range;
		ymax=m_max[ch]/m_range;
	}
	grid_step(ymin, ymax, gridy);
	sy=(m_channel_height-WYSOKOSC_TYT)/(ymax-ymin);
	x = (xpos/m_x_scale)/m_doc->GetRate();
	y = ymin+(m_channel_height-WYSOKOSC_TYT-ypos)/sy;
}

///////////////////////////////////////
void CMeasureView::OnMouseMove(int /*nKeys*/, int xpos, int ypos)
{
	int scroll_x_pos, scroll_y_pos, channel;
	double x,y;
	char napis[80];

	CurrentScrollPos(scroll_x_pos, scroll_y_pos);
	xpos+=scroll_x_pos;
	ypos+=scroll_y_pos;
	channel=klikniety_kanal(ypos);

	if (ypos<0)
	{
		SetCursor(LoadCursor(NULL,IDC_ARROW));
	}
	else
	{
		if (m_drag_object)
		{
			switch (m_drag_object)
			{
			case 1:
				set_begin_block(xpos-m_drag_offset);
				break;

			case 2:
				set_end_block(xpos-m_drag_offset);
				break;
			}
			UpdateBlockInfo();
			Invalidate(FALSE);
			SetCursor(LoadCursor(NULL,IDC_SIZEWE));
		}

		PosToPoint(xpos,ypos,channel,x,y);
		StringCbPrintf(napis,80,"ch=%d  x=%6.4f  y=%6.4f",channel,x,y);
		SendMessage(g_statusbar,SB_SETTEXT,1,(LPARAM)napis);
		if (pointed_object(xpos,ypos,channel))
			SetCursor(LoadCursor(NULL,IDC_SIZEWE));
		else
			SetCursor(LoadCursor(NULL,IDC_CROSS));
	}
}

///////////////////////////////////////
void CMeasureView::OnLButtonDown(int /*nKeys*/, int xpos, int ypos)
{
	int scroll_x_pos, scroll_y_pos;
	int channel;

	CurrentScrollPos(scroll_x_pos, scroll_y_pos);
	xpos+=scroll_x_pos;
	ypos+=scroll_y_pos;
	channel=klikniety_kanal(ypos);

	if (ypos<0)
	{
		if (channel>=0)
		{
			m_expanded_channels[channel]^=1;
			RefreshScrollBars();
			Invalidate(FALSE);
		}
	}
	else
	{
		switch (pointed_object(xpos,ypos,channel))
		{
		case 1:	//ciagne poczatek bloku
			m_drag_object=1;
			m_drag_offset=xpos-begin_block_pos();
			break;
		case 2:	//cign koniec bloku
			m_drag_object=2;
			m_drag_offset=xpos-end_block_pos();
			break;
		default:	//zaczynam cign koniec
			m_drag_offset=0;
			clear_block_at(xpos);
			m_sel_channel=channel;
			m_drag_object=2;
			m_old_capture=SetCapture(m_handle);
			UpdateBlockInfo();
		}
		Invalidate(FALSE);
	}
}

///////////////////////////////////////
void CMeasureView::OnLButtonUp(int /*nKeys*/, int /*nXPos*/, int /*nYPos*/)
{
	if (m_drag_object)
	{
		SetCapture(m_old_capture);
		m_drag_object = 0;
	}
}

/////////////////////////////////////////////////////
//ta funkcja szacuje najlepszy skok siatki. Podany zakres jest
// dzielony na okolo 20 przedziaow. Dopuszczalne kroki to:
// ..., 0.001, 0.01, 0.1, 1, 10, 100, 1000, ...
// ..., 0.002, 0.02, 0.2, 2, 20, 200, 2000, ...
// ..., 0.005, 0.05, 0.5, 5, 50, 500, 5000, ...

void CMeasureView::grid_step(double &ymin, double &ymax, double &step)
{
	double y,d;
	double range=ymax-ymin;

	//znormalizuj skal
	y = floor(log10(range));  //0 for 1-10, 1 for 10-100, ...
	y = pow(10.0,y);                 //1 for 1-10, 10 for 10-100, ...
	d = range / y;                 //normalised scale to range 1-10

	//wybierz znormalizowany krok
	if (d < 2)
		d = 5;		//10
	else
		if (d < 4)
			d = 2;		//5
		else
			d = 1;		//2
	y /= d;		//denormalizuj krok

	ymin=floor(ymin/y)*y;
	ymax=floor(ymax/y)*y + y;
	step=y;
}

///////////////////////////////////////////
//wypenia zmienn globaln doc_size zgodnie z liczb widocznych kanaw,
//skal osi X i Y (g_xscale,g_yscale), trybem pracy (g_sheet).
void CMeasureView::size(POINT &s)
{
	s.x=0;	s.y=0;
	if (m_doc)
	{
		int visible=m_doc->NumberOfChannels();
		s.x = (int)floor(m_doc->NumberOfSamples()*m_x_scale + 0.5);
		s.y = num_visible()*m_channel_height + visible*WYSOKOSC_TYT;
	}
	if (s.x<10) s.x=10;	//zeby troche bylo jesli nic nie ma
	if (s.y<10) s.y=10;
}

///////////////////////////////////////////
void CMeasureView::OnPaint(HDC hDC)
{
	RECT ekran,pasek,pole;
	HBRUSH hSeledinBrush,hWhiteBrush,hTitleBrush,hFftBarBrush;
	float *probki;
	HPEN hOldPen,hDashPen,hGridPen;
	int liczba_probek,liczba_kanalow;
	int numer_kanalu;
	int skok,krok;
	int pierwsza_probka;
	int i,x;
	double skala_y,w1,w2,y,re,im;
	int scroll_x_pos, scroll_y_pos;
	int calkowita_liczba_probek;
	HFONT hOldFont;
	HRGN region;
	int x1,y1,x2,old_bkMode;
	double ymin,ymax,gridy;
	double xmin,xmax,gridx;
	char nap[20];
	HDC dc;
	HBITMAP offscreen, old_image;


	/////////////////////////// przygotowanie pola rysunku //////////////////////////

	SelectClipRgn(hDC,NULL);
	GetClientRect(m_handle,&ekran);
	int window_width=ekran.right-ekran.left;
	int window_height=ekran.bottom-ekran.top;

	if (!m_doc || !m_doc->Buffer())
	{
		FillRect(hDC,&ekran,(HBRUSH)GetStockObject(WHITE_BRUSH));
		return;
	}
	liczba_probek=m_doc->NumberOfSamples();
	if (!liczba_probek)
	{
		FillRect(hDC,&ekran,(HBRUSH)GetStockObject(WHITE_BRUSH));
		return;
	}

	//FillRect(hDC,&ekran,(HBRUSH)GetStockObject(WHITE_BRUSH));
	hSeledinBrush=CreateSolidBrush(PALETTERGB(255,255,220));
	hWhiteBrush=CreateSolidBrush(RGB(255,255,255));
	hFftBarBrush=CreateSolidBrush(RGB(0,0,255));
	hDashPen=CreatePen(PS_DASH,1,RGB(127,127,127));
	hGridPen=CreatePen(PS_DASH,0,RGB(180,180,180));
	hTitleBrush=CreateSolidBrush(RGB(220,255,220));	//tlo paska tytulu;
	CurrentScrollPos(scroll_x_pos, scroll_y_pos);
	probki=m_doc->Buffer();
	liczba_kanalow=m_doc->NumberOfChannels();
	pasek=ekran;


	pole.top=0; pole.left=0;
	pole.bottom=m_channel_height; pole.right=window_width;
	dc=CreateCompatibleDC(hDC);
	offscreen=CreateCompatibleBitmap(hDC,window_width,window_height);
	old_image=(HBITMAP)SelectObject(dc,offscreen);

	////////////// teraz rysowanie odbywa si wycznie na dc a nie na hDC /////////////////

	old_bkMode=SetBkMode(dc,TRANSPARENT);
	hOldFont=(HFONT)GlobalSmallFont.Select(dc);
	hOldPen=(HPEN)GetCurrentObject(dc,OBJ_PEN);

	pierwsza_probka=(int)floor(scroll_x_pos/m_x_scale+0.5);
	calkowita_liczba_probek=liczba_probek*liczba_kanalow*2;

	hOldPen=(HPEN)SelectObject(dc,hDashPen);

	pasek.top -= scroll_y_pos;
	for (numer_kanalu=0; numer_kanalu<liczba_kanalow; numer_kanalu++)
	{
		pole.bottom=WYSOKOSC_TYT;
		if (pasek.top+WYSOKOSC_TYT>0 && pasek.top<ekran.bottom)
		{
			region=CreateRectRgn(pole.left,pole.top,pole.right,pole.bottom);
			SelectClipRgn(dc,region);
			DeleteObject(region);
			FillRect(dc,&pole,hTitleBrush);
			char *s=m_doc->GetChannelName(numer_kanalu);
			SetTextColor(dc,RGB(0,0,0));
			TextOut(dc,pole.left+50,pole.top,s,strlen(s));
			SelectObject(dc,(HPEN)GetStockObject(BLACK_PEN));
			DrawLine(dc,pole.left,pole.top,pole.right,pole.top);
			StretchBlt(hDC,0,pasek.top,window_width,WYSOKOSC_TYT,dc,0,0,window_width,WYSOKOSC_TYT,SRCCOPY);
		}

		pasek.top+=WYSOKOSC_TYT;

		if ( m_expanded_channels[numer_kanalu] )
		{
			if (pasek.top+m_channel_height>0 && pasek.top<ekran.bottom)
			{
				xmin=pierwsza_probka/m_doc->GetRate();
				xmax=xmin + m_visible_samples/m_doc->GetRate();
				grid_step(xmin, xmax, gridx);

				if (m_plot_style & PLOTSTYLE_MODULE)
				{
					ymin=0;
					ymax=m_amp[numer_kanalu]/m_range;
				}
				else
				{
					ymin=m_min[numer_kanalu]/m_range;
					ymax=m_max[numer_kanalu]/m_range;
				}
				grid_step(ymin, ymax, gridy);
				skala_y=(m_channel_height-WYSOKOSC_TYT)/(ymax-ymin);

				///////////////// siatka osi poziomej i pionowej ///////////////////////

				pole.bottom=m_channel_height;
				region=CreateRectRgn(pole.left,pole.top,pole.right,pole.bottom);
				SelectClipRgn(dc,region);
				DeleteObject(region);
				pole.top=m_channel_height-WYSOKOSC_TYT;
				FillRect(dc,&pole,hTitleBrush);
				pole.bottom=pole.top; pole.top=0;
				if (numer_kanalu & 1)
					FillRect(dc,&pole,hSeledinBrush);
				else
					FillRect(dc,&pole,hWhiteBrush);

				SelectObject(dc,hGridPen);
				SetTextColor(dc,RGB(96,96,96));
				for (y=xmin+gridx; y<=xmax; y+=gridx)
				{
					x1=(int)floor(y*m_doc->GetRate()+0.5)-pierwsza_probka;		//numer probki
					x1=pole.left+(int)floor(x1*m_x_scale+0.5);
					DrawLine(dc, x1 ,pole.top, x1 ,pole.bottom );
					StringCbPrintf(nap,20,"%4.2f",y);
					TextOut(dc,x1-12,pole.bottom,nap,strlen(nap));
				}

				for (y=ymin+gridy; y<=ymax-gridy; y+=gridy)
				{
					y1=pole.bottom-(int)floor((y-ymin)*skala_y+0.5);
					DrawLine(dc,40, y1 ,pole.right, y1 );
					StringCbPrintf(nap,20,"%4.2f %s",y,m_doc->GetChannelUnit(numer_kanalu));
					TextOut(dc,pole.left,y1-8,nap,strlen(nap));
				}

				/////////////////////////// rysowanie zalene od trybu //////////////////////////

				region=CreateRectRgn(pole.left,pole.top,pole.right,pole.bottom);
				SelectClipRgn(dc,region);
				DeleteObject(region);

				i = (pierwsza_probka*liczba_kanalow+numer_kanalu)*2;
				skok=2*liczba_kanalow;
				krok=1;

				SelectObject(dc,hDashPen);
				y1=pole.bottom-(int)floor((0.0-ymin)*skala_y+0.5);
				DrawLine(dc,40, y1 ,pole.right, y1 );
				SelectObject(dc,(HPEN)GetStockObject(BLACK_PEN));

				/////////////////////////// rysowanie zalene od trybu //////////////////////////

				if (m_plot_style & PLOTSTYLE_MODULE)
				{
					RECT r;
					for (x=0; x<m_visible_samples && i<calkowita_liczba_probek; x++)
					{
						r.left=(int)floor(x*m_x_scale+0.5)-1;
						r.right=r.left+3;
						re=probki[i];
						im=probki[i+1];
						y=sqrt(re*re + im*im);
						r.top=pole.bottom-(int)floor((y-ymin)*skala_y+0.5)-1;
						r.bottom=pole.bottom-(int)floor((0-ymin)*skala_y+0.5);
						FillRect(dc,&r,hFftBarBrush);
						i+=skok;
					}


				}
				else
				{


					if (m_plot_style & PLOTSTYLE_SIGNAL)
					{
						MoveTo(dc,0, pole.bottom-(int)floor((probki[i]-ymin)*skala_y+0.5));
						i+=skok;
						for (x=krok; x<m_visible_samples && i<calkowita_liczba_probek; i+=skok,x+=krok)
							LineTo(dc,(int)floor(x*m_x_scale+0.5),pole.bottom-(int)floor((probki[i]-ymin)*skala_y+0.5));
					}

					if (m_plot_style & PLOTSTYLE_COSIGNAL)
					{
						i = (pierwsza_probka*liczba_kanalow+numer_kanalu)*2 + 1;
						MoveTo(dc,0, pole.bottom-(int)floor((probki[i]-ymin)*skala_y+0.5));
						i+=skok;
						for (x=krok; x<m_visible_samples && i<calkowita_liczba_probek; i+=skok,x+=krok)
							LineTo(dc,(int)floor(x*m_x_scale+0.5),pole.bottom-(int)floor((probki[i]-ymin)*skala_y+0.5));
					}

					if (m_plot_style & PLOTSTYLE_ENVELOPE)
					{
						i = (pierwsza_probka*liczba_kanalow+numer_kanalu)*2;
						w1=probki[i];
						w2=probki[i+1];
						MoveTo(dc,0, pole.bottom-(int)floor((sqrt(w1*w1+w2*w2)-ymin)*skala_y+0.5));
						i+=skok;
						for (x=krok; x<m_visible_samples && i<calkowita_liczba_probek; i+=skok,x+=krok)
						{
							w1=probki[i];
							w2=probki[i+1];
							LineTo(dc,(int)floor(x*m_x_scale+0.5),pole.bottom-(int)floor((sqrt(w1*w1+w2*w2)-ymin)*skala_y+0.5));
						}

						i = (pierwsza_probka*liczba_kanalow+numer_kanalu)*2;
						w1=probki[i];
						w2=probki[i+1];
						MoveTo(dc,0, pole.bottom-(int)floor((-sqrt(w1*w1+w2*w2)-ymin)*skala_y+0.5));
						i+=skok;
						for (x=krok; x<m_visible_samples && i<calkowita_liczba_probek; i+=skok,x+=krok)
						{
							w1=probki[i];
							w2=probki[i+1];
							LineTo(dc,(int)floor(x*m_x_scale+0.5),pole.bottom-(int)floor((-sqrt(w1*w1+w2*w2)-ymin)*skala_y+0.5));
						}
					}
				}
				/////////////////////////// zaznaczenie bloku //////////////////////////

				if (m_begin_block!=m_end_block && m_sel_channel==numer_kanalu)
				{
					if (m_begin_block<m_end_block)
					{
						x1=(int)floor((m_begin_block-pierwsza_probka)*m_x_scale+0.5);
						x2=(int)floor((m_end_block-pierwsza_probka)*m_x_scale+0.5);
					}
					else
					{
						x1=(int)floor((m_end_block-pierwsza_probka)*m_x_scale+0.5);
						x2=(int)floor((m_begin_block-pierwsza_probka)*m_x_scale+0.5);
					}

					if (x2>=0 && x1<ekran.right)
					{
						HBRUSH hOldBrush=(HBRUSH)SelectObject(dc,CreateHatchBrush(HS_BDIAGONAL,RGB(0,0,0)));
						HPEN hOldPen=(HPEN)SelectObject(dc,CreatePen(PS_DOT,1,RGB(0,0,0)));
						Rectangle(dc,x1,pole.top,x2,pole.bottom);
						DeleteObject(SelectObject(dc,hOldPen));
						DeleteObject(SelectObject(dc,hOldBrush));
					}
				}
				StretchBlt(hDC,0,pasek.top,window_width,m_channel_height,dc,0,0,window_width,m_channel_height,SRCCOPY);
			}
			pasek.top += m_channel_height;
		}
	}

	if (pasek.top<ekran.bottom)
	{
		pasek.bottom=ekran.bottom;
		FillRect(hDC,&pasek,(HBRUSH)GetStockObject(WHITE_BRUSH));
	}

	SelectObject(dc,hOldPen);
	SelectObject(dc,hOldFont);
	SetBkMode(dc,old_bkMode);
	DeleteObject(SelectObject(dc,old_image));
	DeleteDC(dc);

	DeleteObject(hSeledinBrush);
	DeleteObject(hWhiteBrush);
	DeleteObject(hFftBarBrush);
	DeleteObject(hDashPen);
	DeleteObject(hGridPen);
	DeleteObject(hTitleBrush);
}

///////////////////////////////////////////
int CMeasureView::ChangeRange(int direction)
{
	int new_range;

	new_range = m_range;
	if (direction<0)
	{
		if (m_range>1) new_range >>= 1;
	}
	else
		if (direction>0)
		{
			if (m_range<256) new_range <<= 1;
		}
	if (new_range!=m_range)
	{
		m_range=new_range;
		RefreshScrollBars();
		Invalidate(FALSE);
	}
	return m_range;
}
///////////////////////////////////////////
int CMeasureView::ChangeZoomX(int direction)
{
	int new_zoom;

	if (m_doc)
	{
		new_zoom = m_visible_samples;
		if (direction>0)
		{
			if (new_zoom>16) new_zoom >>= 1;
		}
		else
			if (direction<0)
			{
				new_zoom <<= 1;
				if (new_zoom>m_doc->NumberOfSamples()) new_zoom=m_doc->NumberOfSamples();
			}
		if (new_zoom!=m_visible_samples)
		{
			m_visible_samples=new_zoom;
			RefreshScrollBars();
			Invalidate(FALSE);
		}
	}
	return m_visible_samples;
}
///////////////////////////////////////////
int CMeasureView::ChangeZoomY(int direction)
{
	int new_zoom;

	if (m_doc)
	{
		new_zoom = m_visible_channels;
		if (direction>0)
		{
			if (new_zoom>1) --new_zoom;
		}
		else
			if (direction<0)
			{
				if (new_zoom<m_doc->NumberOfChannels()) ++new_zoom;
			}
		if (new_zoom!=m_visible_channels)
		{
			m_visible_channels=new_zoom;
			RefreshScrollBars();
			Invalidate(FALSE);
		}
	}
	return m_visible_channels;
}
////////////////////////////////////////
void CMeasureView::SetScrollPos(int x, int y)
{
	SCROLLINFO si;

	si.cbSize=sizeof(SCROLLINFO);
	si.fMask=SIF_POS;
	si.nPos=x;
	SetScrollInfo(m_handle,SB_HORZ,&si,TRUE);
	si.nPos=y;
	SetScrollInfo(m_handle,SB_VERT,&si,TRUE);
}

///////////////////////////////////////
void CMeasureView::OnRButtonDown(int /*nKeys*/, int xpos, int ypos)
{
	int scroll_x_pos, scroll_y_pos;
	HMENU hMenu,hSubMenu;
	HINSTANCE hInst;
	POINT point;
	RECT r;

	point.x=xpos;
	point.y=ypos;

	CurrentScrollPos(scroll_x_pos, scroll_y_pos);
	xpos+=scroll_x_pos;
	ypos+=scroll_y_pos;
	m_rb_channel=klikniety_kanal(ypos);
	xy2sample(xpos,0,m_rb_sample);

	hInst = (HINSTANCE)GetWindowLong(m_handle,GWL_HINSTANCE);
	hMenu = LoadMenu(hInst,MAKEINTRESOURCE(IDR_MEASUREMENU));
	hSubMenu=GetSubMenu(hMenu,0);
	ClientToScreen(m_handle,&point);
	TrackPopupMenu(hSubMenu,TPM_RIGHTBUTTON,
			point.x, point.y, 0,
			GetParent(GetParent(GetParent(m_handle))),
			&r); // route commands through main window
	DestroyMenu(hMenu);
}

////////////////////////////////////////
void CMeasureView::CurrentScrollPos(int &x, int &y)
{
	SCROLLINFO si;

	si.cbSize=sizeof(SCROLLINFO);
	si.fMask=SIF_POS;
	GetScrollInfo(m_handle,SB_HORZ,&si);
	x= si.nPos;
	GetScrollInfo(m_handle,SB_VERT,&si);
	y= si.nPos;
}

////////////////////////////////////////
void CMeasureView::OnSize(UINT /*uType*/, UINT /*uWidth*/, UINT /*uHeight*/)
{
	RefreshScrollBars();
	Invalidate(FALSE);
}

////////////////////////////////////////
void CMeasureView::SetBeginOfBlock()
{
	if (m_sel_channel!=m_rb_channel)
	{
		m_sel_channel=m_rb_channel;
		m_end_block=m_rb_sample;
	}
	m_begin_block=m_rb_sample;
	UpdateBlockInfo();
	Invalidate(FALSE);
}

////////////////////////////////////////
void CMeasureView::SetEndOfBlock()
{
	if (m_sel_channel!=m_rb_channel)
	{
		m_sel_channel=m_rb_channel;
		m_begin_block=m_rb_sample;
	}
	m_end_block=m_rb_sample;
	UpdateBlockInfo();
	Invalidate(FALSE);
}

////////////////////////////////////////
void CMeasureView::SetBlock(int channel, int begin, int end)
{
	m_sel_channel=channel;
	m_begin_block=begin;
	m_end_block=end;
	UpdateBlockInfo();
	Invalidate(FALSE);
}

////////////////////////////////////////
void CMeasureView::SelectAll()
{
	m_sel_channel=m_rb_channel;
	m_begin_block=0;
	if (m_doc)	m_end_block=m_doc->NumberOfSamples(); else m_end_block=0;
	UpdateBlockInfo();
	Invalidate(FALSE);
}

////////////////////////////////////////
void CMeasureView::UpdateBlockInfo()
{
	double x;
	char napis[80];

	if (m_end_block==m_begin_block)
	{
		StringCbPrintf(napis,80,"%d: %d (%6.4f)",m_sel_channel,m_end_block,m_end_block/m_doc->GetRate());
	}
	else
	{
		x=(m_end_block-m_begin_block)/m_doc->GetRate();
		StringCbPrintf(napis,80,"%d: %d-%d (%6.4f, %6.4f)",m_sel_channel,m_begin_block,m_end_block,x,1.0/x);
	}
	SendMessage(g_statusbar,SB_SETTEXT,2,(LPARAM)napis);
}

///////////////////////////////////////
LRESULT CMeasureView::OnCommand(WPARAM wParam, LPARAM lParam)
{
	CSelection Selection;

	switch (LOWORD(wParam))
	{
	case ID_RANGE_IN: ChangeRange(1); break;
	case ID_RANGE_OUT: ChangeRange(-1); break;
	case ID_ZOOM_X_IN: ChangeZoomX(1); break;
	case ID_ZOOM_X_OUT: ChangeZoomX(-1); break;
	case ID_ZOOM_Y_IN: ChangeZoomY(1); break;
	case ID_ZOOM_Y_OUT: ChangeZoomY(-1); break;
	case IDM_GETSELECTION: GetSelection((CSelection *)lParam); break;
	case IDM_GETSTATE: GetState((int *)lParam); break;

	case ID_EDIT_COPY:
		GetSelection(&Selection);
		if (Selection.m_from!=Selection.m_to)
			m_doc->CopyToClipboard(Selection.m_channel,Selection.m_from,Selection.m_to);
		break;

	case ID_PLOT_SIGNAL: SetPlotStyle(PLOTSTYLE_SIGNAL); RefreshScrollBars(); Invalidate(FALSE); break;
	case ID_PLOT_ENVELOPE: SetPlotStyle(PLOTSTYLE_SIGENV); RefreshScrollBars(); Invalidate(FALSE); break;
	case ID_PLOT_COSIGNAL: SetPlotStyle(PLOTSTYLE_BOTH); RefreshScrollBars(); Invalidate(FALSE); break;
	case ID_PLOT_ALL: SetPlotStyle(PLOTSTYLE_BOTHENV); RefreshScrollBars(); Invalidate(FALSE); break;
	case ID_PLOT_MODULE: SetPlotStyle(PLOTSTYLE_MODULE); RefreshScrollBars(); Invalidate(FALSE); break;

	case ID_ADD_COMPONENT: AddComponent(); break;
	case ID_BEGIN_BLOCK: SetBeginOfBlock(); break;
	case ID_END_BLOCK: SetEndOfBlock(); break;
	case ID_SELECT_ALL: SelectAll(); break;
	}
	return CWnd::OnCommand(wParam, lParam);
}

///////////////////////////////////////////////////////////////
// CMeasureWnd
///////////////////////////////////////////////////////////////

CMeasureWnd::CMeasureWnd()
{
	m_doc=NULL;
	m_display=NULL;
}

CMeasureWnd::~CMeasureWnd()
{
	if (m_doc) delete m_doc;
}

int CMeasureWnd::Create(int x, int y, int w, int h, HWND parent, DWORD style)
{
	CreateMDIWindow(
		"MiwilMeasureClass",
		"Measure Data",
		style,
		x, y, w, h,
		parent,
		(HINSTANCE)GetWindowLong(parent,GWL_HINSTANCE),
		(LPARAM)this
		);
	if (!m_handle) return 1;
	return 0;
}

///////////////////////////////////////
int CMeasureWnd::OnCreate(LPCREATESTRUCT /*lpCreateStruct*/)
{
	RECT r;

	GetClientRect(m_handle,&r);
	m_display = new CMeasureView(PLOTSTYLE_SIGNAL);
	if (m_display->Create(0,0,r.right,r.bottom,m_handle,WS_CLIPCHILDREN|WS_CHILD|WS_VISIBLE)) return 1;
	return 0;
}

///////////////////////////////////////
void CMeasureWnd::OnDestroy()
{
	if (m_display) delete m_display;
	CMDIChildWnd::OnDestroy();
}

///////////////////////////////////////
void CMeasureWnd::OnSize(UINT /*uType*/, UINT width, UINT height)
{
	m_display->Move(0,0,width,height,TRUE);
	m_display->Invalidate(FALSE);
}

///////////////////////////////////////
LRESULT CMeasureWnd::OnMDIActivate(HWND last, HWND next)
{
	return DefMDIChildProc(m_handle, WM_MDIACTIVATE, (WPARAM)last, (LPARAM)next);
}

///////////////////////////////////////
LRESULT CMeasureWnd::OnCommand(WPARAM wParam, LPARAM lParam)
{
	CSelection Selection;

	switch (LOWORD(wParam))
	{
	case ID_RANGE_IN:
	case ID_RANGE_OUT:
	case ID_ZOOM_X_IN:
	case ID_ZOOM_X_OUT:
	case ID_ZOOM_Y_IN:
	case ID_ZOOM_Y_OUT:
	case IDM_GETSELECTION:
	case ID_EDIT_COPY:
	case ID_PLOT_SIGNAL:
	case ID_PLOT_COSIGNAL:
	case ID_PLOT_ENVELOPE:
	case ID_PLOT_ALL:
	case ID_PLOT_MODULE:
	case ID_BEGIN_BLOCK:
	case ID_END_BLOCK:
	case ID_SELECT_ALL:
	case IDM_GETSTATE:
	case ID_ADD_COMPONENT:
		return SendMessage(m_display->m_handle,WM_COMMAND,wParam,lParam);

	case ID_BLOCK_INFO:
		m_display->GetSelection(&Selection);
		if (Selection.m_from!=Selection.m_to)
		{
			CParametrySygnaluDlg dlg(m_doc->GetSignalParams(Selection.m_channel),&Selection);
			dlg.Execute(m_handle);
		}
		break;

	case ID_BLOCK_SET:
		{
			m_display->GetSelection(&Selection);
			CBlockDlg dlg(&Selection);
			dlg.Execute(m_handle);
			m_display->SetBlock(Selection.m_channel,Selection.m_from,Selection.m_to);
		}
		break;

	case ID_BLOCK_CLEAR:
		m_display->GetSelection(&Selection);
		if (Selection.m_from!=Selection.m_to)
		{
			m_doc->Erease(Selection.m_channel,Selection.m_from,Selection.m_to);
			m_display->DocumentChanged();
		}
		break;

	case ID_INFO:
		{
			m_display->GetRightSelection(&Selection);
			CParametrySygnaluDlg dlg(m_doc->GetSignalParams(Selection.m_channel),&Selection);
			dlg.Execute(m_handle);
		}
		break;

	case IDM_GETDOCUMENT:
		*((CMeasure **)lParam) = m_doc;
		break;
	}
	return CMDIChildWnd::OnCommand(wParam, lParam);
}

