#include <windows.h>
#include <commctrl.h>
#include <strsafe.h>
#include <time.h>
#include <math.h>

#include "matrix.h"
#include "app.h"
#include "winfi32.h"
#include "resource.h"
#include "measure.h"
#include "calc.h"
#include "gener.h"

#define g_gravity 9.81


#define GENERATORDOC(a) ((CGenerator *)a)

/////////////////////////////////////////////////////////////////////////////
// CGenerator

CGenerator::CGenerator(CParametryGeneracji &dane) : CMeasure()
{
	m_dane=dane;
}



// n to stopien rozniczkowalnosci
static double *CreateLDA(int n, double eta)
{
	double *a;
	double ab[2];
	int k;

	a=alloc_matrix(n);
	clear_matrix(n,a);
	ab[0]=eta; ab[1]=-eta;
	a[n-1]=1;	//pierwszy wiersz zrobiony
	for (k=1; k<n; k++)
		conv(a+k*n+n-1-k, k, a+(k-1)*n+n-1-k+1, 2, ab);
	return a;
}

static double *CreateFIA(int n, double dt, double eta)
{
	double *a;
	int i,j;

	a=alloc_matrix(n);
	clear_matrix(n,a);

	*a=exp(-eta*dt);
	for (i=1; i<n; i++)
		*(a+i*n) = *(a+(i-1)*n) * eta*dt/i;
	for (j=1; j<n; j++)
		for (i=j; i<n; i++)
			*(a+i*n+j) = *(a+(i-1)*n+(j-1));
	return a;
}

static double power(double x, double y)
{
	return exp(y*log(x));
}

static double  *CreatePDA(int n, double pnn)
{
	double *a;
	double s;
	int k,r;

	a=alloc_matrix(n);
	clear_matrix(n,a);

	a[0]=1.0;
	for (k=1; k<n; k++)
	{
		s = power(2,-k);
		a[k*n] = s;
		a[k] = s;
	}
	for (r=1; r<n; r++)
		for (k=r; k<n; k++)
		{
			s = (a[(r-1)*n+k]+a[r*n+(k-1)])/2;
			a[r*n+k] = s;
			a[k*n+r] = s;
		}
	scalar_mul_matrix(n,a,pnn/a[n*n-1],a);
	return a;
}

static double randn()
{
	int i;
	double a;

	a=0;
	for (i=0; i<12; i++)
		a += rand() ;
	return (a/RAND_MAX)-6.0;
}

static void randn_vector(int n, double *a)
{
	while (n--)
		*(a++) = randn();
}


static const char *g_unit="cm";
static const char *g_diff_unit="cm/s";
static const char *g_name="Generated";

///////////////////////////////////////////
void CGenerator::Init()
{
	char napis[80];

	m_format=6;
	SetDateTime();
	m_channels=2;
	StringCbCopy(m_nazwa,61,g_name);
	m_rate=1.0/m_dane.dt;

	double KH=2*M_PI*m_dane.h/m_dane.L;
	double omega=sqrt(g_gravity/m_dane.h*KH*tanh(KH));
	m_dane.m_T=2*M_PI/omega;
	//double eta=m_dane.etaT/m_dane.m_T;
	double nper=m_dane.m_T/m_dane.dt;
	m_samples=(int)floor((m_dane.n1+m_dane.n2+m_dane.n3)*nper+1.5);

	StringCbCopy(napis,80,g_name);
	m_opis[0].Init(1.0,0.0,g_unit,napis);
	m_parametry[0].Init();
	StringCbPrintf(napis,80,"%s diff",g_name);
	m_opis[1].Init(1.0,0.0,g_diff_unit,napis);
	m_parametry[1].Init();

	AllocBuffer((m_samples*m_channels)<<3);
}

///////////////////////////////////////////
void CGenerator::CalCGenerator()
{
	int i,np1,np2,np3,r;
	double KH,omega,eta,nper,xa,xd,pnn,ca,a,d,da,dd;
	double *LDA, *LDAI, *FIA, *TMP;
	double *XAP, *XDP, *RND, *QA;
	double *WA, *DWA, *WD, *DWD;
	double zn1re,zn1im,zn2re,zn2im,zn3re,zn3im;

	Init();
	srand( (unsigned)time( NULL ) );

	if (!m_buffer) return;
	float *wyniki=m_buffer;
	KH=2*M_PI*m_dane.h/m_dane.L;
	omega=sqrt(g_gravity/m_dane.h*KH*tanh(KH));
	m_dane.m_T=2*M_PI/omega;
	eta=m_dane.etaT/m_dane.m_T;
	nper=m_dane.m_T/m_dane.dt;
	np1=(int)floor(m_dane.n1*nper+0.5);
	np2=(int)floor(m_dane.n2*nper+0.5);
	np3=(int)floor(m_dane.n3*nper+0.5);
	pnn=1;
	ca=sqrt(M_PI/2);

	switch (m_dane.model)
	{
	case 0:
		xa=power(eta,m_dane.m_n+1)/omega/(m_dane.m_n+1);
		xa=xa/(1+eta*eta/omega/omega/(m_dane.m_n+1)/(m_dane.m_n+1));
		xd=eta/omega/(m_dane.m_n+2)*xa;
		break;

	case 1:
		xa=power(eta,m_dane.m_n)*(m_dane.m_n+2)/(m_dane.m_n+1)/2.0;
		xd=xa;
		break;

	case 3:
		xa=m_dane.xa;
		xd=0;
		break;

	default:
		xa=0;
		xd=0;
	}

	LDA=CreateLDA(m_dane.m_n+1,eta);
	LDAI=alloc_matrix(m_dane.m_n+1);
	FIA=CreateFIA(m_dane.m_n+1,m_dane.dt,eta);
	TMP=alloc_vector(m_dane.m_n+1);
	XAP=alloc_vector(m_dane.m_n+1);
	XDP=alloc_vector(m_dane.m_n+1);
	WA=alloc_vector(1+np1+np2+np3);
	WD=alloc_vector(1+np1+np2+np3);
	DWA=alloc_vector(1+np1+np2+np3);
	DWD=alloc_vector(1+np1+np2+np3);
	r=0;

	switch (m_dane.model)
	{
	case 0:
	case 1:
	case 2:
	case 3:
	case 5:
	case 6:
		clear_vector(m_dane.m_n+1,XAP); XAP[m_dane.m_n]=1.0;
		copy_matrix(m_dane.m_n+1,LDAI,LDA);
		Eliminacja(m_dane.m_n+1,LDAI, XAP);

		clear_vector(m_dane.m_n+1,XDP); XDP[0]=1.0;
		copy_matrix(m_dane.m_n+1,LDAI,LDA);
		Eliminacja(m_dane.m_n+1,LDAI, XDP);

		WA[r]=0.0;
		WD[r]=0.0;
		DWA[r]=0.0;
		DWD[r]=0.0;
		++r;
		for (i=0; i<np1+np2; i++)
		{
			matrix_mul_vector(m_dane.m_n+1,TMP,FIA,XAP);
			copy_vector(m_dane.m_n+1,XAP,TMP);
			matrix_mul_vector(m_dane.m_n+1,TMP,FIA,XDP);
			copy_vector(m_dane.m_n+1,XDP,TMP);
			a = XAP[m_dane.m_n];
			da = vectorT_mul_vector(m_dane.m_n+1,LDA+4,XAP);
			d = 1 - XDP[m_dane.m_n];
			dd = - vectorT_mul_vector(m_dane.m_n+1,LDA+4,XDP);
			WA[r] = xa*a;
			WD[r] = d + xd*a;
			DWA[r] = xa*da;
			DWD[r] = dd + xd*da;
			++r;
		}

		clear_vector(m_dane.m_n+1,TMP); TMP[0]=1.0;
		copy_matrix(m_dane.m_n+1,LDAI,LDA);
		Eliminacja(m_dane.m_n+1,LDAI,TMP);
		vector_rsub_vector(m_dane.m_n+1,XDP,TMP);

		for (i=0; i<np3; i++)
		{
			matrix_mul_vector(m_dane.m_n+1,TMP,FIA,XAP);
			copy_vector(m_dane.m_n+1,XAP,TMP);
			matrix_mul_vector(m_dane.m_n+1,TMP,FIA,XDP);
			copy_vector(m_dane.m_n+1,XDP,TMP);
			a = XAP[m_dane.m_n];
			da = vectorT_mul_vector(m_dane.m_n+1,LDA+4,XAP);
			d = XDP[m_dane.m_n];
			dd = vectorT_mul_vector(m_dane.m_n+1,LDA+4,XDP);
			WA[r] = xa*a;
			WD[r] = d + xd*a;
			DWA[r] = xa*da;
			DWD[r] = dd + xd*da;
			++r;
		}

		break;

	case 4:

		QA=CreatePDA(m_dane.m_n+1,pnn);
		calc_sqrform(m_dane.m_n+1,QA,FIA);
		matrix_add_matrixT_div_2(m_dane.m_n+1,QA);
		Choleski(m_dane.m_n+1,QA);
		RND=alloc_vector(m_dane.m_n+1);

		clear_vector(m_dane.m_n+1,XAP); XAP[m_dane.m_n]=ca;
		copy_matrix(m_dane.m_n+1,LDAI,LDA);
		Eliminacja(m_dane.m_n+1,LDAI, XAP);

		clear_vector(m_dane.m_n+1,XDP); XDP[0]=ca;
		copy_matrix(m_dane.m_n+1,LDAI,LDA);
		Eliminacja(m_dane.m_n+1,LDAI, XDP);

		WA[r]=0.0;
		WD[r]=0.0;
		DWA[r]=0.0;
		DWD[r]=0.0;
		++r;
		for (i=0; i<np1; i++)
		{
			matrix_mul_vector(m_dane.m_n+1,TMP,FIA,XAP);
			copy_vector(m_dane.m_n+1,XAP,TMP);
			matrix_mul_vector(m_dane.m_n+1,TMP,FIA,XDP);
			copy_vector(m_dane.m_n+1,XDP,TMP);
			a = XAP[m_dane.m_n];
			da = vectorT_mul_vector(m_dane.m_n+1,LDA+4,XAP);
			d = ca - XDP[m_dane.m_n];
			dd = - vectorT_mul_vector(m_dane.m_n+1,LDA+4,XDP);
			WA[r] = xa*a;
			WD[r] = d + xd*a;
			DWA[r] = xa*da;
			DWD[r] = dd + xd*da;
			++r;
		}

		clear_vector(m_dane.m_n+1,TMP); TMP[0]=ca;
		copy_matrix(m_dane.m_n+1,LDAI,LDA);
		Eliminacja(m_dane.m_n+1,LDAI,TMP);
		vector_rsub_vector(m_dane.m_n+1,XDP,TMP);
		for (i=0; i<m_dane.m_n+1; i++)
		{
			XDP[i]+=xd*XAP[i];
			XAP[i]*=xa;
		}

		for (i=0; i<np2; i++)
		{
			matrix_mul_vector(m_dane.m_n+1,TMP,FIA,XAP);
			randn_vector(m_dane.m_n+1,RND);
			matrix_mul_vector(m_dane.m_n+1,XAP,QA,RND);
			vector_add_vector(m_dane.m_n+1,XAP,TMP);
			matrix_mul_vector(m_dane.m_n+1,TMP,FIA,XDP);
			randn_vector(m_dane.m_n+1,RND);
			matrix_mul_vector(m_dane.m_n+1,XDP,QA,RND);
			vector_add_vector(m_dane.m_n+1,XDP,TMP);
			WA[r] = XAP[m_dane.m_n];
			DWA[r] = vectorT_mul_vector(m_dane.m_n+1,LDA+4,XAP);
			WD[r] = XDP[m_dane.m_n];
			DWD[r] = vectorT_mul_vector(m_dane.m_n+1,LDA+4,XDP);
			++r;
		}

		for (i=0; i<np3; i++)
		{
			matrix_mul_vector(m_dane.m_n+1,TMP,FIA,XAP);
			copy_vector(m_dane.m_n+1,XAP,TMP);
			matrix_mul_vector(m_dane.m_n+1,TMP,FIA,XDP);
			copy_vector(m_dane.m_n+1,XDP,TMP);
			WA[r] = XAP[m_dane.m_n];
			DWA[r] = vectorT_mul_vector(m_dane.m_n+1,LDA+4,XAP);
			WD[r] = XDP[m_dane.m_n];
			DWD[r] = vectorT_mul_vector(m_dane.m_n+1,LDA+4,XDP);
			++r;
		}

		free(QA); free(RND);
		break;
	}

	omega*=m_dane.dt;
	m_samples=1+np1+np2+np3;
	if (m_dane.model==5)	//obliczanie skladowych
	{

		for (i=0;i<m_samples;i++)
		{
			zn1re=WA[i] * cos(omega*i) + WD[i] * sin(omega*i);
			zn1im=WD[i] * cos(omega*i) - WA[i] * sin(omega*i);
			zn2re=zn1re*zn1re - zn1im*zn1im;
			zn2im=2*zn1re*zn1im;
			zn3re=zn2re*zn1re - zn2im*zn1im;
			zn3im=zn2re*zn1im + zn2im*zn1re;

			wyniki[4*i]= (float)(m_dane.w1*zn1re + m_dane.w2*zn2im - m_dane.w3*zn3re);
			wyniki[4*i+1]= (float)(m_dane.w1*zn1im - m_dane.w2*zn2re + m_dane.w3*zn3im);
			wyniki[4*i+2]= (float)0.0;
			wyniki[4*i+3]= (float)0.0;
			//m_kpoints[i]= m_dane.w1*zn1im - m_dane.w2*zn2re + m_dane.w3*zn3im;
			//m_kpoints[i]= sqrt(m_dane.w1*m_dane.w1*(zn1re*zn1re+zn1im*zn1im) + m_dane.w2*m_dane.w2*(zn2re*zn2re+zn2im*zn2im) + m_dane.w3*m_dane.w3*(zn3re*zn3re+zn3im*zn3im));
			//wyniki[4*i+1]= fabs(m_dane.w1)*sqrt(zn1re*zn1re+zn1im*zn1im) + fabs(m_dane.w2)*sqrt(zn2re*zn2re+zn2im*zn2im) + fabs(m_dane.w3)*sqrt(zn3re*zn3re+zn3im*zn3im);
		}
	}
	else
		if (m_dane.model==6)
		{
			//double vg=m_dane.L/m_T/2.0;
			double domega=omega/m_dane.nw/2;
			double dwa,edz;

			for (i=0;i<m_samples;i++)
			{
				dwa=cos(domega*i+m_dane.fio);
				edz=1+m_dane.cr*dwa;
				WA[i]=WA[i]*edz;
				WD[i]=WD[i]*edz;
				zn1re=WA[i] * cos(omega*i) + WD[i] * sin(omega*i);
				zn1im=WD[i] * cos(omega*i) - WA[i] * sin(omega*i);
				wyniki[4*i]= (float)zn1re;
				wyniki[4*i+1]= (float)zn1im;
				wyniki[4*i+2]= (float)0.0;
				wyniki[4*i+3]= (float)0.0;
				//m_kpoints[i]= - WA[i] * sin(omega*i) + WD[i] * cos(omega*i);
				//wyniki[4*i+1]= (float)sqrt(zn1re*zn1re+zn1im*zn1im);
			}
		}
		else	//pozostae modele
		{
			for (i=0;i<m_samples;i++)
			{
				zn1re=WA[i] * cos(omega*i) + WD[i] * sin(omega*i);
				zn1im=WD[i] * cos(omega*i) - WA[i] * sin(omega*i);
				wyniki[4*i]= (float)zn1re;
				wyniki[4*i+1]= (float)zn1im;
				//m_kpoints[i]= - WA[i] * sin(omega*i) + WD[i] * cos(omega*i);
				wyniki[4*i+2]= (float)(DWA[i] * cos(omega*i) + DWD[i] * sin(omega*i)
					+ (omega/m_dane.dt)*( WD[i] * cos(omega*i) - WA[i] * sin(omega*i) ));
				wyniki[4*i+3]=wyniki[4*i+2];
			}
		}

	if (m_dane.model==5 || m_dane.model==6) policz_pochodna();

	//normalizacja
	//m_max_val=m_points[0];
	//for (i=1;i<m_number_of_points;i++)
	//	if (m_points[i]>m_max_val)
	//		m_max_val=m_points[i];
	//m_min_val=m_points[0];
	//for (i=1;i<m_number_of_points;i++)
	//	if (m_points[i]<m_min_val)
	//		m_min_val=m_points[i];
	//if (fabs(m_max_val)>fabs(m_min_val))
	//	m_max_abs=fabs(m_max_val);
	//else
	//	m_max_abs=fabs(m_min_val);

	//s=m_dane.A/m_max_abs;
	//for (i=0;i<m_number_of_points;i++)
	//{
	//	m_points[i]*=s;
	//	m_kpoints[i]*=s;
	//	m_dpoints[i]*=s;
	//}
	//m_max_abs=m_dane.A;

	//zwalnianie pamieci potrzebnej na macierze i wektory
	free(LDA); free(LDAI); free(FIA); free(TMP);
	free(XAP);	free(XDP);
	free(WA); free(WD); free(DWA); free(DWD);
}

/////////////////////////////////////////////////////////////////////////////
// CParametryGeneratoraDlg

static const char nazwa_typu[7][30] =
{
	"xa=..., xd=...",
	"xa=..., xd=xa",
	"xa=0, xd=0",
	"xa=0.3, xd=0",
	"random",
	"Stokes",
	"with groups"
};

BOOL CParametryGeneratoraDlg::OnInitDialog(HWND hWnd)
{
	int i;
	//char napis[80];

	CDialog::OnInitDialog(hWnd);

	SetInteger(IDC_N1,m_new_params.n1);
	SetInteger(IDC_N2,m_new_params.n2);
	SetInteger(IDC_N3,m_new_params.n3);
	SetDouble(IDC_DT,m_new_params.dt);
	SetDouble(IDC_L,m_new_params.L);
	SetDouble(IDC_H,m_new_params.h);
	SetDouble(IDC_ETA,m_new_params.etaT);
	SetDouble(IDC_XA,m_new_params.xa);
	SetDouble(IDC_W1,m_new_params.w1);
	SetDouble(IDC_W2,m_new_params.w2);
	SetDouble(IDC_W3,m_new_params.w3);
	SetDouble(IDC_FIO,m_new_params.fio);
	SetDouble(IDC_CR,m_new_params.cr);
	SetDouble(IDC_NW,m_new_params.nw);

	for (i=0; i<7; ++i)
		SendMessage(IDC_MODEL,CB_ADDSTRING,0,(LPARAM) (LPCTSTR) nazwa_typu[i]);
	SendMessage(IDC_MODEL,CB_SETCURSEL,m_new_params.model,0);

	return TRUE;
}

BOOL CParametryGeneratoraDlg::OnOk()
{
	m_new_params.model=SendMessage(IDC_MODEL,CB_GETCURSEL,0,0);
	GetInteger(IDC_N1,&m_new_params.n1);
	GetInteger(IDC_N2,&m_new_params.n2);
	GetInteger(IDC_N3,&m_new_params.n3);
	GetDouble(IDC_DT,&m_new_params.dt);
	GetDouble(IDC_L,&m_new_params.L);
	GetDouble(IDC_H,&m_new_params.h);
	GetDouble(IDC_ETA,&m_new_params.etaT);
	GetDouble(IDC_XA,&m_new_params.xa);
	GetDouble(IDC_W1,&m_new_params.w1);
	GetDouble(IDC_W2,&m_new_params.w2);
	GetDouble(IDC_W3,&m_new_params.w3);
	GetDouble(IDC_FIO,&m_new_params.fio);
	GetDouble(IDC_CR,&m_new_params.cr);
	GetDouble(IDC_NW,&m_new_params.nw);
	*m_params=m_new_params;
	return CDialog::OnOk();
}

BOOL CParametryGeneratoraDlg::OnCommand(int id, int /*notify*/, HWND /*control*/)
{
	switch (id)
	{
	case IDC_MODEL :
		{
			int sel=SendMessage(IDC_MODEL,CB_GETCURSEL,0,0);
			Enable(IDC_XA,sel==3);
			Enable(IDC_W1,sel==5);
			Enable(IDC_W2,sel==5);
			Enable(IDC_W3,sel==5);
			Enable(IDC_FIO,sel==6);
			Enable(IDC_CR,sel==6);
			Enable(IDC_NW,sel==6);
		}
		break;
	case IDC_COPY :
		copy_all();
		break;
	}
	return TRUE;
}

void CParametryGeneratoraDlg::copy_all()
{
	HGLOBAL clipboard;
	char *block;

	if (OpenClipboard(NULL))
	{
		EmptyClipboard();
		clipboard = GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE|GMEM_DISCARDABLE, 256);
		block = (char *)GlobalLock(clipboard);

		StringCbPrintf(block,256,"%d\t%d\t%d\r\n",
			m_new_params.n1,
			m_new_params.n2,
			m_new_params.n3);
		GlobalUnlock(clipboard);
		SetClipboardData(CF_TEXT, clipboard);
		CloseClipboard();
	}
}

///////////////////////////////////////////////////////////////
// CGeneratorWnd
///////////////////////////////////////////////////////////////

int CGeneratorWnd::Create(int x, int y, int w, int h, HWND parent, DWORD style)
{
	CreateMDIWindow(
		"MiwilGeneratorClass",
		"Generator Analisys",
		style,
		x, y, w, h,
		parent,
		(HINSTANCE)GetWindowLong(parent,GWL_HINSTANCE),
		(LPARAM)this
		);
	if (!m_handle) return 1;
	return 0;
}

int CGeneratorWnd::OnCreate(LPCREATESTRUCT /*create_struct*/)
{
	RECT r;

	GetClientRect(m_handle,&r);
	m_display = new CMeasureView(PLOTSTYLE_SIGNAL);
	if (m_display->Create(0,0,r.right,r.bottom,m_handle,WS_CLIPCHILDREN|WS_CHILD|WS_VISIBLE)) return 1;
	return 0;
}

void CGeneratorWnd::SetDocument(CMeasure *doc)
{
	char nazwa[286];

	GENERATORDOC(doc)->CalCGenerator();
	CMeasureWnd::SetDocument(doc);
	StringCbCopy(nazwa,286,"Generated");
	SetTitle(nazwa);
}

LRESULT CGeneratorWnd::OnCommand(WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam))
	{
	case ID_ANALYSE_RECALC:
		{
			CParametryGeneratoraDlg dlg(GENERATORDOC(m_doc)->GetParams());
			if (dlg.Execute(m_handle)==IDOK)
			{
				GENERATORDOC(m_doc)->CalCGenerator();
				m_display->DocumentChanged();
			}
		}
		break;
	}
	return CMeasureWnd::OnCommand(wParam, lParam);
}

