/*
 ************************************************************************
 *  Program     : Winfi32.exe                                           *
 *  Description : Winfiltr for Windows                                  *
 *  Revision    : 1.10                                                  *
 *  Date        : 26.04.1998                MIWIL Uslugi Informatyczne  *
 ************************************************************************
*/

#include <windows.h>
#include <string.h>
#include <commctrl.h>
#include <time.h>
#include <strsafe.h>
#include <ctype.h>

#include "app.h"
#include "winfi32.h"
#include "resource.h"
#include "measure.h"
#include "calc.h"
#include "gener.h"
#include "fft.h"
#include "kalman.h"
#include "aprox.h"

static char filtr_nazw[]="PCAD 1.0/1.2/1.4/1.5 File (*.INT)\0*.INT\0Wave syntesiser (*.M11)\0*.M11\0Plik tekstowy (*.TXT)\0*.TXT\0Plik dwikowy (*.WAV)\0*.WAV\0All files (*.*)\0*.*\0";
static char custom_filter[80];
static char file_name[512];
static char file_title[256];
static char default_ext[26]="INT";
static OPENFILENAME open_file_dlg= {sizeof(OPENFILENAME),0,0,filtr_nazw,custom_filter,80,1,file_name,512,file_title,256,NULL,NULL,OFN_EXPLORER|OFN_FILEMUSTEXIST,0,0,default_ext,0,NULL,NULL};

static char save_filtr_nazw[]="PCAD 1.5 File (*.INT)\0*.INT\0Wave syntesiser (*.M11)\0*.M11\0Plik tekstowy (*.TXT)\0*.TXT\0";
static char save_filtr_nazw2[]="Plik tekstowy (*.TXT)\0*.TXT\0";
static OPENFILENAME save_file_dlg= {sizeof(OPENFILENAME),0,0,save_filtr_nazw,custom_filter,80,1,file_name,512,file_title,256,NULL,NULL,OFN_EXPLORER|OFN_OVERWRITEPROMPT|OFN_HIDEREADONLY,0,0,default_ext,0,NULL,NULL};


static void usun_rozszerzenie()
{
	int len=strlen(file_name);
	if (len>4)
		if (file_name[len-4]=='.' &&
		  toupper(file_name[len-3])=='I' &&
		  toupper(file_name[len-2])=='N' &&
		  toupper(file_name[len-1])=='T' ||
			file_name[len-4]=='.' &&
		  toupper(file_name[len-3])=='T' &&
		  toupper(file_name[len-2])=='X' &&
		  toupper(file_name[len-1])=='T' ||
			file_name[len-4]=='.' &&
		  toupper(file_name[len-3])=='W' &&
		  toupper(file_name[len-2])=='A' &&
		  toupper(file_name[len-1])=='V' ||
			file_name[len-4]=='.' &&
		  toupper(file_name[len-3])=='M' &&
		  toupper(file_name[len-2])=='1' &&
		  toupper(file_name[len-1])=='1')
			file_name[len-4]='\0';
}


char gszWinfiltrRegisterPath[]="Software\\MIWIL\\Winfiltr\\3.0";

CFont GlobalSmallFont;

/////////////////////////////////////////////////////////////////////////////
// CChannelInfoDlg

CDialog AboutDlg;

/////////////////////////////////////////////////////////////////////////////
// CFftOptionsDlg

class CFftOptionsDlg : public CDialog
{
	CParametryFouriera *m_params;
	CParametryFouriera m_new_params;

public:
	CFftOptionsDlg(CParametryFouriera *params) : CDialog() { m_params=params; m_new_params=*params; }
	virtual BOOL OnInitDialog(HWND hWnd);
	virtual BOOL OnOk();
};


BOOL CFftOptionsDlg::OnInitDialog(HWND hWnd)
{
	char napis[80];

	CDialog::OnInitDialog(hWnd);
	SetCheck(IDC_TUNED,m_new_params.m_tuned);
	StringCbPrintf(napis,80,"%8.6f",m_new_params.m_tune_fr);
	SetText(IDC_TUNEFR,napis);
	SetCheck(IDC_LIMIT,m_new_params.m_limited);
	StringCbPrintf(napis,80,"%8.6f",m_new_params.m_limit);
	SetText(IDC_LIMITFR,napis);
	SetCheck(IDC_PHASE,m_new_params.m_phase);
	SetCheck(IDC_LINEPLOT,m_new_params.m_line_plot);
	return TRUE;
}

BOOL CFftOptionsDlg::OnOk()
{
	GetCheck(IDC_TUNED,&m_new_params.m_tuned);
	GetLimitedDouble(IDC_TUNEFR,&m_new_params.m_tune_fr,0.0,999999.0);
	GetCheck(IDC_LIMITED,&m_new_params.m_limited);
	GetLimitedDouble(IDC_LIMITFR,&m_new_params.m_limit,0.0,999999.0);
	GetCheck(IDC_PHASE,&m_new_params.m_phase);
	GetCheck(IDC_LINEPLOT,&m_new_params.m_line_plot);
	*m_params=m_new_params;
	return CDialog::OnOk();
}

/////////////////////////////////////////////////////////////////////////////
// CKalmanOptionsDlg

class CKalmanOptionsDlg : public CDialog
{
	CParametryKalmana *m_params;
	CParametryKalmana m_new_params;

public:
	CKalmanOptionsDlg(CParametryKalmana *params) : CDialog() { m_params=params; m_new_params=*params; }
	virtual BOOL OnInitDialog(HWND hWnd);
	virtual BOOL OnOk();
};


BOOL CKalmanOptionsDlg::OnInitDialog(HWND hWnd)
{
	int i;
	char napis[80];

	CDialog::OnInitDialog(hWnd);
	for (i=0; i<LICZBA_FILTROW; ++i)
		SendMessage(IDC_FTYPE,CB_ADDSTRING,0,(LPARAM) (LPCTSTR) nazwa_typu[i]);
	SendMessage(IDC_FTYPE,CB_SETCURSEL,m_new_params.m_filter_type,0);
	SetCheck(IDC_ESTOBSER,m_new_params.m_estimate_error);
	StringCbPrintf(napis,80,"%3.1f",m_new_params.m_observation_error);
	SetText(IDC_OBERR,napis);
	StringCbPrintf(napis,80,"%3.1f",m_new_params.m_constant_period);
	SetText(IDC_TO,napis);
	StringCbPrintf(napis,80,"%9.7f",m_new_params.m_asymptotic);
	SetText(IDC_ASYMPTO,napis);
	return TRUE;
}

BOOL CKalmanOptionsDlg::OnOk()
{
	m_new_params.m_filter_type=SendMessage(IDC_FTYPE,CB_GETCURSEL,0,0);
	GetCheck(IDC_ESTOBSER,&m_new_params.m_estimate_error);
	GetLimitedDouble(IDC_OBERR,&m_new_params.m_observation_error,0.0,100.0);
	GetLimitedDouble(IDC_TO,&m_new_params.m_constant_period,0.0,100.0);
	GetLimitedDouble(IDC_ASYMPTO,&m_new_params.m_asymptotic,0.0,100.0);
	*m_params=m_new_params;
	return CDialog::OnOk();
}

/////////////////////////////////////////////////////////////////////////////
// CAproxOptionsDlg

class CAproxOptionsDlg : public CDialog
{
	CParametryAproksymacji *m_params;
	CParametryAproksymacji m_new_params;

public:
	CAproxOptionsDlg(CParametryAproksymacji *params) : CDialog() { m_params=params; m_new_params=*params; }
	virtual BOOL OnInitDialog(HWND hWnd);
	virtual BOOL OnOk();
};


BOOL CAproxOptionsDlg::OnInitDialog(HWND hWnd)
{
	CDialog::OnInitDialog(hWnd);
	SetDouble(IDC_EPSILON,m_new_params.m_epsilon);
	SetDouble(IDC_BREAK,m_new_params.m_break);
	SetDouble(IDC_ZERO,m_new_params.m_zero);
	return TRUE;
}

BOOL CAproxOptionsDlg::OnOk()
{
	GetLimitedDouble(IDC_EPSILON,&m_new_params.m_epsilon,0.0,100.0);
	GetLimitedDouble(IDC_BREAK,&m_new_params.m_break,0.0,100.0);
	GetLimitedDouble(IDC_ZERO,&m_new_params.m_zero,0.0,100.0);
	*m_params=m_new_params;
	return CDialog::OnOk();
}

/////////////////////////////////////////////////////////////////////////////
// CStatystykaDlg

class CStatystykaDlg : public CDialog
{
	CStatyData *m_data;
	CSelection *m_selection;

public:
	CStatystykaDlg(CSelection *selection, CStatyData *data) : CDialog() { m_selection=selection; m_data=data; }
	virtual BOOL OnInitDialog(HWND hWnd);
};


BOOL CStatystykaDlg::OnInitDialog(HWND hWnd)
{
	char napis[80];

	CDialog::OnInitDialog(hWnd);
	SetInteger(IDC_STATY_POCZ,m_selection->m_from);
	SetInteger(IDC_STATY_KON,m_selection->m_to);
	StringCbPrintf(napis,80,"%6.4f",m_data->m_min_wartosc);
	SetText(IDC_STATY_MIN,napis);
	StringCbPrintf(napis,80,"%6.4f",m_data->m_max_wartosc);
	SetText(IDC_STATY_MAX,napis);
	StringCbPrintf(napis,80,"%6.4f",m_data->m_srednia);
	SetText(IDC_STATY_SR,napis);
	StringCbPrintf(napis,80,"%6.4f",m_data->m_odchylenie_std);
	SetText(IDC_STATY_ODCH,napis);
	StringCbPrintf(napis,80,"%6.4f",m_data->m_min_fala);
	SetText(IDC_STATY_MIN_FALA,napis);
	StringCbPrintf(napis,80,"%6.4f",m_data->m_max_fala);
	SetText(IDC_STATY_MAX_FALA,napis);
	StringCbPrintf(napis,80,"%6.4f",m_data->m_srednia_fala);
	SetText(IDC_STATY_SR_FALA,napis);
	StringCbPrintf(napis,80,"%6.4f",m_data->m_min_okres);
	SetText(IDC_STATY_MIN_OKR,napis);
	StringCbPrintf(napis,80,"%6.4f",m_data->m_max_okres);
	SetText(IDC_STATY_MAX_OKR,napis);
	StringCbPrintf(napis,80,"%6.4f",m_data->m_sredni_okres);
	SetText(IDC_STATY_SR_OKR,napis);
	StringCbPrintf(napis,80,"%6.4f",1/m_data->m_max_okres);
	SetText(IDC_STATY_MIN_FREQ,napis);
	StringCbPrintf(napis,80,"%6.4f",1/m_data->m_min_okres);
	SetText(IDC_STATY_MAX_FREQ,napis);
	StringCbPrintf(napis,80,"%6.4f",1/m_data->m_sredni_okres);
	SetText(IDC_STATY_SR_FREQ,napis);
	StringCbPrintf(napis,80,"Wave (%d)",m_data->m_liczba_fal);
	SetText(IDC_STATY_LI_FAL,napis);
	StringCbPrintf(napis,80,"Period (%d)",m_data->m_liczba_okresow);
	SetText(IDC_STATY_LI_OKR,napis);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CChannelInfoDlg
//////////////////////////////////////////////////////
class CChannelInfoDlg : public CDialog
{
	COpisKanalu *m_channel;
	COpisKanalu m_new_channel;

public:
	CChannelInfoDlg(COpisKanalu *channel) : CDialog() { m_channel=channel; m_new_channel=*channel; }
	virtual BOOL OnInitDialog(HWND hWnd);
	virtual BOOL OnOk();
};

BOOL CChannelInfoDlg::OnInitDialog(HWND hWnd)
{
	char napis[80];

	CDialog::OnInitDialog(hWnd);
	SetText(IDC_NAME,m_new_channel.m_nazwa);
	SetText(IDC_UNIT,m_new_channel.m_jednostka);
	StringCbPrintf(napis,80,"%10.8f",m_new_channel.m_wspol);
	SetText(IDC_COEFF,napis);
	StringCbPrintf(napis,80,"%10.8f",m_new_channel.m_stala);
	SetText(IDC_CONST,napis);
	return TRUE;
}

BOOL CChannelInfoDlg::OnOk()
{
	GetText(IDC_NAME,m_new_channel.m_nazwa,41);
	GetText(IDC_UNIT,m_new_channel.m_jednostka,8);
	GetLimitedDouble(IDC_COEFF,&m_new_channel.m_wspol,0.0,999999.0);
	GetLimitedDouble(IDC_CONST,&m_new_channel.m_stala,-999999.0,999999.0);
	*m_channel=m_new_channel;
	return CDialog::OnOk();
}

/////////////////////////////////////////////////////////////////////////////
// CFileInfoDlg
class CFileInfoDlg : public CDialog
{
	BOOL m_byly_zmiany;
	CHeader *m_doc;
	CHeader m_new_doc;

public:
	CFileInfoDlg(CHeader *doc) : CDialog() { m_doc=doc; m_new_doc=*doc; }
	virtual BOOL OnInitDialog(HWND hWnd);
	virtual BOOL OnOk();
	virtual BOOL OnNotify(int nID, NMHDR *lpHdr);
	virtual BOOL OnCommand(int nID, int nNotify, HWND hControl);

private:
	void fill_param(int i);
	void change_settings();
	void create_columns();
};


static LV_COLUMN info_listbox_columns[4]={
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_LEFT,	300,"Name",0,0},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_LEFT,	60,"Unit",0,1},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT,	90,"Coefficient",0,2},
	{LVCF_TEXT|LVCF_FMT|LVCF_WIDTH,LVCFMT_RIGHT,	90,"Constant",0,3}};

void CFileInfoDlg::fill_param(int i)
{
	char napis[80];
	LV_ITEM lvi;
	double a,b;

	lvi.mask = LVIF_TEXT;
	lvi.iItem=i;
	lvi.iSubItem=1;
	lvi.pszText = m_new_doc.GetChannelUnit(i);   // app. maintains text
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	m_new_doc.GetCoeff(i,&a,&b);
	lvi.pszText = napis;   // app. maintains text

	lvi.iSubItem=2;
	StringCbPrintf(napis,80,"%10.8f",a);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);

	lvi.iSubItem=3;
	StringCbPrintf(napis,80,"%10.8f",b);
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
}

void CFileInfoDlg::change_settings()
{
	LV_ITEM lvi;
	int i;

	lvi.mask = LVIF_STATE;
	lvi.iSubItem=0;
	lvi.stateMask = LVIS_SELECTED;
	for (i=0; i<m_new_doc.NumberOfChannels(); i++)
	{
		lvi.iItem=i;
		SendMessage(IDC_CHANNELS,LVM_GETITEM,0,(LPARAM)&lvi);
		if (lvi.state==LVIS_SELECTED)
			break;
	}
	CChannelInfoDlg dlg(m_new_doc.GetChannel(i));
	if (dlg.Execute(m_handle, IDD_CHINFO)==IDOK)
	{
		fill_param(i);
		m_byly_zmiany=TRUE;
	}
}

void CFileInfoDlg::create_columns()
{
	LV_ITEM lvi;
	int i;
	char napis[80];

	for (i=0; i<4; ++i)
		SendMessage(IDC_CHANNELS,LVM_INSERTCOLUMN,i,(LPARAM)(info_listbox_columns+i));
	lvi.mask = LVIF_TEXT;
	lvi.pszText = napis;   // app. maintains text

	for (i=0; i<m_new_doc.NumberOfChannels(); ++i)
	{
		lvi.iItem=i;
		lvi.iSubItem=0;
		lvi.pszText=m_new_doc.GetChannelName(i);
		SendMessage(IDC_CHANNELS,LVM_INSERTITEM,0,(LPARAM)&lvi);
		fill_param(i);
	}
	lvi.mask = LVIF_STATE;
	lvi.state = LVIS_SELECTED;
	lvi.stateMask = LVIS_SELECTED;
	lvi.iItem=0;
	lvi.iSubItem=0;
	SendMessage(IDC_CHANNELS,LVM_SETITEM,0,(LPARAM)&lvi);
}



BOOL CFileInfoDlg::OnInitDialog(HWND hWnd)
{
	//int i;
	char napis[80];

	DebugOut("b1\n");
	CDialog::OnInitDialog(hWnd);
	DebugOut("b2\n");
	SendMessage(IDC_CHANNELS,LVM_SETEXTENDEDLISTVIEWSTYLE,LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES,LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES);
	DebugOut("b3\n");
	m_byly_zmiany=FALSE;
	DebugOut("b4\n");
	SetText(IDC_NAME,m_new_doc.GetName());
	DebugOut("b5\n");
	StringCbPrintf(napis,80,"%10.4f",m_new_doc.GetRate());
	DebugOut("b6\n");
	SetText(IDC_RATE,napis);
	DebugOut("b7\n");
	SetInteger(IDC_NUM_CHANNELS,m_new_doc.NumberOfChannels());
	DebugOut("b8\n");
	SetInteger(IDC_LENGTH,m_new_doc.NumberOfSamples());
	DebugOut("b9\n");
	SetText(IDC_DATETIME,m_new_doc.GetDateTime(napis,80));
	DebugOut("b10\n");
	create_columns();
	DebugOut("b11\n");
	return TRUE;
}

BOOL CFileInfoDlg::OnNotify(int nID, NMHDR *lpHdr)
{
	//int i;

	switch (nID)
	{
	case IDC_CHANNELS:
		if (lpHdr->code == NM_DBLCLK)
			change_settings();
		break;
	}
	return TRUE;
}

BOOL CFileInfoDlg::OnCommand(int nID, int /*nNotify*/, HWND /*hControl*/)
{
	switch (nID)
	{
	case IDC_EDIT :
		change_settings();
		break;
	}
	return TRUE;
}

BOOL CFileInfoDlg::OnOk()
{
	double f;

	GetText(IDC_NAME,m_new_doc.GetName(),61);
	GetLimitedDouble(IDC_RATE,&f,0.0,999999.0);
	m_new_doc.SetRate(f);
	*m_doc=m_new_doc;
	return CDialog::OnOk();
}


/****************************************************************************
    CLASS: Metody klasy CWinfiltrApp

****************************************************************************/
HWND g_statusbar=NULL;
static HWND g_main_window=NULL;
static HWND g_mdi_client=NULL;
HINSTANCE g_instance;
static HWND g_toolbar=NULL;
static CParametryGeneracji g_parametry_generacji;
static CParametryFouriera g_parametry_fouriera;
static CParametryKalmana g_parametry_kalmana;
static CParametryAproksymacji g_parametry_aproksymacji;


/////////////////////////////////
static LRESULT SendToActive(UINT msg, WPARAM wParam, LPARAM lParam)
{
	HWND hWnd;

	hWnd=(HWND)SendMessage(g_mdi_client,WM_MDIGETACTIVE,0,0);
	if (hWnd)
		return SendMessage(hWnd, msg, wParam, lParam);
	else
		return 0;
}


static HWND GetActive()
{
	return (HWND)SendMessage(g_mdi_client,WM_MDIGETACTIVE,0,0);
}


//----------------------------------
static void load_params()
{
	CRegistry Registry;

	g_parametry_generacji.n1=5;
	g_parametry_generacji.n2=5;
	g_parametry_generacji.n3=5;
	g_parametry_generacji.dt=0.02;
	g_parametry_generacji.L=8;
	g_parametry_generacji.h=0.6;
	g_parametry_generacji.etaT=2;
	g_parametry_generacji.A=0.025;
	g_parametry_generacji.xa=0.3;
	g_parametry_generacji.w1=0.231;
	g_parametry_generacji.w2=0.044;
	g_parametry_generacji.w3=0.0;
	g_parametry_generacji.model=1;
	g_parametry_generacji.fio=M_PI;
	g_parametry_generacji.cr=0.05;
	g_parametry_generacji.nw=3;
	g_parametry_generacji.m_n=2;

	g_parametry_fouriera.m_tuned=0;
	g_parametry_fouriera.m_tune_fr=100.0;
	g_parametry_fouriera.m_limited=0;
	g_parametry_fouriera.m_limit=500.0;
	g_parametry_fouriera.m_phase=0;
	g_parametry_fouriera.m_line_plot=0;

	g_parametry_kalmana.m_filter_type=3;
	g_parametry_kalmana.m_estimate_error=1;
	g_parametry_kalmana.m_observation_error=5.0;
	g_parametry_kalmana.m_constant_period=5;
	g_parametry_kalmana.m_asymptotic=1e-5;
	g_parametry_kalmana.m_use_damp=0;

	g_parametry_aproksymacji.m_use_AD=0;
	g_parametry_aproksymacji.m_use_eta=0;
	g_parametry_aproksymacji.m_epsilon=0.001;
	g_parametry_aproksymacji.m_break=0.001;
	g_parametry_aproksymacji.m_zero=1e-14;

	if (Registry.OpenForRead(HKEY_CURRENT_USER,gszWinfiltrRegisterPath)==0)
	{
		Registry.GetInteger("czas_narastania",&g_parametry_generacji.n1);
		Registry.GetInteger("czas_ustalony",&g_parametry_generacji.n2);
		Registry.GetInteger("czas_zaniku",&g_parametry_generacji.n3);
		Registry.GetDouble("okres_probkowania",&g_parametry_generacji.dt);
		Registry.GetDouble("dlugosc_fali",&g_parametry_generacji.L);
		Registry.GetDouble("wysokosc_wody",&g_parametry_generacji.h);
		Registry.GetDouble("narastanie",&g_parametry_generacji.etaT);
		Registry.GetDouble("amplituda",&g_parametry_generacji.A);
		Registry.GetDouble("wspolczynnik_xa",&g_parametry_generacji.xa);
		Registry.GetDouble("wspolczynnik_w1",&g_parametry_generacji.w1);
		Registry.GetDouble("wspolczynnik_w2",&g_parametry_generacji.w2);
		Registry.GetDouble("wspolczynnik_w3",&g_parametry_generacji.w3);
		Registry.GetDouble("wspolczynnik_fio",&g_parametry_generacji.fio);
		Registry.GetDouble("wspolczynnik_cr",&g_parametry_generacji.cr);
		Registry.GetDouble("wielkosc_grupy",&g_parametry_generacji.nw);
		Registry.GetInteger("model_generacji",&g_parametry_generacji.model);
		Registry.GetInteger("stopien_rozniczkowalnosci",&g_parametry_generacji.m_n);

		Registry.GetLimitedInteger("FourierTunned",&g_parametry_fouriera.m_tuned,0,1);
		Registry.GetLimitedDouble("FourierTune",&g_parametry_fouriera.m_tune_fr,0.0,99999.9);
		Registry.GetLimitedInteger("FourierLimited",&g_parametry_fouriera.m_limited,0,1);
		Registry.GetLimitedDouble("FourierLimit",&g_parametry_fouriera.m_limit,0.0,99999.9);
		Registry.GetLimitedInteger("FourierPlotPhase",&g_parametry_fouriera.m_phase,0,1);
		Registry.GetLimitedInteger("FourierLinePlot",&g_parametry_fouriera.m_line_plot,0,1);

		Registry.GetLimitedInteger("KalmanModel",&g_parametry_kalmana.m_filter_type,0,3);
		Registry.GetLimitedInteger("KalmanEstimateError",&g_parametry_kalmana.m_estimate_error,0,1);
		Registry.GetLimitedDouble("KalmanObservationError",&g_parametry_kalmana.m_observation_error,0.0,99.9);
		Registry.GetLimitedDouble("KalmanConstantPeriod",&g_parametry_kalmana.m_constant_period,0.0,999999.9);
		Registry.GetLimitedDouble("KalmanAsymptotic",&g_parametry_kalmana.m_asymptotic,0.0,99.9);
		Registry.GetLimitedInteger("KalmanUseDamp",&g_parametry_kalmana.m_use_damp,0,1);

		Registry.GetLimitedInteger("AproxUseAD",&g_parametry_aproksymacji.m_use_AD,0,1);
		Registry.GetLimitedInteger("AproxUseEta",&g_parametry_aproksymacji.m_use_eta,0,1);
		Registry.GetLimitedDouble("AproxEpsilon",&g_parametry_aproksymacji.m_epsilon,0.0,99.9);
		Registry.GetLimitedDouble("AproxBreak",&g_parametry_aproksymacji.m_break,0.0,99.9);
		Registry.GetLimitedDouble("AproxZero",&g_parametry_aproksymacji.m_zero,0.0,1.0);

		Registry.Close();
	}
}
//----------------------------------
static void save_params()
{
	CRegistry Registry;

	if (Registry.OpenForWrite(HKEY_CURRENT_USER, gszWinfiltrRegisterPath)==0)
	{
		Registry.SetInteger("czas_narastania",g_parametry_generacji.n1);
		Registry.SetInteger("czas_ustalony",g_parametry_generacji.n2);
		Registry.SetInteger("czas_zaniku",g_parametry_generacji.n3);
		Registry.SetDouble("okres_probkowania",g_parametry_generacji.dt);
		Registry.SetDouble("dlugosc_fali",g_parametry_generacji.L);
		Registry.SetDouble("wysokosc_wody",g_parametry_generacji.h);
		Registry.SetDouble("narastanie",g_parametry_generacji.etaT);
		Registry.SetDouble("amplituda",g_parametry_generacji.A);
		Registry.SetDouble("wspolczynnik_xa",g_parametry_generacji.xa);
		Registry.SetDouble("wspolczynnik_w1",g_parametry_generacji.w1);
		Registry.SetDouble("wspolczynnik_w2",g_parametry_generacji.w2);
		Registry.SetDouble("wspolczynnik_w3",g_parametry_generacji.w3);
		Registry.SetDouble("wspolczynnik_fio",g_parametry_generacji.fio);
		Registry.SetDouble("wspolczynnik_cr",g_parametry_generacji.cr);
		Registry.SetDouble("wielkosc_grupy",g_parametry_generacji.nw);
		Registry.SetInteger("model_generacji",g_parametry_generacji.model);
		Registry.SetInteger("stopien_rozniczkowalnosci",g_parametry_generacji.m_n);

		Registry.SetInteger("FourierTunned",g_parametry_fouriera.m_tuned);
		Registry.SetDouble("FourierTune",g_parametry_fouriera.m_tune_fr);
		Registry.SetInteger("FourierLimited",g_parametry_fouriera.m_limited);
		Registry.SetDouble("FourierLimit",g_parametry_fouriera.m_limit);
		Registry.SetInteger("FourierPlotPhase",g_parametry_fouriera.m_phase);
		Registry.SetInteger("FourierLinePlot",g_parametry_fouriera.m_line_plot);

		Registry.SetInteger("KalmanModel",g_parametry_kalmana.m_filter_type);
		Registry.SetInteger("KalmanEstimateError",g_parametry_kalmana.m_estimate_error);
		Registry.SetDouble("KalmanObservationError",g_parametry_kalmana.m_observation_error);
		Registry.SetDouble("KalmanConstantPeriod",g_parametry_kalmana.m_constant_period);
		Registry.SetDouble("KalmanAsymptotic",g_parametry_kalmana.m_asymptotic);
		Registry.SetInteger("KalmanUseDamp",g_parametry_kalmana.m_use_damp);

		Registry.SetInteger("AproxUseAD",g_parametry_aproksymacji.m_use_AD);
		Registry.SetInteger("AproxUseEta",g_parametry_aproksymacji.m_use_eta);
		Registry.SetDouble("AproxEpsilon",g_parametry_aproksymacji.m_epsilon);
		Registry.SetDouble("AproxBreak",g_parametry_aproksymacji.m_break);
		Registry.SetDouble("AproxZero",g_parametry_aproksymacji.m_zero);

		Registry.Close();
	}
}

//----------------------------------
static TBBUTTON tbButtons[] = {
	{ 0, ID_FILE_OPEN,   TBSTATE_ENABLED, TBSTYLE_BUTTON},
	{ 1, ID_FILE_SAVEAS, TBSTATE_ENABLED, TBSTYLE_BUTTON},
	{ 2, ID_FILE_INFO,   TBSTATE_ENABLED, TBSTYLE_BUTTON},
	{ 0, 0,              TBSTATE_ENABLED, TBSTYLE_SEP},
	{ 3, ID_EDIT_COPY,   TBSTATE_ENABLED, TBSTYLE_BUTTON}};

static int OnCreate(HWND hWnd, LPCREATESTRUCT /*lpCreateStruct*/)
{
	int  rnPaneEdge[3];
	RECT r;
	CLIENTCREATESTRUCT ccs;
	HMENU hMenu;

	open_file_dlg.hwndOwner = hWnd;
	open_file_dlg.hInstance = g_instance;
	save_file_dlg.hwndOwner = hWnd;
	save_file_dlg.hInstance = g_instance;

	GlobalSmallFont.Create(12,0,0,0,0,"Tahoma");

	g_toolbar=CreateToolbarEx(hWnd,
		WS_CHILD | TBSTYLE_FLAT | WS_VISIBLE | CCS_TOP,
		10,4,g_instance,IDR_MAINTOOLBAR,tbButtons,5,
		16,16,16,15,sizeof(TBBUTTON));
	g_statusbar=CreateStatusWindow(WS_CHILD|WS_VISIBLE|CCS_BOTTOM,"Ready",hWnd,11);
	rnPaneEdge[0]=400-120-140;
	rnPaneEdge[1]=400-140;
	rnPaneEdge[2]=-1;
	SendMessage(g_statusbar,SB_SETPARTS,3,(DWORD)(LPINT)(rnPaneEdge));
	SendMessage(g_statusbar,SB_SETTEXT,1,(LPARAM)(LPSTR)"x=  y=");
	SendMessage(g_statusbar,SB_SETTEXT,2,(LPARAM)(LPSTR)"block=");

	GetClientRect(hWnd,&r);
	hMenu=GetMenu(hWnd);
	ccs.hWindowMenu=GetSubMenu(hMenu,7);
	ccs.idFirstChild=1000;

	g_mdi_client=CreateWindowEx(WS_EX_CLIENTEDGE,
		"MDICLIENT",
		"" ,
		WS_CLIPCHILDREN|WS_CHILD|WS_VISIBLE,
		0, 0, r.right, r.bottom,
		g_main_window,
		0,
		g_instance,
		&ccs
		);
	if (!g_mdi_client)
		return 2;

	return 0;
}

//----------------------------------
static void OnDestroy(HWND hWnd)
{
	CRegistry Registry;
	WINDOWPLACEMENT wp;

	if (Registry.OpenForWrite(HKEY_CURRENT_USER,gszWinfiltrRegisterPath)==0)
	{
		GetWindowPlacement(hWnd,&wp);
		wp.rcNormalPosition.right-=wp.rcNormalPosition.left;
		wp.rcNormalPosition.bottom-=wp.rcNormalPosition.top;
		Registry.SetInteger("window_xpos",wp.rcNormalPosition.left);
		Registry.SetInteger("window_ypos",wp.rcNormalPosition.top);
		Registry.SetInteger("window_width",wp.rcNormalPosition.right);
		Registry.SetInteger("window_height",wp.rcNormalPosition.bottom);
		Registry.SetInteger("window_state",wp.showCmd);
		Registry.Close();
	}
}

//----------------------------------
static void OnSize(HWND hWnd, UINT uType, UINT /*uWidth*/, UINT /*uHeight*/)
{
	RECT r;
	int  rnPaneEdge[3];
	int th=0, sh=0;

	if (g_toolbar)
	{
		SendMessage(g_toolbar,WM_SIZE,uType,0);		//MAKELONG(uWidth,uHeight)
		GetWindowRect(g_toolbar,&r);
		th=r.bottom-r.top;
	}
	if (g_statusbar)
	{
		SendMessage(g_statusbar,WM_SIZE,uType,0);		//MAKELONG(uWidth,uHeight)
		GetWindowRect(g_statusbar,&r);
		sh=r.bottom-r.top;
	}
	GetClientRect(hWnd,&r);
	MoveWindow(g_mdi_client,0,th,r.right,r.bottom-th-sh,TRUE);
	rnPaneEdge[0]=r.right-160-180;
	rnPaneEdge[1]=r.right-180;
	rnPaneEdge[2]=-1;
	SendMessage(g_statusbar,SB_SETPARTS,3,(DWORD)(LPINT)(rnPaneEdge));

	//return 0;
	//return DefFrameProc(m_handle, m_MDI_client, WM_SIZE, uType, uWidth|(uHeight<<16));
}
//----------------------------------
static int OnClose(HWND /*hWnd*/)
{
	return 1;
}

//----------------------------------
static long OnCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	CMeasureWnd *lpMeasureWnd;
	CMeasure *lpMeasure;
	CGenerator *lpGenerator;
	CGeneratorWnd *lpGeneratorWnd;
	CFft *lpFft;
	CFftWnd *lpFftWnd;
	CKalman *lpKalman;
	CKalmanWnd *lpKalmanWnd;
	CAprox *lpAprox;
	CAproxWnd *lpAproxWnd;
	CSelection Selection;
	HWND hChild;
	CStatyData StatyInfo;
	int pocz,kon,k;

	switch(LOWORD(wParam))
	{
	case ID_FILE_EXIT:
		PostMessage(hWnd,WM_CLOSE,0,0L);
		break;

	case ID_APP_ABOUT:
		AboutDlg.Execute(hWnd, IDD_ABOUT) ;
		break;

	case ID_FILE_OPEN:
		if (GetOpenFileName(&open_file_dlg))
		{
			lpMeasure=new CMeasure();
			if (open_file_dlg.nFilterIndex==2)
				lpMeasure->LoadM11File(file_name);
			else
				if (open_file_dlg.nFilterIndex==3)
					lpMeasure->LoadTextFile(file_name);
				else
					if (open_file_dlg.nFilterIndex==4)
						lpMeasure->LoadWaveFile(file_name);
					else
						lpMeasure->LoadFile(file_name);
			lpMeasureWnd=new CMeasureWnd();
			lpMeasureWnd->Create(CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,g_mdi_client,0);
			lpMeasureWnd->SetDocument(lpMeasure);
			//AdjustMenu();
		}
		break;

	case ID_FILE_SAVEAS:
		hChild=GetActive();
		if (hChild)
		{
			usun_rozszerzenie();
			save_file_dlg.lpstrFilter=save_filtr_nazw;
			if (GetSaveFileName(&save_file_dlg))
			{
				lpMeasureWnd = (CMeasureWnd *)GetWindowLong(hChild,0);
				lpMeasure = lpMeasureWnd->GetDocument();
				if (save_file_dlg.nFilterIndex==2)
					lpMeasure->SaveM11File(file_name);
				else
					if (save_file_dlg.nFilterIndex==3)
						lpMeasure->SaveTextFile(file_name);
					else
						lpMeasure->SaveFile(file_name);
			}
		}
		break;

	case ID_SAVE_BLOCK:
		hChild=GetActive();
		if (hChild)
		{
			usun_rozszerzenie();
			save_file_dlg.lpstrFilter=save_filtr_nazw2;
			if (GetSaveFileName(&save_file_dlg))
			{
				lpMeasureWnd = (CMeasureWnd *)GetWindowLong(hChild,0);
				lpMeasure = lpMeasureWnd->GetDocument();
				lpMeasureWnd->GetSelection(&Selection);
				if (Selection.m_from!=Selection.m_to)
					lpMeasure->SaveBlock(Selection.m_channel,Selection.m_from,Selection.m_to,file_name);
			}
		}
		break;

	case ID_GENERATE:
		{
			CParametryGeneratoraDlg dlg(&g_parametry_generacji);
			if (dlg.Execute(hWnd)==IDOK)
			{
				lpGenerator=new CGenerator(g_parametry_generacji);
				lpGeneratorWnd=new CGeneratorWnd();
				lpGeneratorWnd->Create(CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,g_mdi_client,0);
				lpGeneratorWnd->SetDocument(lpGenerator);	//tu sa obliczenia;
			}
		}
		break;

	case ID_ANALYSE_FOURIER:
		hChild=GetActive();
		if (hChild)
		{
			SendMessage(hChild,WM_COMMAND,IDM_GETSELECTION,(LPARAM)&Selection);
			CParametryFourieraDlg dlg(&g_parametry_fouriera,&Selection);
			if (dlg.Execute(hWnd)==IDOK)
			{
				lpMeasureWnd=(CMeasureWnd *)GetWindowLong(hChild,0);
				lpFft=new CFft(lpMeasureWnd->GetDocument(),Selection,g_parametry_fouriera);
				lpFftWnd=new CFftWnd();
				lpFftWnd->Create(CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,g_mdi_client,IsZoomed(hChild) ? WS_MAXIMIZE : 0);
				lpFftWnd->SetDocument(lpFft);	//tu sa obliczenia;
			}
		}
		break;

	case ID_OPTIONS_FOURIER:
		{
			CFftOptionsDlg dlg(&g_parametry_fouriera);
			dlg.Execute(hWnd,IDD_FFTOPT);
		}
		break;

	case ID_ANALYSE_KALMAN:
		hChild=GetActive();
		if (hChild)
		{
			SendMessage(hChild,WM_COMMAND,IDM_GETSELECTION,(LPARAM)&Selection);
			lpMeasureWnd=(CMeasureWnd *)GetWindowLong(hChild,0);
			CParametryKalmanaDlg dlg(lpMeasureWnd->GetDocument()->GetSignalParams(Selection.m_channel),&g_parametry_kalmana,&Selection);
			if (dlg.Execute(hWnd)==IDOK)
			{
				lpKalman=new CKalman(lpMeasureWnd->GetDocument(),Selection,g_parametry_kalmana);
				lpKalmanWnd=new CKalmanWnd();
				lpKalmanWnd->Create(CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,g_mdi_client,IsZoomed(hChild) ? WS_MAXIMIZE : 0);
				lpKalmanWnd->SetDocument(lpKalman);	//tu sa obliczenia;
			}
		}
		break;

	case ID_OPTIONS_KALMAN:
		{
			CKalmanOptionsDlg dlg(&g_parametry_kalmana);
			dlg.Execute(hWnd,IDD_KALMANOPT);
		}
		break;

	case ID_ANALYSE_APROX:
		hChild=GetActive();
		if (hChild)
		{
			SendMessage(hChild,WM_COMMAND,IDM_GETSELECTION,(LPARAM)&Selection);
			lpMeasureWnd=(CMeasureWnd *)GetWindowLong(hChild,0);
			CParametryAproksymacjiDlg dlg(lpMeasureWnd->GetDocument()->GetSignalParams(Selection.m_channel),&g_parametry_aproksymacji,&Selection);
			if (dlg.Execute(hWnd)==IDOK)
			{
				lpAprox=new CAprox(lpMeasureWnd->GetDocument(),Selection,g_parametry_aproksymacji);
				lpAproxWnd=new CAproxWnd();
				lpAproxWnd->Create(CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,g_mdi_client,IsZoomed(hChild) ? WS_MAXIMIZE : 0);
				lpAproxWnd->SetDocument(lpAprox);	//tu sa obliczenia;
			}
		}
		break;

	case ID_OPTIONS_APROX:
		{
			CAproxOptionsDlg dlg(&g_parametry_aproksymacji);
			dlg.Execute(hWnd,IDD_APROXOPT);
		}
		break;

	case ID_ANALYSE_STATISTIC:
		hChild=GetActive();
		if (hChild)
		{
			SendMessage(hChild,WM_COMMAND,IDM_GETSELECTION,(LPARAM)&Selection);
			if (Selection.m_from-Selection.m_to)
			{
				pocz=min(Selection.m_from, Selection.m_to);
				kon=max(Selection.m_from, Selection.m_to);
				k=Selection.m_channel;
				SendMessage(hChild,WM_COMMAND,IDM_GETDOCUMENT,(LPARAM)&lpMeasure);
				StatyInfo.m_channels=lpMeasure->NumberOfChannels();
				StatyInfo.m_samples=kon-pocz;
				StatyInfo.m_rate=lpMeasure->GetRate();
				StatyInfo.m_probki = lpMeasure->Buffer();
				StatyInfo.m_probki += (k + pocz*StatyInfo.m_channels)*2;
				statystyka(&StatyInfo,OBL_ALL);
				CStatystykaDlg dlg(&Selection, &StatyInfo);
				dlg.Execute(hWnd,IDD_STATY);
			}
		}
		break;

	case ID_FILE_INFO:
		DebugOut("a1\n");
		hChild=GetActive();
		DebugOut("a2\n");
		if (hChild)
		{
			lpMeasure=0;
		DebugOut("a3\n");
			SendMessage(hChild,WM_COMMAND,IDM_GETDOCUMENT,(LPARAM)&lpMeasure);
		DebugOut1("a4 %d\n",lpMeasure);
			CFileInfoDlg dlg(lpMeasure);
		DebugOut("a5\n");
			dlg.Execute(hWnd,IDD_INFO);
		DebugOut("a6\n");
		}
		break;

	case ID_HELP_CONTENTS:
		{
			char napis[512];
			int i;
			GetModuleFileName(g_instance,napis,512);
			i=strlen(napis)-1;
			while (napis[i]!='\\')
			{
				napis[i]='\0';
				--i;
			}
			StringCbPrintf(napis,512,"WINFI32.HLP");
			WinHelp(hWnd,napis,HELP_CONTENTS,0);
		}
		break;

	case ID_INFO:
	case ID_EDIT_COPY:
	case ID_BLOCK_INFO:
	case ID_BLOCK_CLEAR:
	case ID_ANALYSE_RECALC:
	case ID_ZOOM_X_IN:
	case ID_ZOOM_X_OUT:
	case ID_ZOOM_Y_IN:
	case ID_ZOOM_Y_OUT:
	case ID_ADD_SAMPLE:
	case ID_SUB_SAMPLE:
	case ID_RANGE_IN:
	case ID_RANGE_OUT:
	case ID_PLOT_SIGNAL:
	case ID_PLOT_ENVELOPE:
	case ID_PLOT_COSIGNAL:
	case ID_PLOT_ALL:
	case ID_PLOT_MODULE:
	case ID_BEGIN_BLOCK:
	case ID_END_BLOCK:
	case ID_SELECT_ALL:
	case ID_BLOCK_SET:
	case ID_ADD_COMPONENT:
		return SendToActive(WM_COMMAND, wParam, lParam);

	default:
		return DefFrameProc(hWnd,g_mdi_client,WM_COMMAND,wParam,lParam);
	}
	return 0;
}

////////////////////////////////////////
/*static void GetSelection(int &pocz, int &kon, int &ch)
{
	CSelection Selection;

	HWND hChild=GetActive();
	if (hChild)
	{
		SendMessage(hChild,WM_COMMAND,IDM_GETSELECTION,(LPARAM)&Selection);
		pocz=Selection.m_from;
		kon=Selection.m_to;
		ch=Selection.m_channel;
	}
	else
	{
		pocz=0;
		kon=0;
		ch=0;
	}
}*/

static void OnInitMenuPopup(HWND /*hWnd*/, HMENU hMenu, UINT /*uPos*/, BOOL /*fSystemMenu*/)
{
	MENUITEMINFO mii;
	CSelection Selection={0,0,0};

	mii.cbSize=sizeof(MENUITEMINFO);
	mii.fMask=MIIM_STATE;
	HWND hChild=GetActive();
	if (hChild) SendMessage(hChild,WM_COMMAND,IDM_GETSELECTION,(LPARAM)&Selection);
	//HMENU hMainMenu=GetMenu(hWnd);
	switch (GetMenuItemID(hMenu,0))
	{
	case ID_FILE_OPEN:
		if (hChild)
			mii.fState=MFS_ENABLED;
		else
			mii.fState=MFS_GRAYED;
		SetMenuItemInfo(hMenu,ID_FILE_INFO,FALSE,&mii);
		SetMenuItemInfo(hMenu,ID_FILE_SAVEAS,FALSE,&mii);
		if (Selection.m_from!=Selection.m_to)
			mii.fState=MFS_ENABLED;
		else
			mii.fState=MFS_GRAYED;
		SetMenuItemInfo(hMenu,ID_SAVE_BLOCK,FALSE,&mii);
		break;

	case ID_EDIT_COPY:
		if (Selection.m_from!=Selection.m_to)
			mii.fState=MFS_ENABLED;
		else
			mii.fState=MFS_GRAYED;
		SetMenuItemInfo(hMenu,ID_EDIT_COPY,FALSE,&mii);
		SetMenuItemInfo(hMenu,ID_BLOCK_INFO,FALSE,&mii);
		SetMenuItemInfo(hMenu,ID_BLOCK_CLEAR,FALSE,&mii);
		break;

	case ID_PLOT_SIGNAL:
		mii.fState=hChild ? MFS_ENABLED : MFS_GRAYED;
		SetMenuItemInfo(hMenu,ID_PLOT_SIGNAL,FALSE,&mii);
		SetMenuItemInfo(hMenu,ID_PLOT_ENVELOPE,FALSE,&mii);
		SetMenuItemInfo(hMenu,ID_PLOT_COSIGNAL,FALSE,&mii);
		SetMenuItemInfo(hMenu,ID_PLOT_ALL,FALSE,&mii);
		SetMenuItemInfo(hMenu,ID_PLOT_MODULE,FALSE,&mii);
		if (hChild)
		{
			int plot_style;
			SendMessage(hChild,WM_COMMAND,IDM_GETSTATE,(LPARAM)&plot_style);
			switch (plot_style)
			{
			case PLOTSTYLE_SIGNAL: plot_style=ID_PLOT_SIGNAL; break;
			case PLOTSTYLE_SIGENV: plot_style=ID_PLOT_ENVELOPE; break;
			case PLOTSTYLE_BOTH: plot_style=ID_PLOT_COSIGNAL; break;
			case PLOTSTYLE_BOTHENV: plot_style=ID_PLOT_ALL; break;
			case PLOTSTYLE_MODULE: plot_style=ID_PLOT_MODULE; break;
			}
			CheckMenuRadioItem(hMenu,ID_PLOT_SIGNAL,ID_PLOT_MODULE,plot_style,MF_BYCOMMAND);
		}
		break;
	}
}

////////////////////////////////////////////////////////////////////////
static LRESULT FAR PASCAL FrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_CREATE:
		g_main_window=hWnd;
		return OnCreate(hWnd, (CREATESTRUCT *)lParam);

	case WM_CLOSE:
		if (OnClose(hWnd))
			DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		OnDestroy(hWnd);
		PostQuitMessage(0);
		break;

	case WM_GETMINMAXINFO:
		((MINMAXINFO *)lParam)->ptMinTrackSize.x = 100;
		((MINMAXINFO *)lParam)->ptMinTrackSize.y = 100;
		break;

	case WM_SIZE:
		OnSize(hWnd, wParam, LOWORD(lParam), HIWORD(lParam));
		break;
		//return DefFrameProc(hWnd, wnd->m_MDI_client, uMsg, wParam, lParam);

	case WM_COMMAND:
		return OnCommand(hWnd, wParam,lParam);

	case WM_INITMENUPOPUP:
		OnInitMenuPopup(hWnd, (HMENU)wParam, LOWORD(lParam),HIWORD(lParam));
		break;

	default:
		return DefFrameProc(hWnd, g_mdi_client, uMsg, wParam, lParam);
	}

	return 0;
}

/***************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
****************************************************************************/
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE /*hPrevInstance*/, LPSTR lpCmdLine, int nCmdShow)
{
	INITCOMMONCONTROLSEX cc_class;

	cc_class.dwSize=sizeof(INITCOMMONCONTROLSEX);
	cc_class.dwICC=ICC_BAR_CLASSES|ICC_LISTVIEW_CLASSES;
	InitCommonControlsEx(&cc_class);

	WNDCLASSEX  wc;

	g_instance = hInstance;
	load_params();

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)FrameWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 4;                      /* 1 pointer extra data.           */
	wc.hInstance = g_instance;
	wc.hIcon = LoadIcon(g_instance, MAKEINTRESOURCE(IDI_MAINICON));
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = NULL;			//(HBRUSH)(COLOR_WINDOW+1);
	wc.lpszMenuName =  MAKEINTRESOURCE(ID_MAINMENU);
	wc.lpszClassName = "WinfiltrClass";
	wc.hIconSm = NULL;
	if (!RegisterClassEx(&wc))
		return FALSE;

	wc.lpfnWndProc = (WNDPROC)WndProc;
	wc.hIcon = NULL;
	wc.lpszMenuName =  NULL;                      /* Name of menu resource in .RC file.  */
	wc.hCursor = NULL;
	wc.lpszClassName = "MiwilDisplayClass";	  /* Name used in call to CreateWindow. */
	if (!RegisterClassEx(&wc))
		return FALSE;

	wc.lpfnWndProc = (WNDPROC)MDIChildWndProc;
	wc.hIcon = LoadIcon(g_instance, MAKEINTRESOURCE(IDI_MEASURE));
	wc.lpszClassName = "MiwilMeasureClass";	  /* Name used in call to CreateWindow. */
	if (!RegisterClassEx(&wc))
		return FALSE;

	wc.hIcon = LoadIcon(g_instance, MAKEINTRESOURCE(IDI_FOURIER));
	wc.lpszClassName = "MiwilFourierClass";	  /* Name used in call to CreateWindow. */
	if (!RegisterClassEx(&wc))
		return FALSE;

	wc.hIcon = LoadIcon(g_instance, MAKEINTRESOURCE(IDI_KALMAN));
	wc.lpszClassName = "MiwilKalmanClass";	  /* Name used in call to CreateWindow. */
	if (!RegisterClassEx(&wc))
		return FALSE;

	wc.hIcon = LoadIcon(g_instance, MAKEINTRESOURCE(IDI_GENER));
	wc.lpszClassName = "MiwilGeneratorClass";	  /* Name used in call to CreateWindow. */
	if (!RegisterClassEx(&wc))
		return FALSE;


	WINDOWPLACEMENT wp;
	int x=CW_USEDEFAULT,y=CW_USEDEFAULT,w=CW_USEDEFAULT,h=CW_USEDEFAULT,state=nCmdShow;
	CRegistry Registry;

	// Save the instance handle in static variable, which will be used in
	// many subsequence calls from this application to Windows.

	// Create a main window for this application instance.
	if (Registry.OpenForRead(HKEY_CURRENT_USER,gszWinfiltrRegisterPath)==0)
	{
		Registry.GetInteger("window_state",&state);
		Registry.GetInteger("window_xpos",&x);
		Registry.GetInteger("window_ypos",&y);
		Registry.GetInteger("window_width",&w);
		Registry.GetInteger("window_height",&h);
		Registry.Close();
	}

	DebugOut("Start progrmu\n");
	CreateWindowEx(WS_EX_CONTROLPARENT|WS_EX_WINDOWEDGE,
		"WinfiltrClass",
		"Kalman for Windows ver. 3.2" ,
		WS_OVERLAPPEDWINDOW|WS_CLIPCHILDREN,
		x, y, w, h,
		NULL,
		0,
		g_instance,
		NULL
		);

	if (!g_main_window)
		return (FALSE);

	wp.length=sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(g_main_window,&wp);
	wp.rcNormalPosition.left=x;
	wp.rcNormalPosition.top=y;
	wp.rcNormalPosition.right=x+w;
	wp.rcNormalPosition.bottom=y+h;
	wp.showCmd=state;
	SetWindowPlacement(g_main_window,&wp);

	// Make the window visible; update its client area; and return "success"
	ShowWindow(g_main_window, state);
	UpdateWindow(g_main_window);

	if (*lpCmdLine)
	{
		CMeasure *lpMeasure=new CMeasure();
		DebugOut1("otwieram '%s'\n",lpCmdLine);
		if (lpMeasure->LoadFile(lpCmdLine))
		{
			CMeasureWnd *lpMeasureWnd=new CMeasureWnd();
			lpMeasureWnd->Create(CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,g_mdi_client,0);
			lpMeasureWnd->SetDocument(lpMeasure);
		}
		else
			delete lpMeasure;
	}

	// Acquire and dispatch messages until a WM_QUIT message is received.
	MSG msg;
	HACCEL hAccTable;

	hAccTable = LoadAccelerators(g_instance, MAKEINTRESOURCE(IDR_MAINACCELERATOR));

	while (GetMessage(&msg, 0, 0, 0))
		if (!TranslateMDISysAccel(g_mdi_client, &msg))
			if (!TranslateAccelerator(g_main_window, hAccTable, &msg))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}

	save_params();
	return msg.wParam;
}

