; Winfi32.nsi
;
; This script is based on example2.nsi, but it remember the directory, 
; has uninstall support and (optionally) installs start menu shortcuts.
;
; It will install Winfi32.exe into a directory that the user selects,

;--------------------------------
!include "FileFunc.nsh"
!insertmacro RefreshShellIcons
!insertmacro un.RefreshShellIcons

XPStyle on

; The name of the installer
Name "Winfi32"

; The file to write
OutFile "Winfi32Setup.exe"

; The default installation directory
InstallDir $PROGRAMFILES\MIWIL\Winfi32

; Registry key to check for directory (so if you install again, it will 
; overwrite the old one automatically)
InstallDirRegKey HKLM "Software\MIWIL_Winfi32" "Install_Dir"

LoadLanguageFile "${NSISDIR}\Contrib\Language files\Polish.nlf"
LicenseLangString myLicenseData ${LANG_POLISH} "license.txt"
LicenseData $(myLicenseData)

;--------------------------------

; Pages

Page license
Page components
Page directory
Page instfiles

UninstPage uninstConfirm
UninstPage instfiles

;--------------------------------

; The stuff to install
Section "Winfi32 (required)"

  SectionIn RO
  
  ; Set output path to the installation directory.
  SetOutPath $INSTDIR
  
  ; Put file there
  File "Winfi32.exe"
  
  ; Write the installation path into the registry
  WriteRegStr HKLM SOFTWARE\MIWIL_Winfi32 "Install_Dir" "$INSTDIR"
  
  ; Write the uninstall keys for Windows
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Winfi32" "DisplayName" "Winfi32"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Winfi32" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Winfi32" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Winfi32" "NoRepair" 1
  WriteUninstaller "uninstall.exe"
  
SectionEnd

; Optional section (can be disabled by the user)
Section "Start Menu Shortcuts"

  CreateDirectory "$SMPROGRAMS\Winfi32"
  CreateShortCut "$SMPROGRAMS\Winfi32\Uninstall.lnk" "$INSTDIR\uninstall.exe" "" "$INSTDIR\uninstall.exe" 0
  CreateShortCut "$SMPROGRAMS\Winfi32\Winfi32.lnk" "$INSTDIR\Winfi32.exe" "" "$INSTDIR\Winfi32.exe" 0
  
SectionEnd

Section "File Associations"
  WriteRegStr HKCR ".int" "" "Winfi32.file"
  WriteRegStr HKCR "Winfi32.file" "" "Winfi32 data file"
  WriteRegStr HKCR "Winfi32.file\DefaultIcon" "" "$INSTDIR\Winfi32.exe,0"
  WriteRegStr HKCR "Winfi32.file\shell\open\command" "" "$INSTDIR\Winfi32.exe %1"
  ${RefreshShellIcons}
SectionEnd

;--------------------------------

; Uninstaller

Section "Uninstall"
  
  ; Remove registry keys
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Winfi32"
  DeleteRegKey HKLM SOFTWARE\MIWIL_Winfi32
  DeleteRegKey HKCR ".int"
  DeleteRegKey HKCR "Winfi32.file"
  ${un.RefreshShellIcons}

  ; Remove files and uninstaller
  Delete $INSTDIR\Winfi32.exe
  Delete $INSTDIR\uninstall.exe

  ; Remove shortcuts, if any
  Delete "$SMPROGRAMS\Winfi32\*.*"

  ; Remove directories used
  RMDir "$SMPROGRAMS\Winfi32"
  RMDir "$INSTDIR"

SectionEnd
